/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.e4;

import java.util.ArrayList;
import java.util.Collection;

import javax.inject.Inject;

import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.operation.MoveDefinedTermOperation;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.EditViewerDropAdapter;

/**
 *
 * @author pplitzner
 * @since Aug 22, 2017
 *
 */
public class DefinedTermDropAdapterE4 extends EditViewerDropAdapter {


    public static final String TERM_TYPE_ERROR_TITLE = Messages.DefinedTermDropAdapterE4_TERM_TYPE_ERROR_TITLE;
    public static final String TERM_TYPE_ERROR_MESSAGE = Messages.DefinedTermDropAdapterE4_TERM_TYPE_ERROR_MESSAGE;

    public static final String MOVE_FAILED = Messages.DefinedTermDropAdapterE4_MOVE_FAILED;

    private static final String MOVE_FAILED_MESSAGE = Messages.DefinedTermDropAdapterE4_MOVE_FAILED_MESSAGE;

	private final DefinedTermEditorE4 editor;

	@Inject
    private UISynchronize sync;

	@Inject
	protected DefinedTermDropAdapterE4(DefinedTermEditorE4 editor) {
		super(editor.getViewer());
		this.editor = editor;
	}

	@Override
	public boolean performDrop(Object data) {
	    if(StoreUtil.promptCheckIsDirty(editor) || data == null){
	        return false;
	    }
		AbstractTermDto target = (AbstractTermDto) getCurrentTarget();//can be vocab
		int currentLocation = getCurrentLocation();
		IStructuredSelection droppedTerms = (IStructuredSelection) data;
		Collection<TermDto> sourceTerms = new ArrayList<>(); //Arrays.asList(droppedElements)

		for (Object droppedTerm : droppedTerms.toList()) {
		    TermDto term = (TermDto) droppedTerm;
			sourceTerms.add(term);
		}
        // cannot drop a term onto itself or at its own children
        for (Object droppedObject : sourceTerms) {
            if(droppedObject==null){
                MessagingUtils.warningDialog(MOVE_FAILED, this.getClass(),
                        Messages.DefinedTermDropAdapterE4_MOVE_FAILED_SAVE_MESSAGE);
                return false;
            }
            if (droppedObject.equals(target)) {
                MessagingUtils.warningDialog(MOVE_FAILED, this.getClass(),
                        MOVE_FAILED_MESSAGE);
                return false;
            }
            if(target instanceof TermDto){
                TermDto parentTerm = (TermDto)target;
                //check term type compatibility
                if(droppedObject instanceof TermDto && !((TermDto) droppedObject).getTermType().equals(parentTerm.getTermType())){
                    MessagingUtils.warningDialog(TERM_TYPE_ERROR_TITLE, this, TERM_TYPE_ERROR_MESSAGE);
                    return false;
                }

                TermDto partOfDto = parentTerm.getPartOfDto();
                if(partOfDto!=null && partOfDto.equals(droppedObject)){
                    MessagingUtils.warningDialog(MOVE_FAILED, this.getClass(),
                            MOVE_FAILED_MESSAGE);
                    return false;
                }
                TermDto kindOfDto = parentTerm.getKindOfDto();
                if(kindOfDto!=null && kindOfDto.equals(droppedObject)){
                    MessagingUtils.warningDialog(MOVE_FAILED, this.getClass(),
                            MOVE_FAILED_MESSAGE);
                    return false;
                }
            }
            else if(target instanceof TermVocabularyDto){
                TermVocabularyDto vocDto = (TermVocabularyDto)target;
                //check term type compatibility
                if(droppedObject instanceof TermDto && !((TermDto) droppedObject).getTermType().equals(vocDto.getTermType())){
                    MessagingUtils.warningDialog(TERM_TYPE_ERROR_TITLE, this, TERM_TYPE_ERROR_MESSAGE);
                    return false;
                }
            }
        }
		AbstractPostOperation operation = new MoveDefinedTermOperation(Messages.DefinedTermDropAdapterE4_MOVE_DESCRIPTIONS,
		        StoreUtil.getUndoContext(),
		        target,
		        sourceTerms,
		        editor,
		        currentLocation);
		AbstractUtility.executeOperation(operation, sync);

		return true;
	}

	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferType) {
	    boolean valid = LocalSelectionTransfer.getTransfer().isSupportedType(transferType)
                && target instanceof AbstractTermDto;
	    if(target instanceof TermVocabularyDto && getCurrentLocation()!=ViewerDropAdapter.LOCATION_ON ){
	        valid = false;
	    }
	    if (target instanceof TermDto && ((TermDto)target).getOrderIndex() == null && getCurrentLocation() == ViewerDropAdapter.LOCATION_AFTER){
	    	valid = false;
	    }
        return valid;
	}

}
