/*******************************************************************************
 * Copyright (c) 2007, 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package eu.etaxonomy.taxeditor.handler.update;

import java.net.URI;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.equinox.p2.core.IProvisioningAgent;
import org.eclipse.equinox.p2.engine.IProfile;
import org.eclipse.equinox.p2.engine.IProfileRegistry;
import org.eclipse.equinox.p2.operations.ProvisioningJob;
import org.eclipse.equinox.p2.operations.ProvisioningSession;
import org.eclipse.equinox.p2.operations.UpdateOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;


/**
 * UpdateHandler invokes the check for updates UI
 *
 * @since 3.4
 */
public class UpdateHandler {

    private Logger logger = Logger.getLogger(getClass());

    private UpdateOperation operation;

    @Execute
    public void execute(final IProvisioningAgent agent, final Shell shell, final UISynchronize sync,
            final IWorkbench workbench) {

        Job checkUpdateJob = new Job(Messages.UpdateHandler_CHECK_UPDATE_JOB) {
            @Override
            protected IStatus run(final IProgressMonitor monitor) {

                return checkForUpdates(agent, shell, sync, monitor);
            }
        };

        checkUpdateJob.schedule();

        checkUpdateJob.addJobChangeListener(new JobChangeAdapter() {
            @Override
            public void done(IJobChangeEvent event) {
                if (event.getResult().isOK()) {
                    sync.syncExec(new Runnable() {

                        @Override
                        public void run() {
                            if(MessageDialog.openConfirm(shell, Messages.UpdateHandler_UPDATES_FOUND_TITLE, Messages.UpdateHandler_UPDATES_FOUND_MESSAGE)){
                                Job installUpdateJob = new Job(Messages.UpdateHandler_INSTALL_JOB) {
                                    @Override
                                    protected IStatus run(final IProgressMonitor monitor) {
                                        // run installation
                                        ProvisioningJob provisioningJob = operation.getProvisioningJob(monitor);

                                        // updates cannot run from within Eclipse IDE!!!
                                        if (provisioningJob == null) {
                                            logger.error("Trying to update from the Eclipse IDE? This won't work!"); //$NON-NLS-1$
                                            return Status.CANCEL_STATUS;
                                        }
                                        configureProvisioningJob(provisioningJob, shell, sync, workbench);
                                        provisioningJob.schedule();
                                        return Status.OK_STATUS;
                                    }
                                };
                                installUpdateJob.schedule();
                            }
                        }
                    });
                }
            }
        });
    }

	private IStatus checkForUpdates(final IProvisioningAgent agent, final Shell shell, final UISynchronize sync,
            IProgressMonitor monitor) {

        // configure update operation
        final ProvisioningSession session = new ProvisioningSession(agent);
        long startTime = System.currentTimeMillis();
        logger.info("Try to get updates" + startTime);
        try{
	        operation = new UpdateOperation(session);
	        IProfileRegistry registry = (IProfileRegistry) agent.getService(IProfileRegistry.SERVICE_NAME);
	        
	        String profileID = operation.getProfileId();
	        IProfile profile = registry.getProfile(profileID);
	        if (profile == null){
	        	logger.info("There is no profile for profileID: " + profileID);
	        }
	        configureUpdate(operation);
	        // check for updates, this causes I/O
	        final IStatus status = operation.resolveModal(monitor);
	        // failed to find updates (inform user and exit)
	        long actualTime = System.currentTimeMillis();
	        if (status.getCode() == UpdateOperation.STATUS_NOTHING_TO_UPDATE) {
	            showMessage(shell, sync);
	            logger.info("update resolved, but no updates available " + (actualTime - startTime) + "Status: " + status.toString());
	            return Status.CANCEL_STATUS;
	        }
	        
	        logger.info("update resolved " + (actualTime - startTime) + "Status: " + status.toString());
        }catch(Exception e){
        	logger.warn( e.getStackTrace().toString());
        	return Status.CANCEL_STATUS;
        }

        return Status.OK_STATUS;
    }

    private void configureProvisioningJob(ProvisioningJob provisioningJob, final Shell shell, final UISynchronize sync,
            final IWorkbench workbench) {

        // register a job change listener to track
        // installation progress and notify user upon success
        provisioningJob.addJobChangeListener(new JobChangeAdapter() {
            @Override
            public void done(IJobChangeEvent event) {
                if (event.getResult().isOK()) {
                    sync.syncExec(new Runnable() {

                        @Override
                        public void run() {
                            boolean restart = MessageDialog.openQuestion(shell, Messages.UpdateHandler_UPDATE_INSTALLED_TITLE,
                                    Messages.UpdateHandler_UPDATE_INSTALLED_TITLE_MESSAGE);
                            if (restart) {
                                workbench.restart();
                            }
                        }
                    });
                }
                super.done(event);
            }
        });

    }

    private void showMessage(final Shell parent, final UISynchronize sync) {
        sync.syncExec(()->
        MessageDialog.openWarning(parent, Messages.UpdateHandler_NO_UPDATE_TITLE,
                        Messages.UpdateHandler_NO_UPDATE_MESSAGE)
        );
    }

    private UpdateOperation configureUpdate(final UpdateOperation operation) {
        // create uri and check for validity
        URI uri = null;
        uri = P2Util.getP2UpdateRepository();

        // set location of artifact and metadata repo
        operation.getProvisioningContext().setArtifactRepositories(new URI[] { uri });
        operation.getProvisioningContext().setMetadataRepositories(new URI[] { uri });
        
        //if local plugin is installed, then check for updates
        //operation.getProvisioningContext().setExtraInstallableUnits(extraIUs);
        return operation;
    }
}
