/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io.e4.in;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.inject.Inject;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.io.common.IIoConfigurator;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 *
 * @author pplitzner
 * @since Oct 5, 2017
 *
 */
public class GenericConfiguratorWizardPageE4 extends WizardPage {

    private IIoConfigurator configurator;
    private ResourceBundle resourceBundle;
    private List<String> ignoreMethods;

    @Inject
    public GenericConfiguratorWizardPageE4() {
        super("Configuration");
    }

    private void init(String title, String description, IIoConfigurator configurator, List<String> ignoreMethods){
        this.setTitle(title);
        this.setDescription(description);
        this.configurator = configurator;
        this.ignoreMethods = ignoreMethods;
        resourceBundle = getResourceBundle(configurator);
    }

    private ResourceBundle getResourceBundle(IIoConfigurator configurator){
        try{
            if (configurator != null){
                return ResourceBundle.getBundle(configurator.getClass().getName());
            }else{
                return ResourceBundle.getBundle("Configuration");
            }
        }catch(MissingResourceException e){
            return null;
        }
    }

    public void initImport(IIoConfigurator configurator, List<String> ignoreMethods, String label) {
        init("Import Configuration", label, configurator, ignoreMethods);
    }

    public void initExport(IIoConfigurator configurator, List<String> ignoreMethods) {
        init("Export Configuration", "Configure the export mechanism.", configurator, ignoreMethods);
    }

    /** {@inheritDoc} */
    @Override
    public void createControl(Composite parent) {

        // TODO wrap this composite in a scrolled composite
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        GridLayout gridLayout = new GridLayout();
        composite.setLayout(gridLayout);

        List<Method> methods = getConfiguratorsBooleanSetMethods(configurator);
        Collections.sort(methods, new Comparator<Method>() {
            @Override
            public int compare(Method o1, Method o2) {
                if (o1.equals(o2)){
                    return 0;
                }
                if(o1.getName()==null && o2.getName()!=null){
                    return -1;
                }
                if(o1.getName()!=null && o2.getName()==null){
                    return 1;
                }
                if(o1.getName()==null && o2.getName()==null){
                    return o1.toString().compareTo(o1.toString());
                }
                int result = o1.getName().compareTo(o2.getName());
                if (result == 0){
                    return o1.toString().compareTo(o1.toString());
                }
                return result;
            }
        });

	        for (Method method : methods) {
	        	if (ignoreMethods != null){
		            if (!ignoreMethods.contains( method.getName())) {
		                createCheckbox(composite, method, configurator);
		            }
	        	}else{
	        		createCheckbox(composite, method, configurator);
	        	}

	        }

        setControl(composite);
    }

    private void createCheckbox(Composite parent, Method method,
            final IIoConfigurator configurator) {

        final String methodName = method.getName();
        final Button checkBox = new Button(parent, SWT.CHECK);

       // if(configurator.getClass().equals(Abcd206ImportConfigurator.class)){
        checkBox.setText(getLabel(splitMethodName(methodName)));
        // retrieve the default values and set the checkbox accordingly
        boolean defaultSelection = executeBooleanGetMethod(configurator, "is"
                + methodName.substring(3));
        checkBox.setSelection(defaultSelection);
        checkBox.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                executeBooleanSetMethod(configurator, methodName,
                        checkBox.getSelection());
            }

        });

    }

    private String getLabel(String methodName){

        if(resourceBundle == null){
            return methodName;
        }

        try{
            return resourceBundle.getString(methodName);
        }catch(MissingResourceException e){
            return methodName;
        }
    }

    private boolean executeBooleanGetMethod(IIoConfigurator configurator,
            String methodName) {

        Class<? extends IIoConfigurator> configuratorClass = configurator
                .getClass();

        boolean result = false;

        Method[] methods = configuratorClass.getMethods();

        for (Method method : methods) {
            if (!method.getName().equals(methodName)) {
                continue;
            }

            try {
                Object returnType = method.invoke(configurator, null);
                if (returnType.getClass().equals(Boolean.class)) {
                    result = ((Boolean) returnType).booleanValue();
                }

                break;
            } catch (Exception e) {
                MessagingUtils.warn(this.getClass(), "Could not invoke method");
            }
        }

        return result;
    }

    private void executeBooleanSetMethod(IIoConfigurator configurator,
            String methodName, boolean selected) {

        Class<? extends IIoConfigurator> configuratorClass = configurator
                .getClass();

        Method[] methods = configuratorClass.getMethods();

        for (Method method : methods) {
            if (!method.getName().equals(methodName)) {
                continue;
            }

            try {
                method.invoke(configurator, selected);

                break;
            } catch (Exception e) {
                MessagingUtils.warn(this.getClass(), "Could not invoke method");
            }
        }
    }

    private List<Method> getConfiguratorsBooleanSetMethods(
            IIoConfigurator configurator) {
        List<Method> booleanMethods = new ArrayList<Method>();

        Class<? extends IIoConfigurator> configuratorClass = configurator
                .getClass();

        Method[] allMethods = configuratorClass.getMethods();

        for (Method method : allMethods) {
            if (method.getName().startsWith("set")) {

                Class<?>[] typeList = method.getParameterTypes();
                //this is irrelevant for methods without boolean parameters
                if (typeList.length > 1 && typeList[0].getName().equals("boolean")) {
                    throw new IllegalStateException(
                            "Found a setter with parameter count > 1");
                }

                if (typeList[0].getName().equals("boolean")) {
                    booleanMethods.add(method);
                }
            }
        }

        return booleanMethods;
    }

    private String splitMethodName(String s) {
        s = s.replace("set", "");
        return s.replaceAll(
           String.format("%s|%s|%s",
              "(?<=[A-Z])(?=[A-Z][a-z])",
              "(?<=[^A-Z])(?=[A-Z])",
              "(?<=[A-Za-z])(?=[^A-Za-z])"
           ),
           " "
        );
     }
}
