/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.model;

import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.format.DescriptionElementFormatter;
import eu.etaxonomy.cdm.format.common.ExtendedTimePeriodFormatter;
import eu.etaxonomy.cdm.format.description.CategoricalDataFormatter;
import eu.etaxonomy.cdm.format.description.QuantitativeDataFormatter;
import eu.etaxonomy.cdm.model.common.ExtendedTimePeriod;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTerm;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TemporalData;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * DescriptionHelper class.
 *
 * @author p.ciardelli
 * @author n.hoffmann
 */
public class DescriptionHelper {

    public static final String NO_LABEL_STRING = "[no label]";

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class.
	 *
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param language a {@link eu.etaxonomy.cdm.model.common.Language} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getCache(DescriptionElementBase element, Language language) {
	    return DescriptionElementFormatter.format(element, language);
	}

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 *
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getCache(DescriptionElementBase element) {
		return getCache(element, CdmStore.getDefaultLanguage());
	}

	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class.
	 *
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param value a {@link java.lang.String} object.
	 * @param language a {@link eu.etaxonomy.cdm.model.common.Language} object.
	 */
	public static void setCache(DescriptionElementBase element,
			String value, Language language) {
		if (element instanceof TextData) {
			((TextData) element).putText(language, value);
			return;
		}else if (element instanceof CommonTaxonName) {
			((CommonTaxonName) element).setName(value);
			return;
		}else if (element instanceof TaxonInteraction) {

		}else if(element instanceof Distribution){
			MessagingUtils.warn(DescriptionHelper.class, "trying to set cache on distribution, don't know what to do at the moment.");
			return;
		}else{
			MessagingUtils.warn(DescriptionHelper.class, "No matching subclass found for DescriptionElementBase object, 'cache' not set.");
		}
	}

//	public static String getObjectDescription(Object o){
//	    if (o instanceof CdmBase){
//	        return ReferencingObjectFormatter.format((CdmBase)o, CdmStore.getDefaultLanguage());
//	    }else{
//	        return o.toString();
//	    }
//	}

	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 *
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param value a {@link java.lang.String} object.
	 */
	public static void setCache(DescriptionElementBase element,
			String value) {
		setCache(element, value, CdmStore.getDefaultLanguage());
	}

//	public static String getObjectClassname(Object element) {
//		if (element instanceof CdmBase){
//		    return ((CdmBase) element).getUserFriendlyTypeName();
//		}else{
//		    return element.getClass().getSimpleName();
//		}
//	}

	public static String getFeatureNodeContainerText(FeatureNodeContainer element) {
		String result = null;
		if(element.getFeatureNode() != null && element.getFeatureNode().getTerm() != null){
			
			result = element.getFeatureNode().getTerm().getPreferredRepresentation(CdmStore.getDefaultLanguage()).getLabel();
			
		} else{
			return "No label set";
		}
		if (result == null){
			result = element.getFeatureNode().getTerm().getLabel();
		}
		return result;
	}

	public static String getQuantitativeDataText(QuantitativeData element) {
	    QuantitativeDataFormatter formatter = new QuantitativeDataFormatter(null, null);
	    String label = formatter.format(element, getLanguageList());
		return label;
	}

	public static String getCategoricalDataText(CategoricalData element) {
	    CategoricalDataFormatter formatter = new CategoricalDataFormatter(null, null);
		return formatter.format(element, getLanguageList());
	}

	private static List<Language> getLanguageList(){
		return Arrays.asList(new Language[]{CdmStore.getDefaultLanguage()});
	}

	public static String getDistributionText(Distribution element) {

		String text = "EMPTY";

		Distribution distribution = element;

		NamedArea area = distribution.getArea();
		if(area != null){

			text = NamedArea.labelWithLevel(area, CdmStore.getDefaultLanguage());

			PresenceAbsenceTerm status = distribution.getStatus();

			if (status != null) {
				text += ", " + status.getPreferredRepresentation(CdmStore.getDefaultLanguage()).getLabel();
			}else{
				text += ", NO STATUS";
			}
		}
		return text;
	}

	public static  String getImageText(Media media) {
		 LanguageString title = media.getTitle(CdmStore.getDefaultLanguage());
		if (title != null) {
			return title.getText();
		}
		return "No title.";
	}

	public static  String getElementText(TextData element) {
		String text = null;
		if(element.getFeature().equals(Feature.CITATION())){
			text = "";
			for(DescriptionElementSource source : element.getSources()){
				if(source.getCitation() != null){
					text += source.getCitation().getTitleCache();
				}
				if(source.getNameUsedInSource() != null){
					text += " [" + source.getNameUsedInSource().getTitleCache() + "]";
				}
			}
			if(isBlank(text)){
				text = "No sources provided";
			}
		}else{
			List<Language> languages = Arrays.asList(new Language[]{CdmStore.getDefaultLanguage()});
			LanguageString languageString = element.getPreferredLanguageString(languages);
			text = languageString != null ? languageString.getText() : null;
		}
		return isBlank(text) ? "No text provided" : text;
	}

	public static  String getTaxonInteractionText(TaxonInteraction element) {
		String text = "";
		Taxon taxon2 = element.getTaxon2();
		if(taxon2 != null && taxon2.getName() != null){
			text = taxon2.getName().getTitleCache();
		}else{
			text = "No taxon chosen";
		}

		return text;
	}

	public static  String getCommonNameText(CommonTaxonName commonName) {
		String name = commonName.getName();
		if (name == null || name.length() == 0) {
			name = "No name provided";
		}
		Language language = commonName.getLanguage();
		NamedArea area = commonName.getArea();
		String areaLabel = null;
		String languageLabel = null;
		if (area != null){
			areaLabel = area.getPreferredRepresentation(CdmStore.getDefaultLanguage()).getLabel();			
		} 
		if (language != null){			
			languageLabel = language.getPreferredRepresentation(CdmStore.getDefaultLanguage()).getLabel();
		}
		if (areaLabel != null || languageLabel != null){
			name += " (" ;
			if (languageLabel != null){
				name += languageLabel;
			}
			if (areaLabel != null){
				if (languageLabel != null){
					name += ", ";
				}
				name += areaLabel;
			}
			name +=  ")";
		}
		
		
		return name;
	}

	public static  String getDescriptionText(DescriptionBase element) {
		String text = element.getTitleCache();
		if (text == null || text.length() == 0) {
			text = "No label provided";
		}

		return text;
	}

	public static  String getIndividualsAssociationText(IndividualsAssociation element) {
		SpecimenOrObservationBase<?> derivedUnit = element.getAssociatedSpecimenOrObservation();
		if(derivedUnit != null){
			return derivedUnit.getTitleCache();
		}
		return "No unit chosen";
	}

	public static String getLabel(Object element){
		if (element instanceof FeatureNodeContainer){
			return getFeatureNodeContainerText((FeatureNodeContainer) element);
		}
		else if (element instanceof DescriptionBase) {
			return getDescriptionText((DescriptionBase<?>) element);
		}
		else if(element instanceof CategoricalData){
			String categoricalDataText = getCategoricalDataText((CategoricalData) element);
			if(categoricalDataText==null || categoricalDataText.isEmpty()){
			    categoricalDataText = NO_LABEL_STRING;
			}
            return categoricalDataText;
		}
		else if (element instanceof CommonTaxonName) {
			return getCommonNameText((CommonTaxonName) element);
		}
		else if (element instanceof Distribution) {
			return getDistributionText((Distribution) element);
		}
		else if (element instanceof IndividualsAssociation) {
			return getIndividualsAssociationText((IndividualsAssociation) element);
		}
		else if (element instanceof QuantitativeData) {
			String quantitativeDataText = getQuantitativeDataText((QuantitativeData) element);
			if(isBlank(quantitativeDataText)){
                quantitativeDataText = NO_LABEL_STRING;
			}
            return quantitativeDataText;
		}
		else if (element instanceof TaxonInteraction) {
			return getTaxonInteractionText((TaxonInteraction) element);
		}
		else if (element instanceof TextData) {
			return getElementText((TextData) element);
		}
		else if (element instanceof TemporalData) {
            String lable = getTemporalDataText((TemporalData) element);
            if(isBlank(lable)){
                lable = NO_LABEL_STRING;
            }
            return lable;
        }
		else{
			return element.toString();
		}
	}

    private static String getTemporalDataText(TemporalData element) {
        ExtendedTimePeriod period = element.getPeriod();
        ExtendedTimePeriodFormatter formatter = ExtendedTimePeriodFormatter.NewDefaultInstance();
        return period == null ? NO_LABEL_STRING : formatter.format(period);
    }

    private static boolean isNotBlank(String str){
		return StringUtils.isNotBlank(str);
	}

	private static boolean isBlank(String str){
		return StringUtils.isBlank(str);
	}
}