/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.newWizard;

import java.util.Map;
import java.util.UUID;

import javax.inject.Inject;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.application.CdmChangeEvent;
import eu.etaxonomy.cdm.api.application.CdmChangeEvent.Action;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.dto.CreateTaxonDTO;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.reference.NamedSource;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNodeStatus;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.section.classification.TaxonNodeDetailElement;
import eu.etaxonomy.taxeditor.ui.section.classification.TaxonNodeWizardPage;

/**
 * NewTaxonNodeWizard class.
 *
 * @author n.hoffmann
 * @created Sep 15, 2009
 */
public class NewTaxonNodeWizard extends AbstractNewEntityWizardParentChild<TaxonNode>{

    private TaxonNodeWizardPage taxonNodePage;
    private boolean openEmptyEditor;
    private UUID generatedTaxonNodeUuid;
    private IWizardPageListener wizardPageListener;
    private boolean isOpenInEditor = false;
    private boolean success = true;

    @Inject
    public NewTaxonNodeWizard(){
        setWindowTitle(Messages.TaxonNodeWizardPage_new);
    }

    @Override
    public void addPages() {
        taxonNodePage = new TaxonNodeWizardPage(formFactory, getConversationHolder(), getParentEntity(), wizardPageListener, true);
        addPage(taxonNodePage);
    }

    @Override
    protected void saveEntity() {
        CreateTaxonDTO taxon = taxonNodePage.createTaxon();
        if(taxon == null ){
            openEmptyEditor = true;
        }else{
            getConversationHolder().bind();
            try{
             	UpdateResult result;
             	TaxonNode parent = getParentEntity();

             	Map<Language, LanguageString> notes = ((TaxonNodeDetailElement)taxonNodePage.getDetailElement()).getMultiLanguageTextExcludedNotes();
             	NamedSource source = parent.getSource().checkEmpty()? null: parent.getSource();

             	TaxonNodeStatus status = ((TaxonNodeDetailElement)taxonNodePage.getDetailElement()).getTaxonNodeStatus();
             	if (status == null){
             	    notes = null;
             	}

             	result = CdmStore.getService(ITaxonNodeService.class).createNewTaxonNode(parent.getUuid(),
             	        taxon, source, parent.getMicroReference(), status, notes);


            	if (result.isOk()){
            	    this.setEntity((TaxonNode)result.getCdmEntity());
                    CdmApplicationState.getCurrentDataChangeService()
                             .fireChangeEvent(new CdmChangeEvent(Action.Create, result.getUpdatedObjects(), NewTaxonNodeWizard.class), true);

	             }else{
	                 setSuccess(false);
	                 if (!result.isOk()){
	                     if (!result.getExceptions().isEmpty()){
	                         MessagingUtils.error(getClass(), "Taxon could not be created", result.getExceptions().iterator().next());
	                     }else{
	                         MessagingUtils.warn(getClass(), "Taxon could not be created");
	                     }

	                 }
	             }
          }catch(IllegalArgumentException e){
                MessagingUtils.warningDialog("Taxon already exists in classfication", getClass(), e.getMessage());
            }
        }
    }

    public boolean openInEditor() {
        return isOpenInEditor ;
    }

    public void setOpenInEditor(boolean isOpenInEditor) {
        this.isOpenInEditor = isOpenInEditor;
    }

    public boolean openEmpty(){
        return openInEditor() && openEmptyEditor;
    }

    public TaxonNode getTaxonNode(){
        if(generatedTaxonNodeUuid != null){
            return CdmStore.getService(ITaxonNodeService.class).load(generatedTaxonNodeUuid);
        }
        return null;
    }

    public ITaxonTreeNode getParentTreeNode(){
        return taxonNodePage.getParentTreeNode();
    }

    @Override
    protected String getEntityName() {
        return "Taxon";
    }

    public void addWizardPageListener(IWizardPageListener wizardPageListener){
        this.wizardPageListener = wizardPageListener;
    }

    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    @Override
    public void createParentEntity() {
        Object selectedObject = getSelection().getFirstElement();
        if(selectedObject instanceof TaxonNodeDto){
            if(selectedObject instanceof TaxonNodeDto){
                TaxonNodeDto taxonNodeDto = (TaxonNodeDto) selectedObject;
                setParentEntity(CdmStore.getService(ITaxonNodeService.class).load(taxonNodeDto.getUuid()));
            }
        }
    }
}