/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operation;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;

/**
 * Superclass for all operation that have to be committed imediately after execution.
 *
 * Note: You have to call the {@link #bind()} method in the {@link #execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)},
 * {@link #undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)} and {@link #redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}
 * methods so the correct persistence context gets bound.
 *
 * @author n.hoffmann
 * @created 08.05.2009
 * @version 1.0
 */
public abstract class AbstractPersistentPostOperation extends AbstractPostTaxonOperation {
	private final IConversationEnabled conversationEnabled;

	protected TaxonNodeDto parentNode;

	   /**
     * <p>Constructor for AbstractPersistentPostOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
     */
    protected AbstractPersistentPostOperation(String label, IUndoContext undoContext,
            IPostOperationEnabled postOperationEnabled,
            IConversationEnabled conversationEnabled) {
        this(label, undoContext, postOperationEnabled, conversationEnabled, null);
    }

	/**
	 * <p>Constructor for AbstractPersistentPostOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
	 */
	protected AbstractPersistentPostOperation(String label, IUndoContext undoContext,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled,
			ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
		super(label, undoContext, postOperationEnabled, cdmEntitySessionEnabled);
		this.conversationEnabled = conversationEnabled;
	}

	/**
	 * <p>Constructor for AbstractPersistentPostOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxonNode a {@link eu.etaxonomy.cdm.model.taxon.TaxonNode} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
	 */
	public AbstractPersistentPostOperation(String label,
			IUndoContext undoContext, TaxonNodeDto taxonNode,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled,
			ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
		super(label, undoContext, taxonNode, postOperationEnabled, cdmEntitySessionEnabled);
		this.conversationEnabled = conversationEnabled;

	}

	/**
	 * <p>Constructor for AbstractPersistentPostOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
	 * @param parentNode a {@link eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode} object.
	 */
	public AbstractPersistentPostOperation(String label,
			IUndoContext undoContext,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled,
            ICdmEntitySessionEnabled cdmEntitySessionEnabled, TaxonNodeDto parentNode) {
		super(label, undoContext, postOperationEnabled, cdmEntitySessionEnabled);
		this.parentNode = parentNode;
		this.conversationEnabled = conversationEnabled;

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.AbstractPostOperation#postExecute(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	/** {@inheritDoc} */
	@Override
    protected IStatus postExecute(Object objectAffectedByOperation) {
		Assert.isNotNull(conversationEnabled, "Operation has to have a valid conversation attached.");


		if (!conversationEnabled.getConversationHolder().isClosed()){
			conversationEnabled.getConversationHolder().bind();
			conversationEnabled.getConversationHolder().commit(true);
		}
		IStatus status = super.postExecute(objectAffectedByOperation);

		return status;
	}

	/**
	 * Binds the conversation that was attached to this operation.
	 */
	public void bind(){
		conversationEnabled.getConversationHolder().bind();
		if(getCdmEntitySessionEnabled() != null && getCdmEntitySessionEnabled().getCdmEntitySession()!=null) {
		    getCdmEntitySessionEnabled().getCdmEntitySession().bind();
        }
	}

}
