/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceResolver;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 27.04.2018
 */
public class CdmPreferenceCache {
    Map<String,List<CdmPreference>> preferenceCache = new HashMap<>();

    private static CdmPreferenceCache instance;

    private final static Logger logger = Logger.getLogger(CdmPreferenceCache.class);

    public static CdmPreferenceCache instance(){
        if(instance == null){
            instance = new CdmPreferenceCache();
        }
        return instance;
    }

    public CdmPreference get(String predicate){
        if (preferenceCache.get(predicate) != null){
            if (preferenceCache.get(predicate).size() > 1){
                CdmPreference pref = PreferenceResolver.resolve(preferenceCache.get(predicate), CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.getByKey(predicate)));
                return pref;
            }
            return preferenceCache.get(predicate).get(0);
        }
        return null;
    }

    public List<CdmPreference> getAllForPredicate(String predicate){
        if (preferenceCache.get(predicate) != null){
            return preferenceCache.get(predicate);

        }
        return null;
    }

    //looking for the equal prefkey
    public CdmPreference get(PrefKey key){
        if (preferenceCache.containsKey(key.getPredicate())){
            for (CdmPreference pref: preferenceCache.get(key.getPredicate())){
                if (pref.getSubjectString().equals(key.getSubject())){
                    return pref;
                }
            }

        }
        return null;
    }

    public void put(CdmPreference pref){
        if (preferenceCache.containsKey(pref.getPredicate())){
            List<CdmPreference> cachedPreferences = preferenceCache.get(pref.getPredicate());
            CdmPreference prefToRemove = null;
            for (CdmPreference cachedPref:cachedPreferences){
                if (pref.getKey().equals(cachedPref.getKey())){
                    prefToRemove = cachedPref;
                }
            }
            cachedPreferences.remove(prefToRemove);
            cachedPreferences.add(pref);
        }else{
            List<CdmPreference> prefs = new ArrayList<>();
            prefs.add(pref);
            preferenceCache.put(pref.getPredicate(), prefs);
        }
    }

    public boolean remove(CdmPreference pref){
        if (preferenceCache.containsKey(pref.getPredicate())){
            List<CdmPreference> cachedPreferences = preferenceCache.get(pref.getPredicate());
            CdmPreference prefToRemove = null;
            for (CdmPreference cachedPref:cachedPreferences){
                if (pref.getKey().equals(cachedPref.getKey())){
                    prefToRemove = cachedPref;
                }
            }
            cachedPreferences.remove(prefToRemove);
        }
        return true;
    }

    public void getAllTaxEditorDBPreferences(){
        ICdmRepository controller;
        controller = CdmStore.getCurrentApplicationConfiguration();
        List<CdmPreference> allPreferences = controller.getPreferenceService().list();
        preferenceCache.clear();

        for (CdmPreference pref: allPreferences){
            put(pref);
        }

        loadAndSet(PreferencePredicate.AvailableDistributionStatus, TermType.PresenceAbsenceTerm);
        loadAndSet(PreferencePredicate.AvailableRanks, TermType.Rank);
        loadAndSet(PreferencePredicate.AvailableNomenclaturalStatus, TermType.NomenclaturalStatusType);
        loadAndSet(PreferencePredicate.AvailableNameRelationshipType, TermType.NameRelationshipType);
        loadAndSet(PreferencePredicate.AvailableLanguages, TermType.Language);
    }

    private void loadAndSet(PreferencePredicate preferencePredicate, TermType termType) {
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), preferencePredicate);
        if (get(key) != null){
            if (!PreferencesUtil.getOverrideForPreference(preferencePredicate.getKey()) || !get(key).isAllowOverride()){
                //get terms for the uuids... and add them to the termManager as preferred terms
                Collection<TermDto> dtos = createDtoList(key);
                if (dtos != null){
                    CdmStore.getTermManager().setPreferredTermsByType(dtos, termType);
                }
            }
        }
    }

    public Collection<TermDto> createDtoList(PrefKey key) {
        ITermService termService = CdmStore.getService(ITermService.class);
        List<UUID> uuidList = createUuidList(key);
        Collection<TermDto> dtos = termService.findByUUIDsAsDto(uuidList);
        return dtos;
    }

    public List<UUID> createUuidList(PrefKey key) {
        List<UUID> uuidList = new ArrayList<>();
        if (get(key).getValue() != null){
            String[] uuidArray =findBestMatching(key).getValue().split(";");
            for (String uuidString:uuidArray){
                try {
                    uuidList.add(UUID.fromString(uuidString));
                } catch (Exception e) {
                    logger.warn("Preference loading failed", e);
                }
            }
        }
        return uuidList;
    }

    public CdmPreference findBestMatching(PrefKey key) {
        List<CdmPreference> allPrefs = new ArrayList<>();
        for (String predicate: preferenceCache.keySet()){
            allPrefs.addAll(preferenceCache.get(predicate));
        }
        return PreferenceResolver.resolve(allPrefs, key);
    }
}
