/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 11.02.2020
 */
public class IOPreferences extends CdmPreferencePage implements SelectionListener {

    protected Boolean isShowIOMenu;
    protected CdmPreference showIOMenuPref;
    protected boolean isAllowOverrideShowIOMenu;
    protected boolean isOverrideShowIOMenu;

    protected Combo showIOMenuButton;
    protected Button allowOverrideShowIOMenuButton;

    @Override
    protected Control createContents(Composite parent) {
        getValues();
        final Composite composite = createComposite(parent);

        GridData gridData = createTextGridData();
        if (!isAdminPreference && showIOMenuPref != null && !showIOMenuPref.isAllowOverride() ) {
            Label label = new Label(composite, SWT.NONE);
            label.setText(
                    "The CDM settings don't allow to set the preferences for using the distribution editor locally. If you need to make local settings, please ask an administrator.");
            this.setDefaultButtonActivated(false);
        } else {
            Label separator= new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
            separator.setLayoutData(gridData);
            separator.setVisible(false);
            showIOMenuButton = createBooleanCombo(composite, SHOW, DO_NOT_SHOW, PreferencePredicate.ShowImportExportMenu, "Show Export/Import Menu",
                    isAdminPreference);

            int index = 0;
            for (String itemLabel : showIOMenuButton.getItems()) {
                if (isShowIOMenu == null && itemLabel.startsWith(Messages.Preference_Use_Default)) {
                    showIOMenuButton.select(index);
                    break;
                }
                if (isShowIOMenu != null && itemLabel.equals(SHOW) && isShowIOMenu){
                    showIOMenuButton.select(index);
                    break;
                }
                if (isShowIOMenu != null && itemLabel.equals(DO_NOT_SHOW) && !isShowIOMenu){
                    showIOMenuButton.select(index);
                    break;
                }
                index++;
            }

            showIOMenuButton.addSelectionListener(this);


            if (isAdminPreference){
                allowOverrideShowIOMenuButton = createAllowOverrideButton(composite);
                allowOverrideShowIOMenuButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        isAllowOverrideShowIOMenu = allowOverrideShowIOMenuButton.getSelection();
                        setApply(true);
                    }
                });
                allowOverrideShowIOMenuButton.setSelection(isAllowOverrideShowIOMenu);

            }
            if(!isAdminPreference){
                if (!isAllowOverrideShowIOMenu){
                    showIOMenuButton.setEnabled(false);
                }
            }
        }
        return composite;

    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }

        if (isAllowOverrideShowIOMenu){
            if (isShowIOMenu != null){
                PreferencesUtil.setBooleanValue(PreferencePredicate.ShowImportExportMenu.getKey(), isShowIOMenu);
            }
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowImportExportMenu.getKey()), isOverrideShowIOMenu);
        }



       return true;
   }

    @Override
    protected void getValues(){
        isAdminPreference = false;

        CdmPreferenceCache cache = CdmPreferenceCache.instance();
        isOverrideShowIOMenu = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowImportExportMenu.getKey()), true) != null?
                PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowImportExportMenu.getKey()), true): false;
        if (isOverrideShowIOMenu){
            isShowIOMenu = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowImportExportMenu.getKey(), true);
        }


        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowImportExportMenu);
        showIOMenuPref = cache.findBestMatching(key);
        if (showIOMenuPref != null){
            isAllowOverrideShowIOMenu = showIOMenuPref.isAllowOverride();
        }else{
            isAllowOverrideShowIOMenu = true;
        }
    }


    @Override
    public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(this.showIOMenuButton)) {
            String text = showIOMenuButton.getText();
            if (text.startsWith(Messages.Preference_Use_Default)){
                isOverrideShowIOMenu = false;
                isShowIOMenu = null;

            }else if (text.equals(SHOW)){
                isShowIOMenu = true;
                isOverrideShowIOMenu = true;

            }else{
                isShowIOMenu = false;
                isOverrideShowIOMenu = true;
            }
        }
    }

    @Override
    protected void performDefaults() {
        isOverrideShowIOMenu = false;
        isShowIOMenu = null;
        setSelections();
        setApply(true);
    }

    protected void setSelections(){
        if (isShowIOMenu == null || (!isOverrideShowIOMenu && !isAdminPreference)){
            showIOMenuButton.select(0);
        }else{
            if (isShowIOMenu){
                showIOMenuButton.select(1);
            }else{
                showIOMenuButton.select(2);
            }
        }
        if (isAdminPreference){
            allowOverrideShowIOMenuButton.setSelection(isAllowOverrideShowIOMenu);
        }
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

}
