/**
 * Copyright (C) 2018 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;

import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author pplitzner
 * @since Oct 26, 2018
 */
public class CheckBoxTreeComposite extends Composite implements SelectionListener {

    private static final Font FONT_DEFAULT = JFaceResources.getFontRegistry().get(JFaceResources.DEFAULT_FONT);
    private static final Font FONT_ITALIC = JFaceResources.getFontRegistry().getItalic(JFaceResources.DEFAULT_FONT);
    private static final Font FONT_BOLD = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);
    private CheckboxTreeViewer viewer;
    private Button btnCollapse;
    private Button btnExpand;
    private Button btnSelectAllChildren;
    private Button btnSelectDirectChildren;

    protected Object[] checkedElements;

    private boolean allowTopLevelSelection = false;

    public CheckBoxTreeComposite(Composite parent, ITreeContentProvider contentProvider, IBaseLabelProvider labelProvider, int style) {
        super(parent, style);
        GridLayout layout = new GridLayout();
        layout.marginHeight = 0;
        setLayout(layout);

        Composite toolbar = new Composite(this, SWT.NONE);
        toolbar.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
        GridLayout gl_toolbar = new GridLayout(4, false);
        gl_toolbar.horizontalSpacing = 4;
        gl_toolbar.marginHeight = 0;
        gl_toolbar.verticalSpacing = 0;
        toolbar.setLayout(gl_toolbar);

        btnCollapse = new Button(toolbar, SWT.NONE);
        btnCollapse.addSelectionListener(this);
        btnCollapse.setImage(ImageResources.getImage(ImageResources.COLLAPSE_ALL));
        btnCollapse.setToolTipText(""); //$NON-NLS-1$

        btnExpand = new Button(toolbar, SWT.NONE);
        btnExpand.addSelectionListener(this);
        btnExpand.setImage(ImageResources.getImage(ImageResources.EXPAND));
        btnExpand.setToolTipText(""); //$NON-NLS-1$

        btnSelectDirectChildren = new Button(toolbar, SWT.NONE);
        btnSelectDirectChildren.addSelectionListener(this);
        btnSelectDirectChildren.setImage(ImageResources.getImage(ImageResources.FLAT));
        btnSelectDirectChildren.setToolTipText(Messages.CheckBoxTreeComposite_SELECT_DIRECT_CHILDREN);

        btnSelectAllChildren = new Button(toolbar, SWT.NONE);
        btnSelectAllChildren.addSelectionListener(this);
        btnSelectAllChildren.setImage(ImageResources.getImage(ImageResources.HIERARCHICAL));
        btnSelectAllChildren.setToolTipText(""); //$NON-NLS-1$

        Tree tree = new Tree(this, SWT.BORDER | SWT.CHECK);
        tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        tree.addListener(SWT.Selection, event -> {
            if (event.detail == SWT.CHECK) {
                TreeItem item = (TreeItem) event.item;
                if(!allowTopLevelSelection && item.getParentItem()==null){
                    // top level selection not allowed
                    item.setChecked(false);
                    return;
                }

                if (item.getChecked()){
                    if (checkedElements != null){
                        Object[] tempArray = Arrays.copyOf(checkedElements, checkedElements.length +1);
                        tempArray[checkedElements.length] = item.getData();
                        checkedElements = tempArray;
                    }
                }else{
                    int index = 0;
                    if (checkedElements != null){
                        for (Object o:checkedElements){
                            if (o != null && o.equals(item.getData())){
                                break;
                            }
                            index ++;
                        }
                        checkedElements[index] = null;
                    }
                }

                updateItem(item);
            }
        });

        viewer = new CdmCheckBoxTreeViewer(tree);
        viewer.setContentProvider(contentProvider);
        viewer.setLabelProvider(labelProvider);
    }

    private void updateItem(TreeItem treeItem){
        TreeItem parentItem = treeItem.getParentItem();
        if(parentItem!=null){
            updateItem(parentItem);
        }
        else{
            updateChildItems(treeItem);
        }
    }

    private void updateChildItems(TreeItem treeItem){
        treeItem.setFont(FONT_DEFAULT);
        TreeItem[] children = treeItem.getItems();
        if(children.length==0){
            return;
        }
        int checkedCount = 0;
        for (TreeItem child : children) {
            updateChildItems(child);

            if(child.getChecked() ){
                if ((child.getFont().equals(FONT_BOLD)) || (child.getItems().length == 0)) {
                    checkedCount++;
                }
                //some children are checked
                treeItem.setFont(FONT_ITALIC);
            }
            else if(child.getFont().equals(FONT_ITALIC)
                    || child.getFont().equals(FONT_BOLD)){
                //some children are checked
                treeItem.setFont(FONT_ITALIC);
            }
        }
        if(checkedCount==children.length){
            //all children are checked
            treeItem.setFont(FONT_BOLD);
        }
        else if(!treeItem.getFont().equals(FONT_ITALIC)){
            if(checkedCount>0){
                //some children are checked
                treeItem.setFont(FONT_ITALIC);
            }
            else{
                //no children are checked
                treeItem.setFont(FONT_DEFAULT);
            }
        }
    }


    public CheckboxTreeViewer getViewer() {
        return viewer;
    }

    public void setAllowTopLevelSelection(boolean allowTopLevelSelection) {
        this.allowTopLevelSelection = allowTopLevelSelection;
    }

    /**
     * Checks the tree items corresponding to the given elements.<br>
     * @param elements
     */
    public void setCheckedElements(Object[] elements) {

        this.checkedElements = elements;
        viewer.setCheckedElements(elements);
        for (Object element: checkedElements) {
            viewer.reveal(element);
        }
        TreeItem[] rootItems = viewer.getTree().getItems();
        for (TreeItem rootItem : rootItems) {
            updateItem(rootItem);
        }
    }

    /**
     * Grays the tree items corresponding to the given elements.<br>
     * @param elements
     */
    public void setGrayedElements(Object[] elements) {
        viewer.setGrayedElements(elements);
        for (Object element: elements) {
            viewer.reveal(element);
        }
        TreeItem[] rootItems = viewer.getTree().getItems();
        for (TreeItem rootItem : rootItems) {
            updateItem(rootItem);
        }
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        ITreeSelection structuredSelection = viewer.getStructuredSelection();
        Object firstElement = structuredSelection.getFirstElement();
        if(e.widget == btnCollapse){
            viewer.collapseAll();
        }
        else if(e.widget == btnExpand){
            viewer.expandToLevel(firstElement, AbstractTreeViewer.ALL_LEVELS);
        }
        else if(e.widget == btnSelectDirectChildren){
            Widget item = viewer.testFindItem(firstElement);
            if(item instanceof TreeItem){
                viewer.expandToLevel(firstElement, 1);
                TreeItem treeItem = (TreeItem) item;
                TreeItem[] childItems = treeItem.getItems();
                List<TreeItem> childItemList = Arrays.asList(childItems);
                boolean allChecked = childItemList.stream().allMatch(childItem->childItem.getChecked());
                childItemList.stream().forEach(childItem->childItem.setChecked(!allChecked));
                TreeItem parent = getRoot(treeItem);
                updateItem(parent);
            }
        }
        else if(e.widget == btnSelectAllChildren){
            Widget item = viewer.testFindItem(firstElement);
            if(item instanceof TreeItem){
                viewer.expandToLevel(firstElement, AbstractTreeViewer.ALL_LEVELS);
                viewer.setSubtreeChecked(firstElement, !((TreeItem) item).getFont().equals(FONT_BOLD));
                TreeItem parent = getRoot((TreeItem) item);
                updateItem(parent);
                ((TreeItem) item).setChecked(false);
            }
        }
    }

    private TreeItem getRoot(TreeItem treeItem) {
        TreeItem parent = treeItem;
        while(parent.getParentItem()!=null){
            parent = parent.getParentItem();
        }
        return parent;
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
    }
}