/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizard;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermStore;

/**
 * <p>VocabularyTermWizard class.</p>
 *
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class VocabularyTermWizard<T extends DefinedTermBase> extends Wizard implements IConversationEnabled {
	
	public T term = null;
	
	private boolean editMode = false;

	private Class<T> termClass;

	private ConversationHolder conversation;

	/**
	 * <p>Constructor for VocabularyTermWizard.</p>
	 *
	 * @param vocabularyClass a {@link java.lang.Class} object.
	 */
	public VocabularyTermWizard(Class<T> vocabularyClass, ConversationHolder conversation){	
		this.conversation = conversation;
		termClass = vocabularyClass;
	}
	
	/**
	 * <p>Constructor for VocabularyTermWizard.</p>
	 *
	 * @param vocabularyClass a {@link java.lang.Class} object.
	 * @param selectedTerm a T object.
	 */
	public VocabularyTermWizard(Class<T> vocabularyClass, T selectedTerm, ConversationHolder conversation){
		this(vocabularyClass, conversation);		
		term = (T) CdmStore.getService(ITermService.class).load(selectedTerm.getUuid());
		editMode = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	/** {@inheritDoc} */
	@Override
	public void addPages() {
		super.addPages();
		
		addPage(new VocabularyTermWizardPage());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {

		if(editMode){
			TermStore.updateVocabulary(getTerm());
		}else{
			TermStore.saveTerm(getTerm());
		}
		
		getConversationHolder().commit(true);
		
		return true;
	}

	/**
	 * <p>Setter for the field <code>term</code>.</p>
	 *
	 * @param term a T object.
	 */
	public void setTerm(T term) {
		this.term = term;
	}

	/**
	 * <p>Setter for the fields <code>term</code>.</p>
	 *
	 * @param text
	 * @param label a {@link java.lang.String} object.
	 * @param abbreviatedLabel a {@link java.lang.String} object.
	 */
	public void setTerm(String text, String label, String abbreviatedLabel) {
		Representation representation = getTerm().getRepresentation(CdmStore.getDefaultLanguage());
		representation.setText(text);
		representation.setLabel(label);
		representation.setAbbreviatedLabel(abbreviatedLabel);
	}
	
	/**
	 * <p>Getter for the field <code>term</code>.</p>
	 *
	 * @return the term
	 */
	public T getTerm() {
		if(term == null){
			try {
				term = (T) termClass.newInstance();
				
				term.addRepresentation(Representation.NewInstance("", "", "", CdmStore.getDefaultLanguage())); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} catch (InstantiationException e) {
				MessagingUtils.error(this.getClass(), "InstantiationException when setting term", e); //$NON-NLS-1$
			} catch (IllegalAccessException e) {
				MessagingUtils.error(this.getClass(), "IllegalAccessException when setting term", e); //$NON-NLS-1$
			}
		}
		return term;
	}
	
	public Representation getRepresentation(){		
		return getTerm().getRepresentation(CdmStore.getDefaultLanguage());
	}

	/**
	 * <p>isEditMode</p>
	 *
	 * @return the editMode
	 */
	public boolean isEditMode() {
		return editMode;
	}

	/**
	 * <p>Getter for the field <code>termClass</code>.</p>
	 *
	 * @return a {@link java.lang.Class} object.
	 */
	public Class<T> getTermClass() {
		return termClass;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap changeEvents) {
		// not needed
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

}
