package eu.etaxonomy.taxeditor.security;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.ui.AbstractSourceProvider;
import org.eclipse.ui.ISources;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.persistence.permission.Role;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;

/**
 * Provides the Roles assigned to the currently authenticated principal as the
 * variable {@code eu.etaxonomy.taxeditor.security.userRoles}
 *
 * @author a.kohlbecker
 */
public class AuthenticationSourceProvider extends AbstractSourceProvider implements Observer {

	public final static String USER_ROLES_VARIABLE = "eu.etaxonomy.taxeditor.security.userRoles";
	public final static String USER_NAME_VARIABLE = "eu.etaxonomy.taxeditor.security.userName";
	public final static String USER_VARIABLE = "eu.etaxonomy.taxeditor.security.user";

	private final static String[] PROVIDED_SOURCE_NAMES = new String[] {
	    USER_ROLES_VARIABLE,
	    USER_NAME_VARIABLE,
	    USER_VARIABLE
	    };

	private final static Map<String, Object> stateMap = new HashMap<String, Object>();

	public AuthenticationSourceProvider() {
		super();
		initialize();
	}

	private void initialize() {
		CdmStore.getLoginManager().addObserver(this);
	}

	@Override
	public void dispose() {
		CdmStore.getLoginManager().deleteObserver(this);
	}

	@Override
	public Map getCurrentState() {

		// SecurityContextHolder.getContext().
		List<Role> roles = getCurrentAuthenticationsRoles();
		List<String> rolesStr = new ArrayList<String>(roles.size());
		for(Role r : roles){
			rolesStr.add(r.getAuthority());
		}

		stateMap.put(USER_ROLES_VARIABLE, rolesStr);
		stateMap.put(USER_VARIABLE, getCurrentAutheticationPrincipal());
		stateMap.put(USER_NAME_VARIABLE, getCurrentAutheticationName());
		return stateMap;
	}

    private String getCurrentAutheticationName() {
        Authentication authentication = CdmStore.getCurrentAuthentiation();
        if (authentication != null) {
            return authentication.getName();
        }
        return null;
    }

    /**
     * @return most probably a Cdm User instance or null
     */
    private Object getCurrentAutheticationPrincipal() {
        Authentication authentication = CdmStore.getCurrentAuthentiation();
        if (authentication != null) {
            return authentication.getPrincipal();
        }
        return null;
    }


    /*
	 * TODO refactor into User once Role is a model class
	 */
	private List<Role> getCurrentAuthenticationsRoles() {

		List<Role> roles = new ArrayList<>();
		Authentication authentication = CdmStore.getCurrentAuthentiation();
		if (authentication == null) {
			return roles;
		}

		Collection<? extends GrantedAuthority> authorities = authentication
				.getAuthorities();
		if (authorities == null) {
			return roles;
		}

		Role role = null;
		for (GrantedAuthority ga : authorities) {
			try {
				// check if it is a valid role
				role = Role.fromString(ga.getAuthority());
				if (role != null) {
					roles.add(role);
				}
			} catch (Exception e) {
				/* IGNORE */
			}
		}
		return roles;

	}

	@Override
    public String[] getProvidedSourceNames() {
		return PROVIDED_SOURCE_NAMES;
	}

	@Override
	public void update(Observable o, Object arg) {
		if(o instanceof LoginManager){
			/*
			 * This triggers an update of the variable state, and will update also
			 * all listeners to the evaluation service. So that every menu point,
			 * which is also expression controlled, gets updated too.
			 */
			fireSourceChanged(ISources.WORKBENCH, getCurrentState());
		}
	}

}
