/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.store;

import java.io.File;
import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.XMLMemento;
import org.eclipse.ui.internal.Workbench;
import org.springframework.remoting.RemoteAccessException;

import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.MementoHelper;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * The context manager mediates context start/stop and workbench shutdowns to all registered listeners.
 *
 * @author n.hoffmann
 * @created Sep 30, 2010
 */
public class ContextManager implements IWorkbenchListener{

    private final ListenerList<IContextListener> contextListeners = new ListenerList<>();

    private IMemento memento;

    /**
     * <p>Constructor for ContextManager.</p>
     */
    protected ContextManager() {
        if(Workbench.getInstance() != null) {
            PlatformUI.getWorkbench().addWorkbenchListener(this);
        }
    }

    public void addContextListener(IContextListener listener){
        contextListeners.add(listener);
    }

    public void removeContextListener(IContextListener listener) {
        contextListeners.remove(listener);
    }

    public void notifyContextStartWithoutDialog(IProgressMonitor monitor) {
        MessagingUtils.info("Notifying context listeners, that the context has started.");

        readMemento();

        for(final Object listener : contextListeners.getListeners()){
            ((IContextListener) listener).contextStart(memento, monitor);
        }
    }

    public void notifyContextStart() throws RemoteAccessException {
        MessagingUtils.info("Notifying context listeners, that the context has started.");
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(AbstractUtility.getShell());

        try {
            dialog.run(false, false, new IRunnableWithProgress() {

                @Override
                public void run(IProgressMonitor monitor)
                        throws InvocationTargetException, InterruptedException {
                    monitor.beginTask("Starting context", contextListeners.size());


                    readMemento();

                    for(final Object listener : contextListeners.getListeners()){
                        ((IContextListener) listener).contextStart(memento, monitor);
                        monitor.worked(1);
                    }
                    monitor.done();
                }
            });
        } catch (InvocationTargetException e) {
            if (e.getTargetException().getClass().equals(RemoteAccessException.class)){
                if (e.getTargetException().getMessage().contains("403")){
                   // new CdmAuthenticationException("Access Denied", e.getTargetException());
                    throw new CdmAuthenticationException("You are logged in now but you are not permitted to use the TaxEditor with the selected data source", e.getTargetException());
                }else{
                    MessagingUtils.error(getClass(), e.getTargetException());
                }
            }else{
                MessagingUtils.error(getClass(), e);
            }
        } catch (InterruptedException e) {
            MessagingUtils.error(getClass(), e);
        }
    }

    public void notifyContextRefresh() {
        MessagingUtils.info("Notifying context listeners, that the context needs to be refreshed.");
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(AbstractUtility.getShell());

        try {
            dialog.run(false, false, new IRunnableWithProgress() {

                @Override
                public void run(IProgressMonitor monitor)
                        throws InvocationTargetException, InterruptedException {
                    monitor.beginTask("Refreshing context", contextListeners.size());

                    for(final Object listener : contextListeners.getListeners()){
                        ((IContextListener) listener).contextRefresh(monitor);
                        monitor.worked(1);
                    }
                    monitor.done();
                }
            });
        } catch (InvocationTargetException e) {
            if (e.getTargetException().getClass().equals(RemoteAccessException.class)){
                if (e.getTargetException().getMessage().contains("403")){
                    MessagingUtils.confirmDialog("Access denied", "Access denied");
                }else{
                    MessagingUtils.error(getClass(), e.getTargetException());
                }
            }else{
                MessagingUtils.error(getClass(), e);
            }
        } catch (InterruptedException e) {
            MessagingUtils.error(getClass(), e);
        }
    }

    public void notifyContextAboutToStop(final IProgressMonitor monitor){

        IProgressMonitor subMonitor = AbstractUtility.getSubProgressMonitor(monitor, 1);

        subMonitor.beginTask("Stoping context", contextListeners.size());
        // we are creating the memento here; even if the context is not stopped
        createMemento();

        for(final Object listener : contextListeners.getListeners()){
            ((IContextListener) listener).contextAboutToStop(memento, subMonitor);
            subMonitor.worked(1);
        }

        subMonitor.done();
    }

    public void notifyContextStop(IProgressMonitor monitor) {

        IProgressMonitor subMonitor = AbstractUtility.getSubProgressMonitor(monitor, 1);

        subMonitor.beginTask("Stoping context", contextListeners.size());
        MessagingUtils.info("Notifying context listeners, that the context has stopped.");

        for(Object listener : contextListeners.getListeners()){
            ((IContextListener) listener).contextStop(memento, subMonitor);
            subMonitor.worked(1);
        }

        saveMemento();
        subMonitor.done();
    }

    @Override
    public boolean preShutdown(IWorkbench workbench, boolean forced) {

        createMemento();

        IProgressMonitor monitor = null;

        for(Object listener : contextListeners.getListeners()){
            ((IContextListener) listener).workbenchShutdown(memento, monitor);
        }

        saveMemento();

        // return true in any case, otherwise the application will not stop
        return true;
    }

    @Override
    public void postShutdown(IWorkbench workbench) {
    }

    private void readMemento(){
        try {
            memento = MementoHelper.readMementoFromFile(getStateFileForCurrentDatabase());
        } catch (FileNotFoundException e) {
            // no memento -> no previous state
            MessagingUtils.info("No state file for datasource");
        }
    }

    private void createMemento(){

        if (CdmStore.getActiveCdmSource() != null) {

            try {
                String name = CdmStore.getActiveCdmSource().getName();
                name = name.trim();
                name = name.replace(" ", "_");
                memento = XMLMemento.createWriteRoot(name);

                MessagingUtils.info("DataSource found. Memento created.");
            } catch (Exception e) {
                // The memento could not be created, but a not closable editor is avoided for this case.
                MessagingUtils.error(this.getClass(), "The memento could not be created", e);
            }
        } else {
            MessagingUtils.info("Not storing state data, because no DataSource present.");
        }
    }

    private boolean saveMemento(){
        return MementoHelper.saveMementoToFile(memento, getStateFileForCurrentDatabase()) != null;
    }

    protected File getStateFileForCurrentDatabase() {
        if(CdmStore.getActiveCdmSource() == null){
            return null;
        }

        IPath path = TaxeditorStorePlugin.getDefault().getStateLocation();
        if (path == null) {
            return null;
        }
        path = path.append("editor_state_" + CdmStore.getActiveCdmSource().getName() + ".xml");
        return path.toFile();
    }
}
