/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.term.IHasTermType;
import eu.etaxonomy.cdm.model.term.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.editor.definedterm.TermTransfer;
import eu.etaxonomy.taxeditor.editor.definedterm.e4.DefinedTermDropAdapterE4;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.TermNodeTransfer;
import eu.etaxonomy.taxeditor.termtree.e4.operation.AddFeatureOperation;
import eu.etaxonomy.taxeditor.termtree.e4.operation.AddOntologyTermOperation;
import eu.etaxonomy.taxeditor.termtree.e4.operation.MoveFeatureOperation;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TermVocabularySelectionDialog;
import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.OntologyTermWrapper;

public class TermTreeDropAdapter extends ViewerDropAdapter {

    protected ITermTreeEditor editor;
    private UISynchronize sync;

    public TermTreeDropAdapter(ITermTreeEditor editor, Viewer viewer, UISynchronize sync) {
		super(viewer);
		this.editor = editor;
		this.sync = sync;
	}

	@Override
	public boolean performDrop(Object data) {
//	    if(StoreUtil.promptCheckIsDirty(editor)){
//	        return false;
//	    }
		Object currentTarget = getCurrentTarget();
		TermNode target = null;
		if(currentTarget instanceof TermTree){
		    target = ((TermTree) currentTarget).getRoot();
		}
		else if(currentTarget instanceof TermNode){
		    target = (TermNode) currentTarget;
		}

		if (target.getUuid() == null){
		    if(StoreUtil.promptCheckIsDirty(editor)){
	            return false;
	        }
		}
		int position = 0;

		if (target != null) {
			int location = getCurrentLocation();
			TermNode parent = target.getParent();
			if(parent !=null){

			    if (location == LOCATION_BEFORE) {
			        position = Math.max(0, parent.getIndex(target));
			        target = parent;
			    }

			    if (location == LOCATION_AFTER) {
			        position = parent.getIndex(target)+1;
			        target = parent;
			    }
			}
		}

        if(target==null){
            MessagingUtils.warningDialog("", this, ""); //$NON-NLS-1$ //$NON-NLS-1$
            return false;
        }
		Collection<Object> droppedObjects = Collections.emptyList();
		if(data instanceof Object[]){
		    droppedObjects = Arrays.asList((Object[])data);
		}
		else if(data instanceof IStructuredSelection){
		    droppedObjects = ((IStructuredSelection) data).toList();
		}
		TreeViewer viewer = (TreeViewer) getViewer();

		// cannot drop a feature node onto itself
		for (Object droppedObject : droppedObjects) {
		    if(droppedObject==null){
		        MessagingUtils.warningDialog(DefinedTermDropAdapterE4.MOVE_FAILED, this.getClass(),
		                Messages.FeatureTreeDropAdapter_MOVE_FAILED_SAVE_MESSAGE);
		        return false;
		    }
			if (droppedObject.equals(target)) {
				return false;
			}

		}
		for (Object droppedObject : droppedObjects) {
		    //check term type compatibility
		    TermType targetType = target.getTermType();
		    TermType droppedType = null;
		    if(droppedObject instanceof IHasTermType){
		        droppedType = ((IHasTermType)droppedObject).getTermType();
		        if(droppedType!=null && !droppedType.equals(targetType) && !droppedType.isKindOf(targetType)){
		            MessagingUtils.warningDialog(DefinedTermDropAdapterE4.TERM_TYPE_ERROR_TITLE, this, DefinedTermDropAdapterE4.TERM_TYPE_ERROR_MESSAGE);
		            continue;
		        }
		    }
		    else if(droppedObject instanceof TermDto){
		        droppedType = ((TermDto)droppedObject).getTermType();
		        if(droppedType!=null && !droppedType.equals(targetType) && !droppedType.isKindOf(targetType)){
                    MessagingUtils.warningDialog(DefinedTermDropAdapterE4.TERM_TYPE_ERROR_TITLE, this, DefinedTermDropAdapterE4.TERM_TYPE_ERROR_MESSAGE);
                    continue;
                }
		    }
		    if(droppedObject instanceof TermNodeDto){
		        if (((TermNodeDto)droppedObject).getUuid() == null){
		            if(StoreUtil.promptCheckIsDirty(editor)){
		                return false;
		            }
		        }
		    	TermNodeDto droppedNode = (TermNodeDto) droppedObject;
		        TermNodeDto oldParent = editor.getNodeDtoForUuid(droppedNode.getParentUuid());
		        int currentPosition = oldParent.getIndex(droppedNode);

		        if(currentPosition<position && target.equals(oldParent)){
		            position -= 1;
		        }
		        MoveFeatureOperation operation = new MoveFeatureOperation(droppedNode.getUuid(), target.getUuid(), droppedNode.getTerm().getUuid(), position, getCurrentOperation(), editor, editor);

		        Object o = oldParent.getChildren().remove(currentPosition);
		        target.getChildNodes().add(position, droppedNode);
		        droppedNode.setParentUuid(target.getUuid());
		        editor.setDirty();
		        editor.addOperation(operation);
		    }
		    if(droppedObject instanceof TermNode){
                if (((TermNode)droppedObject).getUuid() == null){
                    if(StoreUtil.promptCheckIsDirty(editor)){
                        return false;
                    }
                }
                TermNode droppedNode = (TermNode) droppedObject;
                TermNode oldParent = droppedNode.getParent();
                int currentPosition = oldParent.getIndex(droppedNode);

                if(currentPosition<position && target.equals(oldParent)){
                    position -= 1;
                }
                MoveFeatureOperation operation = new MoveFeatureOperation(droppedNode.getUuid(), target.getUuid(), droppedNode.getTerm().getUuid(), position, getCurrentOperation(), editor, editor);

                oldParent.removeChild(droppedNode);
                target.addChild(droppedNode, position);

                editor.setDirty();
                editor.addOperation(operation);
            }
		    else if(droppedObject instanceof Feature){
		        Feature droppedFeature = (Feature) droppedObject;
		        AddFeatureOperation operation = new AddFeatureOperation(
		                droppedFeature.getUuid(),
		                target.getUuid(), position, editor, editor);
		        target.addChild(droppedFeature, position);
		        editor.setDirty();
		        editor.addOperation(operation);
		    } else if(droppedObject instanceof OntologyTermWrapper){
		        OntologyTermWrapper wrapper = (OntologyTermWrapper)droppedObject;
		        TermVocabulary vocabulary = wrapper.getTermVocabulary();
		        if(vocabulary==null){
		            vocabulary =  TermVocabularySelectionDialog.select(
		                    Messages.FeatureTreeDropAdapter_CHOOSE_VOC, viewer.getControl().getShell(), null);

		            if(vocabulary instanceof OrderedTermVocabulary){
		                MessagingUtils.warningDialog(Messages.FeatureTreeDropAdapter_IMPORT_NOT_POSSIBLE, this,
		                        Messages.FeatureTreeDropAdapter_ORDER_VOC_NOT_POSSIBLE);
		                return false;
		            }
                    if(vocabulary == null){
                        return false;
                    }

		        }


		        AddOntologyTermOperation operation = new AddOntologyTermOperation(wrapper, target.getUuid(), vocabulary, editor, editor);

		        editor.setDirty();
		        editor.addOperation(operation);

		    }
		}
		viewer.refresh();

		return true;
	}

    @Override
    public boolean validateDrop(Object target, int operation, TransferData transferData) {
        boolean
        isSupported = TermNodeTransfer.getInstance().isSupportedType(transferData);
        isSupported |= TermTransfer.getInstance().isSupportedType(transferData);
        isSupported |= LocalSelectionTransfer.getTransfer().isSupportedType(transferData);
        isSupported &= getViewer().getInput()!=null;
        if(target instanceof TermTree && getCurrentLocation()!=ViewerDropAdapter.LOCATION_ON){
            isSupported = false;
        }
        return isSupported;
    }



}