/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.term.IHasTermType;
import eu.etaxonomy.cdm.model.term.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.editor.definedterm.TermTransfer;
import eu.etaxonomy.taxeditor.editor.definedterm.e4.DefinedTermDropAdapterE4;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.termtree.TermNodeDtoTransfer;
import eu.etaxonomy.taxeditor.termtree.e4.operation.AddFeatureOperation;
import eu.etaxonomy.taxeditor.termtree.e4.operation.AddOntologyTermOperation;
import eu.etaxonomy.taxeditor.termtree.e4.operation.MoveFeatureOperation;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TermVocabularySelectionDialog;
import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.OntologyTermWrapper;

public class TermTreeDtoDropAdapter extends ViewerDropAdapter {

    protected ITermTreeEditor editor;
    private UISynchronize sync;

    public TermTreeDtoDropAdapter(ITermTreeEditor editor, Viewer viewer, UISynchronize sync) {
		super(viewer);
		this.editor = editor;
		this.sync = sync;
	}

	@Override
	public boolean performDrop(Object data) {
		Object currentTarget = getCurrentTarget();
		TermNodeDto target = null;
		if(currentTarget instanceof TermTreeDto){
		    target = ((TermTreeDto) currentTarget).getRoot();
		}
		else if(currentTarget instanceof TermNodeDto){
		    target = (TermNodeDto) currentTarget;
		}

		int position = 0;
		int location = getCurrentLocation();
		UUID parentUuid = target.getParentUuid();
		if(parentUuid!=null){
		    TermNodeDto parent = editor.getNodeDtoForUuid(parentUuid);
		    if (location == LOCATION_BEFORE) {
		        position = Math.max(0, parent.getIndex(target));
		        target = parent;
		    }

		    if (location == LOCATION_AFTER) {
		        position = parent.getIndex(target)+1;
		        target = parent;
		    }
		}

		Collection<Object> droppedObjects = Collections.emptyList();
		if(data instanceof Object[]){
		    droppedObjects = Arrays.asList((Object[])data);
		}
		else if(data instanceof IStructuredSelection){
		    droppedObjects = ((IStructuredSelection) data).toList();
		}
		TreeViewer viewer = (TreeViewer) getViewer();

		// cannot drop a feature node onto itself
		for (Object droppedObject : droppedObjects) {
		    if(droppedObject==null){
		        MessagingUtils.warningDialog(DefinedTermDropAdapterE4.MOVE_FAILED, this.getClass(),
		                Messages.FeatureTreeDropAdapter_MOVE_FAILED_SAVE_MESSAGE);
		        return false;
		    }
			if (droppedObject.equals(target)) {
				return false;
			}

		}
		for (Object droppedObject : droppedObjects) {
		    //check term type compatibility
		    TermType targetType = target.getType();

		    TermType droppedType = null;
		    if(droppedObject instanceof IHasTermType){
		        droppedType = ((IHasTermType)droppedObject).getTermType();
		        if(droppedType!=null && !droppedType.equals(targetType) && !droppedType.isKindOf(targetType)){
		            MessagingUtils.warningDialog(DefinedTermDropAdapterE4.TERM_TYPE_ERROR_TITLE, this, DefinedTermDropAdapterE4.TERM_TYPE_ERROR_MESSAGE);
		            continue;
		        }
		    }
		    else if(droppedObject instanceof TermDto){
		        droppedType = ((TermDto)droppedObject).getTermType();
		        if(droppedType!=null && !droppedType.equals(targetType) && !droppedType.isKindOf(targetType)){
                    MessagingUtils.warningDialog(DefinedTermDropAdapterE4.TERM_TYPE_ERROR_TITLE, this, DefinedTermDropAdapterE4.TERM_TYPE_ERROR_MESSAGE);
                    continue;
                }
		    }
		    if(droppedObject instanceof TermNodeDto){
		        if (((TermNodeDto)droppedObject).getUuid() == null || target.getUuid() == null){
		            MessagingUtils.warningDialog("The new imported node needs to be saved first", this, "Newly created nodes can not be moved or used as parent without saving");
		            return false;
		        }
		    	TermNodeDto droppedNode = editor.getNodeDtoForUuid(((TermNodeDto) droppedObject).getUuid());
		    	ITermTreeEditor<?> sourceEditor = editor;
		    	if (droppedNode == null){
		            //the node does not come from the same editor, need to find the source editor
		            TermType type = ((TermNodeDto) droppedObject).getType();
		            TermEditorInput termEditorInput = new TermEditorInput(type);

		            EPartService partService = TaxeditorStorePlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getService(EPartService.class);
		            List<MPart> alreadyOpenedEditors = partService.getParts().stream()
		                    .filter(part->part.getObject()!=null && part.getObject() instanceof ITermTreeEditor)
		                    .filter(part->((ITermTreeEditor<?>)part.getObject()).getTermType().equals(type))
		                    .collect(Collectors.toList());

		            if(!alreadyOpenedEditors.isEmpty()){
		                //there should never be more than one already opened editor
		                //so we just open the first
		                sourceEditor = (ITermTreeEditor<?>)alreadyOpenedEditors.iterator().next().getObject();
		                droppedNode = sourceEditor.getNodeDtoForUuid(((TermNodeDto) droppedObject).getUuid());
		            }
		        }
		        if (droppedNode == null){
		            return false;
		        }

		    	TermNodeDto oldParent = sourceEditor.getNodeDtoForUuid(droppedNode.getParentUuid());
		        boolean isCircle = checkCircle(droppedNode, target);
		        if (isCircle || droppedNode.equals(target)){
		            return false;
		        }
		        int currentPosition = oldParent.getIndex(droppedNode);

		        if(currentPosition<position && target.equals(oldParent)){
		            position -= 1;
		        }
		        MoveFeatureOperation operation = new MoveFeatureOperation(droppedNode.getUuid(), droppedNode.getTerm().getUuid(), target.getUuid(), position, getCurrentOperation(), editor, editor);

		        Object o = oldParent.getChildren().remove(currentPosition);

		        target.getChildren().add(position, droppedNode);
		        droppedNode.setParentUuid(target.getUuid());
		        editor.setNodeDtoForUuid(droppedNode);
		        editor.setNodeDtoForUuid(target);
		        editor.setNodeDtoForUuid(oldParent);
		        editor.setDirty();
		        editor.addOperation(operation);
		    }
		    else if(droppedObject instanceof Feature){
		        Feature droppedFeature = (Feature) droppedObject;
		        AddFeatureOperation operation = new AddFeatureOperation(
		                droppedFeature.getUuid(),
		                target.getUuid(), position, editor, editor);
		        editor.setDirty();
		        editor.addOperation(operation);
		    }
		    else if(droppedObject instanceof TermDto){
		        TermDto termDto = (TermDto) droppedObject;
		        // check if target feature tree has the same type as the dropped term
		        if(!termDto.getTermType().equals(targetType)){
		            MessagingUtils.warningDialog(DefinedTermDropAdapterE4.TERM_TYPE_ERROR_TITLE, this, DefinedTermDropAdapterE4.TERM_TYPE_ERROR_MESSAGE);
		            continue;
		        }
		        AddFeatureOperation operation = new AddFeatureOperation(
		                termDto.getUuid(),
		                target.getUuid(), position, editor, editor);
		        TermNodeDto newDto = new TermNodeDto(termDto, target, position, target.getTree(), null, null, null);

                editor.setDirty();
		        editor.addOperation(operation);
		    }
		    else if(droppedObject instanceof OntologyTermWrapper){
		        OntologyTermWrapper wrapper = (OntologyTermWrapper)droppedObject;
		        TermVocabulary<?> vocabulary = wrapper.getTermVocabulary();
		        if(vocabulary==null){
		            vocabulary =  TermVocabularySelectionDialog.select(
		                    Messages.FeatureTreeDropAdapter_CHOOSE_VOC, viewer.getControl().getShell(), null);

		            if(vocabulary instanceof OrderedTermVocabulary){
		                MessagingUtils.warningDialog(Messages.FeatureTreeDropAdapter_IMPORT_NOT_POSSIBLE, this,
		                        Messages.FeatureTreeDropAdapter_ORDER_VOC_NOT_POSSIBLE);
		                return false;
		            }
                    if(vocabulary == null){
                        return false;
                    }
		        }

		        AddOntologyTermOperation operation = new AddOntologyTermOperation(wrapper, target.getUuid(), vocabulary, editor, editor);

		        editor.setDirty();
		        editor.addOperation(operation);

		    }
		}
		editor.getViewer().refresh();
		return true;
	}

    private boolean checkCircle(TermNodeDto droppedNode, TermNodeDto newParent) {
        boolean result = false;
       if (droppedNode.getChildren().contains(newParent) ){
           return true;
       }
       if (newParent.getParentUuid() != null){
           result = checkCircle(droppedNode, editor.getNodeDtoForUuid(newParent.getParentUuid()));

       }
       return result;
    }

    @Override
    public boolean validateDrop(Object target, int operation, TransferData transferData) {
        boolean
        isSupported = TermNodeDtoTransfer.getInstance().isSupportedType(transferData);
        isSupported |= TermTransfer.getInstance().isSupportedType(transferData);
        isSupported |= LocalSelectionTransfer.getTransfer().isSupportedType(transferData);
        isSupported &= getViewer().getInput()!=null;
        DropTargetEvent event = this.getCurrentEvent();

        TermTreeDto rootTree = null;
        if (getSelectedObject() instanceof TermNodeDto){
            TermNodeDto selectedNode = (TermNodeDto)getSelectedObject();
            rootTree = selectedNode.getTree();
        }
        if(target instanceof TermTreeDto && getCurrentLocation()!=ViewerDropAdapter.LOCATION_ON){
            return false;
        }
        if (target == null){
            return false;
        }
        if (target instanceof TermNodeDto && ((TermNodeDto)target).getTree().isFlat() && getCurrentLocation() == LOCATION_ON){
            return false;
        }
        TermDto term = null;
        if (getSelectedObject() instanceof TermNodeDto){
            term = ((TermNodeDto)getSelectedObject()).getTerm();
            isSupported = checkDuplication(target, rootTree, term);
        }

        if (term == null){
            TermTransfer transfer = TermTransfer.getInstance();
            if (transfer.getSelection() != null){
               TermDto dto = transfer.getSelection();
               isSupported = checkDuplication(target, rootTree, dto);

            }
        }
        return isSupported;
    }

    private boolean checkDuplication(Object target, TermTreeDto rootTree, TermDto term) {
        TermTreeDto targetTree = null;
        if (target instanceof TermNodeDto){
            targetTree = ((TermNodeDto)target).getTree();
        }else if (target instanceof TermTreeDto){
            targetTree = (TermTreeDto)target;
        }
        if (targetTree != null && !targetTree.isAllowDuplicate()){
            for (TermDto dto: targetTree.getTerms()){
                if (dto != null && dto.getUuid().equals(term.getUuid()) && !(rootTree != null && targetTree.equals(rootTree))){
                    return false;
                }
            }
        }
        return true;
    }
}