/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.osgi.service.component.annotations.ConfigurationPolicy;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.ITermTreeService;
import eu.etaxonomy.cdm.api.service.description.AggregationMode;
import eu.etaxonomy.cdm.api.service.description.AggregationSourceMode;
import eu.etaxonomy.cdm.api.service.description.DistributionAggregation;
import eu.etaxonomy.cdm.api.service.description.DistributionAggregationConfiguration;
import eu.etaxonomy.cdm.filter.LogicFilter;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.OriginalSourceTypeComparator;
import eu.etaxonomy.taxeditor.ui.dialog.SuperAreaSelectionWizard;

/**
 * @author k.luther
 * @since 15.11.2019
 */
public class DistributionAggregationWizardPage
        extends AggregationConfigurationWizardPage<DistributionAggregation,DistributionAggregationConfiguration>
        implements Listener, SelectionListener{

    private Button buttonSuperArea;
    private Combo comboAreaLevel;
    private Combo comboStatusOrder;

    private Combo comboClassificationSelection;
    private List<Classification> classifications;
    private Classification selectedClassification;

    

    private Object[] checkedElements;

    public DistributionAggregationWizardPage(DistributionAggregationConfiguration configurator) {
        super(Messages.DistributionAggregationWizardPage_TITLE);
        this.configurator = configurator;

        this.setDescription(Messages.DistributionAggregationWizardPage_DESCRIPTION);
    }

    @Override
    public void createControl(Composite parent) {
        final Composite composite = new Composite(parent, SWT.NULL);

        String taxonStr = ""; //$NON-NLS-1$
        int count = configurator.getTaxonNodeFilter().getSubtreeFilter().size();
        boolean areaChecked = configurator.getAggregationModes().contains(AggregationMode.WithinTaxon);
        boolean toParentChecked = configurator.getAggregationModes().contains(AggregationMode.ToParent);
        for (LogicFilter<TaxonNode> filter: configurator.getTaxonNodeFilter().getSubtreeFilter()){
            subTreeNode = CdmApplicationState.getCurrentAppConfig().getTaxonNodeService().load(filter.getUuid());
            count--;
            if (subTreeNode.hasTaxon()){
                taxonStr += subTreeNode.getTaxon().getName().getTitleCache();
                if (count>0){
                    taxonStr += ", "; //$NON-NLS-1$
                }
            }
        }
        if (StringUtils.isNotBlank(taxonStr)){
            checkUseSelectedSubtree= new Button(composite, SWT.RADIO);
            checkUseSelectedSubtree.setText(Messages.AggregationWizardPage_SUBTREE + " "+ taxonStr);
            checkUseSelectedSubtree.addListener(SWT.Selection, new Listener() {
                @Override
                public void handleEvent(Event e) {
                    Button b = (Button) e.widget;
                    GridData data = (GridData)  comboClassificationSelection.getLayoutData();
                    data.exclude = b.getSelection();
                    comboClassificationSelection.setEnabled(!data.exclude);

                    updateHigherRankCombo();
               }
            });

            checkUseSelectedTaxonNode= new Button(composite,  SWT.RADIO);
            checkUseSelectedTaxonNode.setText(Messages.AggregationWizardPage_SINGLE_TAXON + " " + taxonStr + " "+Messages.AggregationWizardPage_WITHOUT_CHILDREN);
            checkUseSelectedTaxonNode.addListener(SWT.Selection, new Listener() {
                @Override
                public void handleEvent(Event e) {
                    Button b = (Button) e.widget;
                    GridData data = (GridData)  comboClassificationSelection.getLayoutData();
                    data.exclude = b.getSelection();
                    comboClassificationSelection.setEnabled(!data.exclude);
                    updateHigherRankCombo();
                }
            });
        }
        checkUseHigherLevel= new Button(composite, SWT.RADIO);
        checkUseHigherLevel.setText(Messages.DistributionAggregationWizardPage_CLASSIFICATION);
        checkUseHigherLevel.addListener(SWT.Selection, new Listener() {
           @Override
           public void handleEvent(Event e) {
               Button b = (Button) e.widget;
               GridData data = (GridData)  comboClassificationSelection.getLayoutData();
               data.exclude = b.getSelection();
               comboClassificationSelection.setEnabled(data.exclude);
               updateHigherRankCombo();
           }
        });

        GridLayoutFactory.fillDefaults();

        if (subTreeNode!= null){
            selectedClassification = subTreeNode.getClassification();
        }
        GridData gridDataClassificationCombo = new GridData();
        gridDataClassificationCombo = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
        gridDataClassificationCombo.horizontalIndent = 5;

        comboClassificationSelection = new Combo(composite, SWT.BORDER| SWT.READ_ONLY);
        comboClassificationSelection.setLayoutData(gridDataClassificationCombo);
        classifications = CdmStore.getService(IClassificationService.class).list(null, null, null, null, null);
        Collections.sort(classifications, new Comparator<Classification>() {

            @Override
            public int compare(Classification o1, Classification o2) {
                if (o1.equals(o2)){
                    return 0;
                }
                int result = o1.getTitleCache().compareTo(o2.getTitleCache());
                if (result == 0){
                    return o1.getUuid().compareTo(o2.getUuid());
                }
                return result;
            }
        });
        for(Classification tree : classifications){
            comboClassificationSelection.add(tree.getName().getText(), classifications.indexOf(tree));
        }

        comboClassificationSelection.select(classifications.indexOf(selectedClassification));
        comboClassificationSelection.addSelectionListener(this);

        if (StringUtils.isNotBlank(taxonStr)){
            checkUseSelectedSubtree.setSelection(true);
            comboClassificationSelection.setEnabled(false);
        }else{
            checkUseHigherLevel.setSelection(true);
        }
        Label separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 2;
        gridData.verticalIndent = 5;
        separator.setLayoutData(gridData);

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 1;

        composite.setLayout(gridLayout);
        final Composite control = new Composite(composite, SWT.NULL);
        GridLayout gridLayoutControl = new GridLayout();
        gridLayoutControl.numColumns = 2;
        control.setLayout(gridLayoutControl);
        control.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));

        Label higherRankLabel = new Label(control, SWT.NULL);
        higherRankLabel.setText(Messages.DistributionAggregationWizardPage_HIGHEST_RANK);

        comboHigherRank = new Combo(control, SWT.BORDER | SWT.READ_ONLY);
        updateHigherRankCombo();
        setConfigValueToRankCombo(comboHigherRank);
        comboHigherRank.addListener(SWT.Selection, this);

        Label lowerRankLabel = new Label(control, SWT.NULL);
        lowerRankLabel.setText(Messages.DistributionAggregationWizardPage_LOWEST_RANK);
        comboLowerRank = new Combo(control, SWT.BORDER | SWT.READ_ONLY);
        updateLowerRankCombo();
        
        setConfigValueToRankCombo(comboLowerRank);
        comboLowerRank.addListener(SWT.Selection, this);

        separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
        gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 2;
        gridData.verticalIndent = 5;
        separator.setLayoutData(gridData);

        Label aggregationModeLabel = new Label(control, SWT.NULL);
        aggregationModeLabel.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);
        aggregationModeLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_AGGR_MODE);
        aggregationModeViewer = CheckboxTableViewer.newCheckList(control, SWT.BORDER | SWT.SINGLE);
        aggregationModeViewer.setContentProvider(new ArrayContentProvider());
        aggregationModeViewer.setLabelProvider(new LabelProvider(){
            @Override
            public String getText(Object element){
                if (element instanceof AggregationMode){
                    if (((AggregationMode)element).equals(AggregationMode.ToParent)){
                        return Messages.DistributionAggregationWizardPage_CHILD_PARENT;
                    }
                    if (((AggregationMode)element).equals(AggregationMode.WithinTaxon)){
                        return Messages.DistributionAggregationWizardPage_AREA;
                    }
                }
                return null;
            }

        });
        List<AggregationMode> aggregationModeList = new ArrayList<>();
        for (AggregationMode mode: AggregationMode.values()){
            aggregationModeList.add(mode);
        }

        aggregationModeViewer.setInput(aggregationModeList);
        
        for (AggregationMode mode: configurator.getAggregationModes()){
            if (mode instanceof AggregationMode){
            	aggregationModeViewer.setChecked(mode, true);
            }
        }
        

        aggregationModeViewer.addCheckStateListener(new ICheckStateListener(){
            @Override
            public void checkStateChanged(    CheckStateChangedEvent event){
                Object[] checked =aggregationModeViewer.getCheckedElements();
                boolean areachecked = false;
                boolean toParentChecked = false;
                for (int i = 0; i<checked.length;i++){
                    if (checked[i] instanceof AggregationMode){
                        if (((AggregationMode)checked[i]).equals(AggregationMode.WithinTaxon)){
                            areachecked = true;
                        }
                        if (((AggregationMode)checked[i]).equals(AggregationMode.ToParent)){
                            toParentChecked = true;
                        }
                    }
                }
                comboAreaLevel.setEnabled(areachecked);
                buttonSuperArea.setEnabled(areachecked);
                comboSourceModeWithinTaxon.setEnabled(areachecked);
                comboSourceModeChildParent.setEnabled(toParentChecked);
                AggregationSourceMode areaMode = (AggregationSourceMode)comboSourceModeWithinTaxon.getData(comboSourceModeWithinTaxon.getText());
                AggregationSourceMode taxonMode = (AggregationSourceMode)comboSourceModeChildParent.getData(comboSourceModeChildParent.getText());
                sourceTypeViewer.getTable().setEnabled(((areachecked && (areaMode.equals(AggregationSourceMode.ALL) || areaMode.equals(AggregationSourceMode.ALL_SAMEVALUE) ) )|| toParentChecked && (taxonMode.equals(AggregationSourceMode.ALL) || taxonMode.equals(AggregationSourceMode.ALL_SAMEVALUE) )) );
                getWizard().getContainer().updateButtons();
              }
        });
        Label areaLevelLabel = new Label(control, SWT.NULL);
        areaLevelLabel.setText(Messages.DistributionAggregationWizardPage_AREA_LEVEL);
        areaLevelLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_AREA_LEVEL);
        comboAreaLevel = new Combo(control,  SWT.BORDER | SWT.READ_ONLY);
        List<NamedAreaLevel> namedAreaLevels = CdmStore.getTermManager().getAllTerms(TermType.NamedAreaLevel, null);
        for (NamedAreaLevel level: namedAreaLevels) {
            comboAreaLevel.add(level.getLabel());
            comboAreaLevel.setData(level.getLabel(), level);
        }
        comboAreaLevel.addSelectionListener(this);
        comboAreaLevel.setEnabled(areaChecked);
        
        
        GridData gridData2 = new GridData();
        gridData2.horizontalSpan = 2;
        gridData2.horizontalAlignment = SWT.LEFT;

        buttonSuperArea = new Button(control, SWT.PUSH );
        buttonSuperArea.setLayoutData(gridData2);
        buttonSuperArea.setEnabled(configurator.getAggregationModes().contains(AggregationMode.WithinTaxon));
        buttonSuperArea.setText(Messages.DistributionAggregationWizardPage_SELECT_AREA);
        buttonSuperArea.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_AREA_SELECTION);
        buttonSuperArea.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                SuperAreaSelectionWizard availableDistributionWizard = new SuperAreaSelectionWizard(configurator, (NamedAreaLevel)comboAreaLevel.getData(comboAreaLevel.getText()));
                WizardDialog dialog = new WizardDialog(AbstractUtility.getShell(),
                        availableDistributionWizard);
                int open = dialog.open();
            }
        });

        Label sourceModeLabel = new Label(control, SWT.NULL);

        sourceModeLabel.setText(Messages.DistributionAggregationWizardPage_SOURCEMODE_CHILD_PARENT);
        sourceModeLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCEMODE_CHILD_PARENT);
        comboSourceModeChildParent = new Combo(control,  SWT.BORDER| SWT.READ_ONLY);
        comboSourceModeChildParent.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);

        for (AggregationSourceMode mode :AggregationSourceMode.values()){
            comboSourceModeChildParent.add(mode.getLabel());
            comboSourceModeChildParent.setData(mode.getLabel(), mode);
        }

        comboSourceModeChildParent.addSelectionListener(this);
        comboSourceModeChildParent.setEnabled(toParentChecked);
        int index = 0;
        if (configurator.getToParentSourceMode() == null ){
        	comboSourceModeChildParent.select(index);
        }else{
        	for (String text:comboSourceModeChildParent.getItems()){
        		if (comboSourceModeChildParent.getData(text).equals(configurator.getToParentSourceMode())){
        			comboSourceModeChildParent.select(index);
        		}
        		index++;
        	}
        }

        Label sourceModeLabelSuperArea = new Label(control, SWT.NULL);
        sourceModeLabelSuperArea.setText(Messages.DistributionAggregationWizardPage_SOURCE_MODE_AREA);
        sourceModeLabelSuperArea.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCEMODE_AREA);
        comboSourceModeWithinTaxon = new Combo(control,  SWT.BORDER| SWT.READ_ONLY);
        comboSourceModeWithinTaxon.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);

        for (AggregationSourceMode mode :AggregationSourceMode.values()){
            comboSourceModeWithinTaxon.add(mode.getLabel());
            comboSourceModeWithinTaxon.setData(mode.getLabel(), mode);
        }

        comboSourceModeWithinTaxon.addSelectionListener(this);
        comboSourceModeWithinTaxon.setEnabled(configurator.getAggregationModes().contains(AggregationMode.WithinTaxon));
        
        index = 0;
        if (configurator.getWithinTaxonSourceMode() == null ){
        	comboSourceModeWithinTaxon.select(index);
        }else{
        	for (String text:comboSourceModeWithinTaxon.getItems()){
        		if (comboSourceModeWithinTaxon.getData(text).equals(configurator.getWithinTaxonSourceMode())){
        			comboSourceModeWithinTaxon.select(index);
        		}
        		index++;
        	}
        }

        

        Label sourceTypeLabel = new Label(control, SWT.NULL);
        sourceTypeLabel.setText(Messages.DistributionAggregationWizardPage_SOURCE_TYPE);
        sourceTypeLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCE_TYPE);
        sourceTypeViewer = CheckboxTableViewer.newCheckList(control, SWT.BORDER | SWT.SINGLE);
        sourceTypeViewer.setContentProvider(new ArrayContentProvider());

        sourceTypeViewer.addCheckStateListener(new ICheckStateListener(){
            @Override
            public void checkStateChanged(    CheckStateChangedEvent event){
                checkedElements=sourceTypeViewer.getCheckedElements();
                getWizard().getContainer().updateButtons();
              }
        });

        List<String> typeStrings = new ArrayList<>();
        OriginalSourceType[] typeArray = OriginalSourceType.values();
        Arrays.sort(typeArray, new OriginalSourceTypeComparator(null));
        Arrays.stream(typeArray).forEach(p ->typeMap.put(p.getLabel(), p));
        Arrays.stream(typeArray).forEach(p ->typeStrings.add(p.getLabel()));
        sourceTypeViewer.setInput(typeStrings);
        for (OriginalSourceType sourceType: configurator.getAggregatingSourceTypes()){
        	sourceTypeViewer.setChecked(sourceType.getLabel(), true);
        }
//        sourceTypeViewer.setChecked(OriginalSourceType.PrimaryTaxonomicSource.getLabel(), true);
//        sourceTypeViewer.getTable().setEnabled(false);
        AggregationSourceMode taxonMode = configurator.getToParentSourceMode();
        AggregationSourceMode areaMode = configurator.getWithinTaxonSourceMode();
        sourceTypeViewer.getTable().setEnabled(((areaChecked && (areaMode.equals(AggregationSourceMode.ALL) || areaMode.equals(AggregationSourceMode.ALL_SAMEVALUE) ) )|| toParentChecked && (taxonMode.equals(AggregationSourceMode.ALL) || taxonMode.equals(AggregationSourceMode.ALL_SAMEVALUE) )) );

        //aggregation mode

        Label selectStatusLabel = new Label(control, SWT.NULL);
        selectStatusLabel.setText(Messages.DistributionAggregationWizardPage_STATUS_ORDER);
        comboStatusOrder = new Combo(control, SWT.PUSH );
        comboStatusOrder.add(Messages.DistributionAggregationWizardPage_DEFAULT);
        comboStatusOrder.setData(Messages.DistributionAggregationWizardPage_DEFAULT, null);
        for (TermTree<?> tree: CdmStore.getService(ITermTreeService.class).list(TermType.PresenceAbsenceTerm, null, 0, null,null)){
            comboStatusOrder.add(tree.getTitleCache());
            comboStatusOrder.setData(tree.getTitleCache(), tree);
        }

        comboStatusOrder.addSelectionListener(this);
        index = 0;
        if (configurator.getStatusOrder() == null ){
        	comboStatusOrder.select(index);
        }else{
        	for (String text:comboStatusOrder.getItems()){
        		if (text.equals(configurator.getStatusOrder().getTitleCache())){
        			comboStatusOrder.select(index);
        		}
        		index++;
        	}
        }

        checkIncludeUnpublishedTaxa = new Button(composite,  SWT.CHECK);
        checkIncludeUnpublishedTaxa.setText(Messages.DistributionAggregationWizardPage_EXPORT_UNPUBLISHED);
        checkIncludeUnpublishedTaxa.setSelection(true);

        GridLayoutFactory.fillDefaults();

        setControl(composite);
    }

	
    @Override
    public void widgetSelected(SelectionEvent e) {

        if (e.getSource().equals(comboSourceModeChildParent) || e.getSource().equals(comboSourceModeWithinTaxon)){
            AggregationSourceMode modeChild = (AggregationSourceMode)comboSourceModeChildParent.getData(comboSourceModeChildParent.getText());
            AggregationSourceMode modeArea = (AggregationSourceMode)comboSourceModeWithinTaxon.getData(comboSourceModeWithinTaxon.getText());
            if ((comboSourceModeChildParent.isEnabled() && (modeChild != null && modeChild.equals(AggregationSourceMode.ALL) || modeChild.equals(AggregationSourceMode.ALL_SAMEVALUE))) ||(comboSourceModeWithinTaxon.isEnabled() && (modeArea != null && modeArea.equals(AggregationSourceMode.ALL) || modeArea.equals(AggregationSourceMode.ALL_SAMEVALUE)))){
                sourceTypeViewer.getTable().setEnabled(true);
            }else{
                sourceTypeViewer.getTable().setEnabled(false);
            }
        }

        if (e.getSource().equals(comboAreaLevel)){
            buttonSuperArea.setEnabled(StringUtils.isNotBlank(comboAreaLevel.getText()));
        }
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub
    }

    @Override
    public void handleEvent(Event event) {
        if (event.widget.equals(comboHigherRank)){
            updateLowerRankCombo();
        }
        if (event.widget.equals(comboClassificationSelection)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
        if (event.widget.equals(this.checkUseHigherLevel)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
        if (event.widget.equals(this.checkUseSelectedSubtree)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
        if (event.widget.equals(this.checkUseSelectedTaxonNode)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
    }

   public TermTree getStatusOrder(){
       if (StringUtils.isNotBlank(comboStatusOrder.getText())){
           return (TermTree<?>)comboStatusOrder.getData(comboStatusOrder.getText());
       }
       return null;
   }

   public boolean useUnpublishedData(){
       return checkIncludeUnpublishedTaxa.getSelection();
   }

    public Classification getSelectedClassification() {
        return selectedClassification;
    }

    public TaxonNode getSubTreeNode() {
        return subTreeNode;
    }
}