/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.description.AggregationMode;
import eu.etaxonomy.cdm.api.service.description.AggregationSourceMode;
import eu.etaxonomy.cdm.api.service.description.AggregationType;
import eu.etaxonomy.cdm.api.service.description.StructuredDescriptionAggregation;
import eu.etaxonomy.cdm.api.service.description.StructuredDescriptionAggregationConfiguration;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.DescriptiveDataSetBaseDto;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @since 11.12.2019
 */
public class StructuredDescriptionAggregationConfigurationWizardPage
        extends AggregationConfigurationWizardPage<StructuredDescriptionAggregation,StructuredDescriptionAggregationConfiguration>
        implements Listener {

    protected CheckboxTableViewer subTreeSelectionViewer;
    private List<TaxonNodeDto> selectedNodes;

    protected Button checkIncludeDefaultDescriptions;
    protected Button checkIncludeLiteratureDescriptions;
    protected DescriptiveDataSetBaseDto ddsDTO;


    private Label selectSubtreeLabel;

    protected StructuredDescriptionAggregationConfigurationWizardPage(StructuredDescriptionAggregationConfiguration configurator, List<TaxonNodeDto> nodes) {
        super(Messages.DistributionAggregationWizardPage_TITLE);
        this.configurator = configurator;
        this.setDescription(Messages.DistributionAggregationWizardPage_DESCRIPTION);
        selectedNodes = nodes;
        ddsDTO = CdmApplicationState.getCurrentAppConfig().getDescriptiveDataSetService().getDescriptiveDataSetDtoByUuid(configurator.getDatasetUuid());

    }

    @Override
    public void createControl(Composite parent) {

        final Composite control = new Composite(parent, SWT.NULL);
        GridLayout gridLayoutControl = new GridLayout();
        gridLayoutControl.numColumns = 2;
        control.setLayout(gridLayoutControl);
        control.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));

        //taxon node filter
        GridData gridDataRadioComposite = new GridData();
        gridDataRadioComposite.horizontalSpan = 2;
        GridLayout gridLayoutRadioComposite = new GridLayout();
        gridLayoutRadioComposite.numColumns = 1;
        Composite radioComposite = new Composite(control, SWT.NULL);
        radioComposite.setLayoutData(gridDataRadioComposite);
        radioComposite.setLayout(gridLayoutRadioComposite);

        boolean hasTaxonFilter = !configurator.getTaxonNodeFilter().getSubtreeFilter().isEmpty();

        //   ... from descriptive data set
        checkUseHigherLevel = new Button(radioComposite,  SWT.RADIO);
        checkUseHigherLevel.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_ALL_SUBTREES);
        checkUseHigherLevel.addListener(SWT.Selection, new Listener() {
           @Override
           public void handleEvent(Event e) {
               subTreeSelectionViewer.getTable().setEnabled(!checkUseHigherLevel.getSelection());
               selectSubtreeLabel.setEnabled(!checkUseHigherLevel.getSelection());
               updateHigherRankCombo();
               if (checkUseHigherLevel.getSelection()){
                   setPageComplete(true);
               }
           }
        });


        //   ... from selected subtree
        checkUseSelectedSubtree= new Button(radioComposite, SWT.RADIO);
        checkUseSelectedSubtree.setText(Messages.AggregationWizardPage_SUBTREE);
        checkUseSelectedSubtree.addListener(SWT.Selection, new Listener() {
            @Override
            public void handleEvent(Event e) {
                subTreeSelectionViewer.getTable().setEnabled(!checkUseHigherLevel.getSelection());
                selectSubtreeLabel.setEnabled(checkUseSelectedSubtree.getSelection() || checkUseSelectedTaxonNode.getSelection());
                if (checkUseSelectedSubtree.getSelection() && subTreeSelectionViewer.getCheckedElements().length == 0){
                    setPageComplete(false);
                }else{
                    setPageComplete(true);
                }
           }
        });

        //    ... from selected taxon
        checkUseSelectedTaxonNode= new Button(radioComposite, SWT.RADIO);
        checkUseSelectedTaxonNode.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_SELECTED_TAXA_ONLY);
        checkUseSelectedTaxonNode.addListener(SWT.Selection, new Listener() {
            @Override
            public void handleEvent(Event e) {
                subTreeSelectionViewer.getTable().setEnabled(!checkUseHigherLevel.getSelection());
                selectSubtreeLabel.setEnabled(!checkUseHigherLevel.getSelection());
                if (checkUseSelectedTaxonNode.getSelection() && subTreeSelectionViewer.getCheckedElements().length == 0){
                    setPageComplete(false);
                }else{
                    setPageComplete(true);
                }
            }
        });
        checkUseSelectedSubtree.setSelection(hasTaxonFilter);

        //subtree selection
        selectSubtreeLabel = new Label(control, SWT.NULL);
        selectSubtreeLabel.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_SUBTREE);
        selectSubtreeLabel.setToolTipText(Messages.StructuredDescriptionAggregationWizardPage_TOOLTIP_SELECT_SUBTREE);

        GridData gridData = new GridData();
        gridData.horizontalIndent = 25;
        gridData.verticalAlignment = SWT.TOP;
        selectSubtreeLabel.setLayoutData(gridData);

        subTreeSelectionViewer = CheckboxTableViewer.newCheckList(control, SWT.BORDER | SWT.SINGLE);
        subTreeSelectionViewer.setContentProvider(new ArrayContentProvider());
        subTreeSelectionViewer.setLabelProvider(new LabelProvider(){
            @Override
            public String getText(Object element){
                if (element instanceof TaxonNodeDto){
                   return ((TaxonNodeDto)element).getNameTitleCache();
                }else if (element instanceof TaxonNode){
                    return ((TaxonNode)element).getTaxon().getName().getTitleCache();
                }
                return null;
            }
        });

        if (selectedNodes == null || selectedNodes.isEmpty()){
            List<TaxonNode> nodeList;
            List<UUID> uuidList = new ArrayList<>();
            configurator.getTaxonNodeFilter().getSubtreeFilter().stream().forEach(p ->uuidList.add(p.getUuid()));
            List<String> propertyPaths = new ArrayList<>();
            propertyPaths.add("taxon.name");
            nodeList = CdmApplicationState.getCurrentAppConfig().getTaxonNodeService().load(uuidList, propertyPaths);
            subTreeSelectionViewer.setInput(nodeList);
        }else{
            subTreeSelectionViewer.setInput(selectedNodes);
        }
        subTreeSelectionViewer.setAllChecked(true);
        subTreeSelectionViewer.addCheckStateListener(new ICheckStateListener(){
            @Override
            public void checkStateChanged(    CheckStateChangedEvent event){
                Object[] checked =subTreeSelectionViewer.getCheckedElements();
                if (checked.length == 0){
                   setPageComplete(false);
                }else{
                    setPageComplete(true);
                }
            }
        });

//        GridLayoutFactory.fillDefaults();

        //scope (published taxa, literature + default descriptions)
        Label scopeLabel = new Label(control, SWT.NULL);
        scopeLabel.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_DEFINE_SCOPE);

        Composite scopeComposite = new Composite(control, SWT.NULL);
        GridLayout gridDataScopeComposite = new GridLayout();
        gridDataScopeComposite.numColumns = 1;
        scopeComposite.setLayout(gridDataScopeComposite);

//        GridLayoutFactory.fillDefaults();

        checkIncludeUnpublishedTaxa = new Button(scopeComposite, SWT.CHECK);
        checkIncludeUnpublishedTaxa.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_UNPUBLISHED_TAXA);
        checkIncludeUnpublishedTaxa.setSelection(true);

        checkIncludeDefaultDescriptions = new Button(scopeComposite, SWT.CHECK);
        checkIncludeDefaultDescriptions.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_DEFAULT_DESCRIPTION);
        checkIncludeDefaultDescriptions.setEnabled(false);

        checkIncludeLiteratureDescriptions = new Button(scopeComposite, SWT.CHECK);
        checkIncludeLiteratureDescriptions.setText(Messages.StructuredDescriptionAggregationWizardPage_SELECT_LITERATURE_DESCRIPTION);
        checkIncludeLiteratureDescriptions.setSelection(configurator.isIncludeLiterature());
        GridLayoutFactory.fillDefaults();

        //ranks
        Label higherRankLabel = new Label(control, SWT.NULL);
        higherRankLabel.setText(Messages.DistributionAggregationWizardPage_HIGHEST_RANK);
        comboHigherRank = new Combo(control, SWT.BORDER | SWT.READ_ONLY);
        updateHigherRankCombo();
        setConfigValueToRankCombo(comboHigherRank, ddsDTO.getMaxRank() != null? ddsDTO.getMaxRank().getUuid(): null);
        comboHigherRank.addListener(SWT.Selection, this);

        Label lowerRankLabel = new Label(control, SWT.NULL);
        lowerRankLabel.setText(Messages.DistributionAggregationWizardPage_LOWEST_RANK);
        comboLowerRank = new Combo(control, SWT.BORDER | SWT.READ_ONLY);
        updateLowerRankCombo();
        setConfigValueToRankCombo(comboLowerRank, ddsDTO.getMinRank() != null? ddsDTO.getMinRank().getUuid(): null);
        comboLowerRank.addListener(SWT.Selection, this);

        //aggregation mode viewer
        Label aggregationModeLabel = new Label(control, SWT.NULL);
        aggregationModeLabel.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);
        aggregationModeLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_AGGR_MODE);

        aggregationModeViewer = CheckboxTableViewer.newCheckList(control, SWT.BORDER | SWT.SINGLE);
        aggregationModeViewer.setContentProvider(new ArrayContentProvider());
        aggregationModeViewer.setLabelProvider(new LabelProvider(){
            @Override
            public String getText(Object element){
                if (element instanceof AggregationMode){
                	if (((AggregationMode)element).getKey().equals("INTAX")){
                		return Messages.StructuredDescriptionAggregationWizardPage_SOURCEMODE_WITHIN_TAXON;
                	}else if (((AggregationMode)element).getKey().equals("TOPAR")){
                		return Messages.StructuredDescriptionAggregationWizardPage_SOURCEMODE_CHILD_PARENT;
                	}
                   return ((AggregationMode)element).getLabel();
                }
                return null;
            }
        });
        List<AggregationMode> aggregationModeList = AggregationMode.list(AggregationType.StructuredDescription);
        aggregationModeViewer.setInput(aggregationModeList);
        aggregationModeViewer.setCheckedElements(configurator.getAggregationModes().toArray());
        aggregationModeViewer.addCheckStateListener(new ICheckStateListener(){
            @Override
            public void checkStateChanged(    CheckStateChangedEvent event){
                Object[] checked =aggregationModeViewer.getCheckedElements();
                boolean withinChecked = false;
                boolean toParentChecked = false;
                for (int i = 0; i<checked.length;i++){
                    if (checked[i] instanceof AggregationMode){
                        if (((AggregationMode)checked[i]).equals(AggregationMode.WithinTaxon)){
                            withinChecked = true;
                        }
                        if (((AggregationMode)checked[i]).equals(AggregationMode.ToParent)){
                            toParentChecked = true;
                        }
                    }
                }

                comboSourceModeChildParent.setEnabled(toParentChecked);
                comboSourceModeWithinTaxon.setEnabled(withinChecked);
                getWizard().getContainer().updateButtons();
            }
        });

        //source mode within taxon
        Label sourceModeLabelWithin = new Label(control, SWT.NULL);
        sourceModeLabelWithin.setText(Messages.StructuredDescriptionAggregationWizardPage_SOURCEMODE_WITHIN_TAXON);
        sourceModeLabelWithin.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCEMODE_WITHIN_TAXON);
        comboSourceModeWithinTaxon = new Combo(control,  SWT.BORDER| SWT.READ_ONLY);
        comboSourceModeWithinTaxon.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);

        int index = 0;
        int defaultSelectionIndex = 0;
        for (AggregationSourceMode mode :AggregationSourceMode.list(AggregationMode.WithinTaxon, AggregationType.StructuredDescription) ){
            comboSourceModeWithinTaxon.add(mode.getLabel());
            comboSourceModeWithinTaxon.setData(mode.getLabel(), mode);
            if (configurator.getWithinTaxonSourceMode().equals(mode)){
                defaultSelectionIndex = index;
            }
            index++;
        }

        comboSourceModeWithinTaxon.setEnabled(configurator.getAggregationModes().contains(AggregationMode.WithinTaxon));
        comboSourceModeWithinTaxon.select(defaultSelectionIndex);
        comboSourceModeWithinTaxon.addListener(SWT.Selection, this);

        //source mode child parent
        Label sourceModeChildParentLabel = new Label(control, SWT.NULL);
        sourceModeChildParentLabel.setText(Messages.StructuredDescriptionAggregationWizardPage_SOURCEMODE_CHILD_PARENT);
        sourceModeChildParentLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCEMODE_CHILD_PARENT);
        comboSourceModeChildParent = new Combo(control,  SWT.BORDER| SWT.READ_ONLY);
        comboSourceModeChildParent.setText(Messages.DistributionAggregationWizardPage_AGGREGATION_MODE);

        index = 0;
        defaultSelectionIndex = 0;
        for (AggregationSourceMode mode :AggregationSourceMode.list(AggregationMode.ToParent, AggregationType.StructuredDescription) ){
            comboSourceModeChildParent.add(mode.getLabel());
            comboSourceModeChildParent.setData(mode.getLabel(), mode);
            if (configurator.getToParentSourceMode().equals(mode)){
                defaultSelectionIndex = index;
            }
            index++;
        }

        comboSourceModeChildParent.setEnabled(configurator.getAggregationModes().contains(AggregationMode.ToParent));
        comboSourceModeChildParent.select(defaultSelectionIndex);
        comboSourceModeChildParent.addListener(SWT.Selection, this);
        GridLayoutFactory.fillDefaults();

        /*source type
        //TODO: distinguish within taxon and parent child source types (#9805)
        Label sourceTypeLabel = new Label(control, SWT.NULL);
        sourceTypeLabel.setText(Messages.DistributionAggregationWizardPage_SOURCE_TYPE);
        sourceTypeLabel.setToolTipText(Messages.DistributionAggregationWizardPage_TOOLTIP_SOURCE_TYPE);
        sourceTypeViewer = CheckboxTableViewer.newCheckList(control, SWT.BORDER | SWT.SINGLE);
        sourceTypeViewer.setContentProvider(new ArrayContentProvider());

        sourceTypeViewer.addCheckStateListener(new ICheckStateListener(){
            @Override
            public void checkStateChanged(    CheckStateChangedEvent event){
//                checkedElements=sourceTypeViewer.getCheckedElements();
                getWizard().getContainer().updateButtons();
              }
        });

        List<String> typeStrings = new ArrayList<>();
        OriginalSourceType[] typeArray = OriginalSourceType.values();
        Arrays.sort(typeArray, new OriginalSourceTypeComparator(null));
        Arrays.stream(typeArray).forEach(p ->typeMap.put(p.getLabel(), p));
        Arrays.stream(typeArray).forEach(p ->typeStrings.add(p.getLabel()));
        sourceTypeViewer.setInput(typeStrings);
        sourceTypeViewer.setChecked(OriginalSourceType.PrimaryMediaSource.getLabel(), true);
        sourceTypeViewer.setChecked(OriginalSourceType.PrimaryTaxonomicSource.getLabel(), true);
        sourceTypeViewer.getTable().setEnabled(false);
        */
        setControl(control);
    }

    @Override
    public void handleEvent(Event event) {
        if (event.widget.equals(comboHigherRank)){
            updateLowerRankCombo();
        }

        if (event.widget.equals(this.checkUseHigherLevel)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
        if (event.widget.equals(this.checkUseSelectedSubtree)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }
        if (event.widget.equals(this.checkUseSelectedTaxonNode)){
            updateHigherRankCombo();
            updateLowerRankCombo();
        }

        if (event.widget.equals(this.comboSourceModeChildParent) || event.widget.equals(this.comboSourceModeWithinTaxon)){
//            if (getSourceModeChildParent().equals(AggregationSourceMode.NONE) && getSourceModeWithinTaxon().equals(AggregationSourceMode.NONE)){
//                sourceTypeViewer.getTable().setEnabled(false);
//            }else {
//                sourceTypeViewer.getTable().setEnabled(true);
//            }
        }
    }

    public List<TaxonNode> getSelectedTaxonNodes(){

        List<TaxonNode> result = new ArrayList<>();
        for (Object o: subTreeSelectionViewer.getCheckedElements()){
            if (o instanceof TaxonNode){
                result.add((TaxonNode)o);
            }
        }
        return result;
    }

    protected void setConfigValueToRankCombo(Combo rankCombo, UUID rankUuid) {
		int index = 0;

       if (rankUuid == null){
       		rankCombo.select(index);
       }else{
	        Rank rank = null;
	        for (String item: rankCombo.getItems()){
	        	if (rankCombo.getData(item) instanceof Rank){
	        		rank = (Rank) rankCombo.getData(item);
	        		if (rank.getUuid().equals(rankUuid)){
	        			rankCombo.select(index);
	        			break;
	        		}
	        	}
	        	index++;
	        }
       }
	}
}