/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.dto.EntityDTOBase;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * Abstract AbstractFilteredCdmResourceSelectionDialog class.
 *
 * @author n.hoffmann
 * @created 04.06.2009
 */
public abstract class AbstractFilteredCdmResourceSelectionDialog<T extends ICdmBase>
        extends SearchDialog<T> {

    protected List<UuidAndTitleCache<T>> model;
	private final Set<T> transientCdmObjects = new HashSet<>();
	private final String settings;
	protected final Integer limitOfInitialElements = null;

	protected Set<UUID> cdmBaseToBeFiltered;

	protected Job searchJob;

	protected AbstractFilteredCdmResourceSelectionDialog(Shell shell,
	        String title, boolean multi, String settings, Set<T> objectsToBeFiltered) {
	    super(shell, title);

	    setMessage(Messages.SearchDialog_patternLabel);
        this.settings = settings;
        if (objectsToBeFiltered != null){
            this.cdmBaseToBeFiltered = new HashSet<>();
            objectsToBeFiltered.forEach(filter->this.cdmBaseToBeFiltered.add(filter.getUuid()));
        }
        Cursor cursor = null;
        if (shell != null){
        	cursor = shell.getCursor();
        	shell.setCursor(shell.getDisplay().getSystemCursor(SWT.CURSOR_WAIT));
        }
        
        init();
        if (shell != null){   
        	shell.setCursor(cursor);
        }
        setListLabelProvider(createListLabelProvider());

	}

	protected AbstractFilteredCdmResourceSelectionDialog(Shell shell,
	        String title, boolean multi, String settings) {
        this(shell, title, multi, settings, (Set<T>)null);
	}

	protected AbstractFilteredCdmResourceSelectionDialog(Shell shell,
	        String title, boolean multi, String settings, T objectToBeFiltered) {
	    this(shell, title, multi, settings, objectToBeFiltered!=null?Collections.singleton(objectToBeFiltered):null);
	}

	/**
	 * By default, we are returning the standard list label provider
	 *
	 * Override in subclasses if you want different behavior
	 *
	 * @return
	 */
	protected ILabelProvider createDetailsLabelProvider() {
		return createListLabelProvider();
	}

	protected ILabelProvider createListLabelProvider() {
		return new FilteredCdmResourceLabelProvider();
	}

	/**
	 * Override in subclasses.
	 * Will run before initModel()
	 */
	protected void init() {}

	/**
	 * <p>getSelectionFromDialog</p>
	 *
	 * @param dialog a {@link eu.etaxonomy.taxeditor.ui.dialog.selection.AbstractFilteredCdmResourceSelectionDialog} object.
	 * @param <TYPE> a TYPE object.
	 * @return a TYPE object.
	 */
	protected static <TYPE extends CdmBase> TYPE getSelectionFromDialog(AbstractFilteredCdmResourceSelectionDialog<TYPE> dialog) {
	    UuidAndTitleCache<?> result = getUuidAndTitleCacheSelectionFromDialog(dialog);
	    if (result != null){
	        return dialog.getCdmObjectByUuid(result.getUuid());
	    } else {
            return null;
        }
	}

	/**
     * getSelectionFromDialog
     */
    protected static <S extends ICdmBase> UuidAndTitleCache<S> getUuidAndTitleCacheSelectionFromDialog(
            AbstractFilteredCdmResourceSelectionDialog<S> dialog) {
        if (dialog == null){
            return null;
        }
        int result = dialog.open();

        if (result == Window.CANCEL) {
            return null;
        }

        UuidAndTitleCache<S> uuid = dialog.getSelectedUuidAndTitleCache();

        return uuid;
    }

	/**
	 * Check if object was created during the life of this dialog. If not,
	 * retrieve it from the CdmStore.
	 */
	protected T getCdmObjectByUuid(UUID cdmUuid) {
		for (T cdmObject : transientCdmObjects) {
			if (cdmObject.getUuid().equals(cdmUuid)) {
				return cdmObject;
			}
		}
		return getPersistentObject(cdmUuid);
	}

	/**
	 * <p>getPersistentObject</p>
	 *
	 * @param uuid a {@link java.util.UUID} object.
	 * @return a T object.
	 */
	abstract protected T getPersistentObject(UUID uuid);

	/**
	 * <p>isObjectTransient</p>
	 *
	 * @param cdmObject a T object.
	 * @return a boolean.
	 */
	protected boolean isObjectTransient(T cdmObject) {
		return (getPersistentObject(cdmObject.getUuid()) == null);
	}

	/**
	 * <p>getTitle</p>
	 *
	 * @param cdmObject a T object.
	 * @return a {@link java.lang.String} object.
	 */
	protected String getTitle(T cdmObject) {
		if(cdmObject == null){
			return "";
		}

		if (cdmObject instanceof IIdentifiableEntity) {
			return ((IIdentifiableEntity) cdmObject).getTitleCache();
		}

		throw new IllegalArgumentException("Generic method only" +
				" supports cdmObject of type IIdentifiableEntity." +
				" Please implement specific method in subclass.");
	}

	/**
	 * Set the filter input to the Agent's title cache
	 *
	 * @param cdmObject a T object.
	 */
	protected void setPattern(T cdmObject) {
		String pattern = getTitle(cdmObject);
		getSearchField().setText(pattern);
		
	}

	@Override
    protected void fillContentProvider(IProgressMonitor progressMonitor)
		 {
		try {
		    if (model == null){
		        model = new ArrayList<UuidAndTitleCache<T>>();
		    }
			if(model != null){
			    if (progressMonitor != null){
			        progressMonitor.beginTask("Looking for entities", model.size());
			    }
			    sort();

			    contentProvider.reset();
				Iterator<UuidAndTitleCache<T>> iterator = model.iterator();
				UuidAndTitleCache<T> element;
				while(iterator.hasNext()){
				    element = iterator.next();

				    if (cdmBaseToBeFiltered == null || !cdmBaseToBeFiltered.contains(element.getUuid())){
				        contentProvider.add(element);
				    }
				    if (progressMonitor != null){
    					if (progressMonitor.isCanceled()) {
    						return;
    					}
    					progressMonitor.worked(1);
				    }
				}
				this.refresh();
			}else{

				MessagingUtils.warn(getClass(), "Model for Filtered Selection is null:" + this.getClass().getSimpleName());
			}
		}
		finally {
		    if (progressMonitor != null) {
                progressMonitor.done();
            }
		}
	}

    protected void sort() {
        Collections.sort(model, getItemsComparator());
    }

	protected IDialogSettings getDialogSettings() {
		IDialogSettings settings = TaxeditorStorePlugin.getDefault().getDialogSettings().getSection(getSettings());

		if (settings == null) {
			settings = TaxeditorStorePlugin.getDefault().getDialogSettings().addNewSection(getSettings());
		}
		return settings;
	}

	@Override
    protected Comparator<UuidAndTitleCache<T>> getItemsComparator() {
		return new Comparator<UuidAndTitleCache<T>>() {
			@Override
			public int compare(UuidAndTitleCache<T> entity1,
					UuidAndTitleCache<T> entity2) {
				Collator collator = Collator.getInstance();
				if (entity1 == entity2){
				    return 0;
				}else if (entity1 == null){
				    return -1;
				}else if (entity2 == null){
				    return 1;
				}else if (entity1.getUuid().equals(entity2.getUuid())){
                    return 0;
                }else if (entity1.getTitleCache() == null && entity2.getTitleCache() != null){
				    return -1;
				}else if (entity2.getTitleCache() == null){
				    return 1;
				}
				int result = collator.compare(entity1.getTitleCache(), entity2.getTitleCache());
				if (result == 0){
				    result = entity1.getUuid().compareTo(entity2.getUuid());
				}
				return result;
			}
		};
	}

	/**
	 * getSelectedUuidAndTitleCache
	 */
	protected UuidAndTitleCache<T> getSelectedUuidAndTitleCache() {
		Object result = getResult();
		if (result instanceof UuidAndTitleCache){
		    return (UuidAndTitleCache<T>) result;
		}
		return null;
	}

    private Object getResult() {
        StructuredSelection selection = getCurrentSelection();
        if (selection == null){
            return null;
        }
        return selection.getFirstElement();
    }

    /**
	 * <p>Getter for the field <code>settings</code>.</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getSettings()  {
		if(settings == null){
			throw new IllegalStateException("No SETTINGS set.");
		}
		return settings;
	}

	/**
	 * <p>getNewWizardLinkText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	protected abstract String[] getNewWizardText();

	/**
	 * <p>getNewEntityWizard</p>
	 * @param parameter
	 * @return a {@link eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard} object.
	 */
	protected abstract AbstractNewEntityWizard<T> getNewEntityWizard(String parameter);

	public class FilteredCdmResourceLabelProvider extends LabelProvider {
		@Override
		public String getText(Object element) {
			if (element == null) {
				return null;
			}
			UuidAndTitleCache<?> uuidAndTitleCache = (UuidAndTitleCache<?>) element;
			String titleCache = uuidAndTitleCache.getTitleCache();
			if(PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSelectionDialog.getKey())){
			    titleCache += " ["+uuidAndTitleCache.getId()+"]";
			}
			if (element instanceof EntityDTOBase){
			    titleCache += "(" + ((IdentifiedEntityDTO)element).getIdentifier().getTypeLabel() +": " + ((IdentifiedEntityDTO)element).getIdentifier().getIdentifier() + ")";
			}

            return titleCache;
		}
	}

//	@Override
//    protected Control createExtendedContentArea(Composite parent) {
//		String newWizardLinkText = getNewWizardLinkText();
////        if(newWizardLinkText != null){
////            newButton1 = this.createButton(this.getShell(), new_id, newWizardLinkText, false);
////
////            newButton1.addSelectionListener(getNewWizardLinkSelectionListener());
////			return newButton1;
////		}
//		return null;
//	}

	@Override
    protected void createButtonsForButtonBar(Composite parent) {
	    String[] newButtonText = getNewWizardText();

	    if (newButtonText!= null){
	        this.newButton1 = createButton(parent, this.new_id, newButtonText[0], false);
	        newButton1.addSelectionListener(getNewWizardButtonSelectionListener());

	        if (newButtonText.length > 1){
	            newButton2 = createButton(parent, this.new_id2, newButtonText[1], false);
	            newButton2.addSelectionListener(getNewWizardButtonSelectionListener());
	        }
	    }
	    Button space = createButton(parent, this.space_id, " ", false);
	    space.setEnabled(false);
	    space.setVisible(false);
	    GridData gridData = new GridData();
        gridData.grabExcessHorizontalSpace = false;
        gridData.widthHint = 3;
	    space.setLayoutData(gridData);
	    GridLayout gridLayout = new GridLayout();
	    gridLayout.makeColumnsEqualWidth= false;
	    if (newButtonText != null){
	    	gridLayout.numColumns=newButtonText.length+2;
	    }else{
	    	gridLayout.numColumns=2;
	    }
	    parent.setLayout(gridLayout);

        super.createButtonsForButtonBar(parent);
        super.getButton(IDialogConstants.OK_ID).setEnabled(false);
    }

	protected SelectionListener getNewWizardButtonSelectionListener(){
		return new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
			    Object source = e.getSource();
			    String text = null;
			    if (source instanceof Button){
			        Button sourceButton = (Button) source;
			        text = sourceButton.getText();
			    }
			    AbstractNewEntityWizard<?> wizard = getNewEntityWizard(text);
			    if(wizard != null){
			        if (wizard.getEntity() == null){
			            wizard.init(null, null);
			        }
			        if(wizard.getEntity() != null) {
			            WizardDialog dialog = new WizardDialog(getShell(), wizard);
			            int status = dialog.open();

			            if (status == IStatus.OK) {

			                T entity = (T) wizard.getEntity();
			                refresh();
			                setPattern(entity);

//			                if (getConversationHolder() != null){
//			                    getConversationHolder().bind();
//			                }
			            }
			            //FIXME : Need to make sure this is a stable fix (ticket 3822)
//			            if (getConversationHolder() != null){
//			                getConversationHolder().commit();
//			            }
			        }
			    }
			}
		};
	}

	/**
	 * Don't want to add for example a taxon or synonym to itself
	 * so filter the list to remove the taxon in question
	 * (<code>cdmBaseToBeFiltered</code>)
	 * so it is not available in the filtered list.
	 */
	private void filterExcludedObjects() {
		if (model != null && cdmBaseToBeFiltered != null) {

			UuidAndTitleCache<?> uuidAndTitleCacheToRemove = null;

			for (UuidAndTitleCache<?> uuidAndTitleCache : model){
				if (cdmBaseToBeFiltered != null && cdmBaseToBeFiltered.contains(uuidAndTitleCache.getUuid())) {
					uuidAndTitleCacheToRemove = uuidAndTitleCache;
				}
			}
			model.remove(uuidAndTitleCacheToRemove);
		}
	}

	@Override
	void createFilterButton(Composite searchAndFilter){
	    //as default no filter button available
	}

    @Override
    protected void search() {
        Control control =getSearchField();
        String pattern = null;
        if (control != null){
            pattern = ((Text)control).getText();
            if (pattern.equals("*") || pattern.equals("?") || (this instanceof CollectorSelectionDialog && StringUtils.isBlank(pattern) && ((CollectorSelectionDialog)this).collectorTeam != null)){
                callService(null);
            }else if (StringUtils.isNotBlank(pattern)){
//                callService(CdmUtils.replaceNonWordCharacters(pattern, "."));
                callService(pattern);
            }
            fillContentProvider(null);
        }

//        if (pattern.equals("?")){
//            model = CdmStore.getService(INameService.class).getUuidAndTitleCache(null, null);
//        }else if (pattern != null){
//            model = CdmStore.getService(INameService.class).getUuidAndTitleCache(limitOfInitialElements, pattern);
//        }
    }

    protected abstract void callService(String pattern);

}
