/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.List;
import java.util.UUID;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.dto.EntityDTOBase;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.persistence.dto.TeamOrPersonUuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewPersonWizard;
import eu.etaxonomy.taxeditor.newWizard.NewTeamWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * FilteredAgentSelectionDialog class.
 *
 * @author n.hoffmann
 * @created Sep 10, 2009
 */
public class AgentSelectionDialog<T extends AgentBase>
        extends AbstractFilteredCdmResourceSelectionDialog<T> {

	protected static final String PERSON = "New Person";
	protected static final String TEAM = "New Team";

	protected boolean selectTeamMember;

	/**
	 * select
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.agent.AgentBase} object.
	 * @return a {@link eu.etaxonomy.cdm.model.agent.AgentBase} object.
	 */
	public static <S extends AgentBase> S select(Shell shell,
	        S entity, boolean selectTeamMember) {
		AgentSelectionDialog<S> dialog = new AgentSelectionDialog<>(shell,
				"Choose Agent", false, AgentSelectionDialog.class.getCanonicalName(), entity, selectTeamMember);
		return getSelectionFromDialog(dialog);
	}

	/**
	 * <p>Constructor for FilteredAgentSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param title a {@link java.lang.String} object.
	 * @param agent a {@link eu.etaxonomy.cdm.model.agent.AgentBase} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 */
	protected AgentSelectionDialog(Shell shell,
	        String title, boolean multi, String settings, T agent, boolean selectTeamMember) {
		super(shell, title, multi, settings, agent);
		this.selectTeamMember = selectTeamMember;
	}

	public class DetailsLabelProvider extends LabelProvider {
		@Override
        public String getText(Object element) {
		    T agent = getCdmObjectByUuid(((UuidAndTitleCache<AgentBase>) element).getUuid());
			if (agent instanceof INomenclaturalAuthor) {
				return "Nomenclatural title: '" + ((INomenclaturalAuthor) agent).getNomenclaturalTitleCache() + "'";
			} else {
				return "'" + agent.getTitleCache() + "' is not a nomenclatural author.";
			}
		}
	}

	@Override
	protected T getPersistentObject(UUID cdmUuid) {
		return (T)CdmStore.getService(IAgentService.class).load(cdmUuid);
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		if(TEAM.equals(parameter)){
			return new NewTeamWizard();
		}
		else if(PERSON.equals(parameter)){
			return new NewPersonWizard();
		}
		else{
			throw new IllegalArgumentException("Could not determine the desired wizard.");
		}
	}

	@Override
	protected String[] getNewWizardText() {
	    String[] result;
		if (selectTeamMember){
		    result = new String[1];
		    result[0] = PERSON;
		}else{
		    result = new String[2];
            result[0] = PERSON;
            result[1] = TEAM;
		}
		return result;
	}

	@Override
	protected SelectionListener getNewWizardButtonSelectionListener() {
		return super.getNewWizardButtonSelectionListener();
	}

    @Override
    protected void callService(String pattern) {
        Class<T> clazz = (Class)AgentBase.class;
        if (selectTeamMember){
            clazz = (Class)Person.class;
        }

        model = (List)CdmStore.getService(IAgentService.class).getTeamOrPersonUuidAndTitleCache(clazz, limitOfInitialElements, pattern);
    }
    @Override
    protected ILabelProvider createListLabelProvider() {
        return new AgentLabelProvider();
    }

    public class AgentLabelProvider extends LabelProvider {
        @Override
        public String getText(Object element) {
            if (element == null) {
                return null;
            }
            UuidAndTitleCache<?> uuidAndTitleCache = (UuidAndTitleCache<?>) element;
            String titleCache = uuidAndTitleCache.getTitleCache();
            String abbrevTitleCache = uuidAndTitleCache.getAbbrevTitleCache();
            String collectorTitleCache = null;
            if (element instanceof TeamOrPersonUuidAndTitleCache){
                collectorTitleCache = ((TeamOrPersonUuidAndTitleCache)element).getCollectorTitleCache();
            }
            if (!titleCache.equals(abbrevTitleCache) && abbrevTitleCache != null){
                titleCache += " - " + abbrevTitleCache;
            }
            if (collectorTitleCache != null && !uuidAndTitleCache.getTitleCache().equals(collectorTitleCache) && !abbrevTitleCache.equals(collectorTitleCache)){
                titleCache += " - " + collectorTitleCache;
            }
            if(PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSelectionDialog.getKey())){
                titleCache += " ["+uuidAndTitleCache.getId()+"]";
            }
            if (element instanceof EntityDTOBase){
                titleCache += "(" + ((IdentifiedEntityDTO)element).getIdentifier().getTypeLabel() +": " + ((IdentifiedEntityDTO)element).getIdentifier().getIdentifier() + ")";
            }

            return titleCache;
        }
    }
}