/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.element;

import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TypedListener;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.forms.widgets.ToggleHyperlink;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * <p>
 * Abstract super class for a {@link Section} GUI element that visualizes a CDM
 * entity, manages a conversation and listens to selections
 * </p>
 *
 * @param <ENTITY> A CDM entity which should be visualized by this section.
 *
 * @author n.hoffmann
 * @created Feb 22, 2010
 */
//TODO shouldn't ENTITY be bound with super class ICdmBase for example (AbstractFormSection<ENTITY extends ICdmBase>)?
public abstract class AbstractFormSection<ENTITY>
        extends Section
        implements ISelectionChangedListener, IEntityElement<ENTITY>, IConversationEnabled {

    /**
     * The default number of columns in detail sections
     */
    public static final int DEFAULT_NUM_COLUMNS = 2;

	private ISelectionProvider selectionProvider;

	private ENTITY entity;

	private final Set<ICdmFormElement> elements = new HashSet<>();

	protected CdmFormFactory formFactory;

	private List<IPropertyChangeListener> propertyChangeListeners;

	private ICdmFormElement parentElement;

	private Color persistentBackgroundColor;

	/**
	 * Constructor for AbstractFormSection.
	 */
    protected AbstractFormSection(CdmFormFactory formFactory, ICdmFormElement parentElement, int style) {
		super(parentElement.getLayoutComposite(), style);

		this.parentElement = parentElement;

		this.formFactory = formFactory;

		this.setLayoutData(LayoutConstants.FILL());

		Composite client = formFactory.createComposite(this, SWT.WRAP);
		client.setBackgroundMode(SWT.INHERIT_DEFAULT);

		TableWrapLayout layout = LayoutConstants.LAYOUT();
		layout.bottomMargin = 10;
		layout.rightMargin = 5;
		layout.horizontalSpacing = 5;

		client.setLayout(layout);

		this.setClient(client);
	}

	/**
     * Constructor for AbstractFormSection.
     */
    protected AbstractFormSection(CdmFormFactory formFactory, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
        this(formFactory, parentElement, style);
        this.selectionProvider = selectionProvider;
    }

	/**
	 * Getter for the field <code>propertyChangeListeners</code>.
	 */
	@Override
    public List<IPropertyChangeListener> getPropertyChangeListeners() {
		return propertyChangeListeners;
	}

	@Override
    public void setPropertyChangeListeners(
			List<IPropertyChangeListener> propertyChangeListeners) {
		this.propertyChangeListeners = propertyChangeListeners;
	}

	@Override
	public ENTITY getEntity() {
	    return entity;
	}
	public void setEntity(ENTITY entity) {
		this.entity = entity;
	}

	public ToggleHyperlink getToggle() {
		return this.toggle;
	}

	public AbstractFormSection<ENTITY> getSection() {
		return this;
	}

	@Override
    public void firePropertyChangeEvent(CdmPropertyChangeEvent event) {
        Assert.isNotNull(propertyChangeListeners, "No property change listeners.");
        try {
            for (Object listener : propertyChangeListeners) {
                ((IPropertyChangeListener) listener).propertyChange(event);
            }
        } catch (ConcurrentModificationException e) {
            MessagingUtils.warn(getClass(), "ConcurrentModificationException while handling PropertyChangeEvents."
                    + " It seems like this is not critical");
        }
	}

	/**
	 * Fires a {@link CdmPropertyChangeEvent} with the given object as source.
	 *
	 * @param object
	 *            the object on which the property changed
	 */
	public void firePropertyChangeEvent(Object object) {
		firePropertyChangeEvent(object, null);
	}

	/**
	 * Fires a {@link CdmPropertyChangeEvent} with the given object as source
	 * also containing the originating event
	 *
	 * @param object
	 *            the object on which the property changed
	 * @param originatingEvent
	 *            the originating event
	 */
	public void firePropertyChangeEvent(Object object,
			PropertyChangeEvent originatingEvent) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(object,
				originatingEvent));
	}

	@Override
	public boolean setFocus() {
		return getClient().setFocus();
	}

	@Override
    public void propertyChange(PropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
	}

	@Override
	public void setBackground(Color color) {
		for (ICdmFormElement element : getElements()) {
			element.setBackground(color);
		}
		if (!getLayoutComposite().isDisposed()){
			getLayoutComposite().setBackground(color);
		}
		super.setBackground(color);
	}

	@Override
	public void setPersistentBackground(Color color) {
		persistentBackgroundColor = color;
		setBackground(color);
	}

	@Override
	public Color getPersistentBackground() {
		return persistentBackgroundColor;
	}

	public void widgetSelected(SelectionEvent e) {
        Widget widget = e.widget;

        if (widget instanceof Control) {
            Control control = (Control) widget;
            if (checkControlAncestryForWidget(control)) {
                if (getEntity() != null) {
                    IStructuredSelection selection = new StructuredSelection(getEntity());
                    if (selectionProvider != null) {
                        selectionProvider.setSelection(selection);
                    }
                }
            }
        }
	}

	private boolean checkControlAncestryForWidget(Control control) {
		if (control.equals(this)) {
			return true;
		} else {
			Control parent = control.getParent();
			if (parent == null) {
				return false;
			} else {
				return checkControlAncestryForWidget(parent);
			}
		}
	}

	@Override
    public void setSelected(boolean selected) {
		if (selected) {
			setBackground(Display.getCurrent().getSystemColor(
					SWT.COLOR_LIST_SELECTION));
		} else {
			setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		}
	}

	@Override
    public void selectionChanged(SelectionChangedEvent event) {
		if (event.getSelection() == CdmFormFactory.EMPTY_SELECTION) {
			return;
		}

		IStructuredSelection selection = (IStructuredSelection) event
				.getSelection();
		setSelected(false);

		Object selectedObject = selection.getFirstElement();

		if (selectedObject != null && selectedObject.equals(getEntity())) {
			setSelected(true);
		}
	}

	public void addSelectionListener(SelectionListener listener) {
		addListener(SWT.Selection, new TypedListener(listener));
	}

	public void removeSelectionListener(SelectionListener listener) {
		removeListener(SWT.Selection, listener);
	}

	@Override
    public void addElement(ICdmFormElement element) {
		elements.add(element);
	}

	protected void removeElement(ICdmFormElement element) {
		elements.remove(element);
	}

	@Override
    public void removeElements() {
		for (Iterator<ICdmFormElement> formElementIterator = getElements().iterator();formElementIterator.hasNext();) {
		    ICdmFormElement childElement = formElementIterator.next();
			// recursion
			childElement.removeElements();

			// unregister selection arbitrator
			if (childElement instanceof ISelectableElement) {
				ISelectableElement selectableElement = (ISelectableElement) childElement;
				if (selectableElement.getSelectionArbitrator() != null) {
					formFactory.destroySelectionArbitrator(selectableElement
							.getSelectionArbitrator());
				}
			}

			// unregister propertyChangeListener
			formFactory.removePropertyChangeListener(childElement);

			// dispose of the controls
			for (Iterator<Control> controlIterator = childElement.getControls().iterator();controlIterator.hasNext();) {
			    Control control = controlIterator.next();
				// we added the layoutComposite of the parental element as the
				// layout composite to this formElement
				// but we do not want to destroy it.
				if (control.equals(childElement.getLayoutComposite())) {
					continue;
				} else {
					control.dispose();
					control = null;
				}
			}
		}

		elements.clear();
	}

	/**
	 * Getter for the field <code>parentElement</code>.
	 */
	@Override
    public ICdmFormElement getParentElement() {
		return parentElement;
	}

	/**
	 * Getter for the field <code>elements</code>.
	 */
	@Override
    public Set<ICdmFormElement> getElements() {
		return elements;
	}

	@Override
    public Set<Control> getControls() {
		Set<Control> controls = new HashSet<>();

		for (Control control : getChildren()) {
			controls.add(control);
		}

		return controls;
	}

	@Override
	public void dispose() {
		removeElements();
		super.dispose();
	}

	@Override
    public Composite getLayoutComposite() {
		return (Composite) getClient();
	}

	@Override
    public boolean containsFormElement(ICdmFormElement formElement) {
		if (formElement == this) {
			return true;
		} else {
			for (ICdmFormElement element : getElements()) {
				boolean contains = element.containsFormElement(formElement);
				if (contains == true) {
					return true;
				}
			}
			return false;
		}
	}

	/**
	 * Getter for the field <code>formFactory</code>.
	 */
	@Override
    public CdmFormFactory getFormFactory() {
		return formFactory;
	}

	@Override
    public void refresh() {
		// empty default implementation
	}

	@Override
    public ConversationHolder getConversationHolder() {
	    if(AbstractUtility.getActivePart() instanceof IConversationEnabled){
            return ((IConversationEnabled) AbstractUtility.getActivePart()).getConversationHolder();
        }
	    if(getParentElement() instanceof RootElement || getParentElement() == null){

	        Object activeEditor = AbstractUtility.getActiveEditor();
	        if(activeEditor instanceof IConversationEnabled){
	            ConversationHolder conversation = ((IConversationEnabled) AbstractUtility.getActiveEditor()).getConversationHolder();
	            return conversation;
	        }
	    }else if(getParentElement() instanceof IConversationEnabled){
	        return ((IConversationEnabled) getParentElement()).getConversationHolder();
	    }
		MessagingUtils.error(getClass(), "Could not get conversation for AbstractFormSection. There is an error in the implementation. There should have been an active editor but it wasn't",
				new IllegalArgumentException());
		return null;

	}

	@Override
    public void update(CdmDataChangeMap changeEvents) {

	}

}
