/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.element;

import java.text.ParseException;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.location.Point;
import eu.etaxonomy.cdm.model.location.ReferenceSystem;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;

/**
 * <p>
 * PointElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Oct 15, 2010
 */
public class PointElement extends AbstractCdmFormElement implements
		IEntityElement<Point>, ISelectable {

	private Point point;

	private final TextWithLabelElement text_latitude;
	private final TextWithLabelElement text_longitude;
	private final NumberWithLabelElement number_errorRadius;
	private final TermComboElement<ReferenceSystem> combo_referenceSystem;

	private final TextWithLabelElement text_latitudeParsed;

	private final TextWithLabelElement text_longitudeParsed;

	/**
	 * <p>
	 * Constructor for PointElement.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param point
	 *            a {@link eu.etaxonomy.cdm.model.location.Point} object.
	 * @param style
	 *            a int.
	 */
	public PointElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, Point point, final int style) {
		super(formFactory, formElement);

		formFactory.addPropertyChangeListener(this);

		text_latitude = formFactory.createTextWithLabelElement(formElement,
				"Latitude", null, style);
		text_latitudeParsed = formFactory.createTextWithLabelElement(
				formElement, "", null, style);
		text_latitudeParsed.setEnabled(false);
		text_longitude = formFactory.createTextWithLabelElement(formElement,
				"Longitude", null, style);
		text_longitudeParsed = formFactory.createTextWithLabelElement(
				formElement, "", null, style);
		text_longitudeParsed.setEnabled(false);
		number_errorRadius = formFactory.createFloatTextWithLabelElement(
				formElement, "Error Radius (m)", null, style);
		combo_referenceSystem = formFactory.createDefinedTermComboElement(TermType.ReferenceSystem,
						formElement, "Reference System", null, style);

		setPoint(point);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement#propertyChange(org
	 * .eclipse.jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}

		boolean propagate = false;

		Object eventSource = event.getSource();
		if (eventSource == text_latitude) {
			try {
			    if (StringUtils.isBlank(text_latitude.getText())){
			        getPoint().setLatitudeByParsing(null);
			        text_latitudeParsed.setText(null);
			        text_latitude
                    .setBackground(getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
			    }else{
    				getPoint().setLatitudeByParsing(text_latitude.getText());
    				text_latitudeParsed.setText(point.getLatitudeSexagesimal()
    						.toString(false));
    				text_latitude
    						.setBackground(getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
			    }
				propagate = true;
			} catch (ParseException e) {
				text_latitude
						.setBackground(getColor(Resources.COLOR_PARSE_ERROR));
			}
		} else if (eventSource == text_longitude) {
			try {
			    if (StringUtils.isBlank(text_longitude.getText())){
                    getPoint().setLongitudeByParsing(null);
                    text_longitudeParsed.setText(null);
                    text_longitude
                    .setBackground(getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
                }else{
    				getPoint().setLongitudeByParsing(text_longitude.getText());
    				text_longitudeParsed.setText(point.getLongitudeSexagesimal()
    						.toString(false));
    				text_longitude
    						.setBackground(getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
                }
				propagate = true;
			} catch (ParseException e) {
				text_longitude
						.setBackground(getColor(Resources.COLOR_PARSE_ERROR));
			}
		} else if (eventSource == number_errorRadius) {
			getPoint().setErrorRadius(number_errorRadius.getInteger());
			propagate = true;
		} else if (eventSource == combo_referenceSystem) {
			getPoint().setReferenceSystem(combo_referenceSystem.getSelection());
			propagate = true;
		}

		if (propagate) {
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
		}
	}

	/** {@inheritDoc} */
	@Override
	public Point getEntity() {
		return getPoint();
	}

	/**
	 * <p>
	 * setEntity
	 * </p>
	 *
	 * @param point
	 *            a {@link eu.etaxonomy.cdm.model.location.Point} object.
	 */
	public void setEntity(Point point) {
		setPoint(point);
	}

	/**
	 * <p>
	 * Setter for the field <code>point</code>.
	 * </p>
	 *
	 * @param point
	 *            the point to set
	 */
	public void setPoint(Point point) {
		this.point = point;
		if (point != null) {
			String latitude = point.getLatitudeSexagesimal() == null ? ""
					: point.getLatitudeSexagesimal().toString(false);
			String longitude = point.getLongitudeSexagesimal() == null ? ""
					: point.getLongitudeSexagesimal().toString(false);
			text_latitude.setText(latitude);
			text_latitudeParsed.setText(latitude);
			text_longitude.setText(longitude);
			text_longitudeParsed.setText(longitude);
			number_errorRadius.setNumber(point.getErrorRadius());
			combo_referenceSystem.setSelection(point.getReferenceSystem());
		}
	}

	/**
	 * <p>
	 * Getter for the field <code>point</code>.
	 * </p>
	 *
	 * @return the point
	 */
	public Point getPoint() {
		if (point == null) {
			point = Point.NewInstance();
		}

		return point;
	}

	@Override
    public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

}
