/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * When the button is pressed, this textfield may be edited.
 *
 * @author n.hoffmann
 * @created Nov 18, 2009
 */
public class ToggleableTextElement
        extends AbstractRelevanceFormElement
        implements SelectionListener, ModifyListener,
        IEnableableFormElement, ISelectable {

	private final Text text_cache;
	private final Button btnToggleProtectTitleCache;
	private boolean state;
	private final Set<SelectionListener> selectionListener = new HashSet<>();
	private final Label label;
	private final Composite container;
	private final CacheRelevance relevance;

	private boolean isEnabled = true;
	//TODO for RL
	public void setVisible(boolean b) {
		text_cache.setVisible(b);
		btnToggleProtectTitleCache.setVisible(b);
		label.setVisible(b);
		container.setVisible(b);
	}

	protected ToggleableTextElement(CdmFormFactory formFactory, ICdmFormElement parentElement,
			String labelString, String initialText, boolean initialState,
			CacheRelevance relevance, int style) {
		super(formFactory, parentElement);

		label = formFactory.createLabel(getLayoutComposite(), labelString, style);
		addControl(label);

		container = formFactory.createComposite(getLayoutComposite(), SWT.WRAP);
		container.setLayout(LayoutConstants.LAYOUT(2, false));
		container.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
		addControl(container);

		text_cache = formFactory.createText(container, initialText, SWT.WRAP | SWT.MULTI);
		addControl(text_cache);
		if (initialState){
		    text_cache.addModifyListener(this);
		}
		text_cache.setLayoutData(LayoutConstants.FILL());

		// Don't accept carriage returns as input
		text_cache.addKeyListener( new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
			    if (!state){
			        return;
			    }
    			if(e.character == SWT.CR) {
                    e.doit = false;
                }
			}
		});

		btnToggleProtectTitleCache = formFactory.createButton(container, "Edit", SWT.TOGGLE);
		btnToggleProtectTitleCache.setText("");
		btnToggleProtectTitleCache.setImage(ImageResources.getImage(ImageResources.LOCK_ICON));
		addControl(btnToggleProtectTitleCache);
		btnToggleProtectTitleCache.addSelectionListener(this);

		this.relevance = relevance;
		setState(initialState);
	}

	public void setText(String text){
		if(text != null){
			// store current caret position
			int caretPosition = text_cache.getCaretPosition();

			StoreUtil.setTextWithoutModifyListeners(text_cache, text != null? text.trim(): text);
//			text_cache.removeModifyListener(this);
//			text_cache.setText(text);
//			text_cache.addModifyListener(this);

			// restore caret position
			text_cache.setSelection(caretPosition);
		}
	}

	public String getText(){
		return text_cache.getText();
	}

	private void setState(boolean state) {
	    if(state){
	        btnToggleProtectTitleCache.setImage(ImageResources.getImage(ImageResources.LOCK_OPEN_ICON));
	        btnToggleProtectTitleCache.setToolTipText(Messages.ToggleableText_ToolTip_open);
	    }
	    else{
	        btnToggleProtectTitleCache.setImage(ImageResources.getImage(ImageResources.LOCK_ICON));
	        btnToggleProtectTitleCache.setToolTipText(Messages.ToggleableText_ToolTip_closed);
	    }
	    setCacheEnabled(state);
		this.state = state;
	}


    public void setCacheEnabled(boolean enabled) {
        text_cache.setEditable(enabled);
        if (enabled){
            text_cache.removeModifyListener(this);
            text_cache.addModifyListener(this);
        }else{
            text_cache.removeModifyListener(this);
        }
       	btnToggleProtectTitleCache.setGrayed(enabled);
		btnToggleProtectTitleCache.setSelection(enabled);
		Color color = enabled ? null : AbstractUtility.getColor(Resources.COLOR_TEXT_DISABLED);

		text_cache.setForeground(color);
	}

    @Override
    public void setEnabled(boolean isEnabled){
        this.isEnabled = isEnabled;
        if (state){
            text_cache.setEditable(isEnabled);
        }
        btnToggleProtectTitleCache.setEnabled(isEnabled);
    }

	@Override
	public boolean isEnabled() {
	    return isEnabled;
	}

	public boolean isCacheEnabled() {
        return btnToggleProtectTitleCache.getSelection();
    }

    public CacheRelevance getRelevance() {
        return this.relevance;
    }

	public boolean getState(){
		return state;
	}

    public Button getBtnToggleProtectTitleCache() {
        return btnToggleProtectTitleCache;
    }

	@Override
    public void widgetSelected(SelectionEvent e) {
	    setState(btnToggleProtectTitleCache.getSelection());
		for(SelectionListener listener : selectionListener){
			listener.widgetSelected(e);
		}
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	public void addSelectionListener(SelectionListener listener){
		selectionListener.add(listener);
	}

	public void removeSelectionListener(SelectionListener listener){
		selectionListener.remove(listener);
	}

	@Override
    public void widgetDefaultSelected(SelectionEvent e) {}

	@Override
    public void modifyText(ModifyEvent e) {
	    if (state){
	        firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	    }
	}

	@Override
    public void updateCacheRelevance() {
		Color color = cacheRelevance().getColor();
		text_cache.setBackground(color);
	}

    @Override
	public void setBackground(Color color) {
		label.setBackground(color);
		container.setBackground(color);
	}

	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

    @Override
    public String toString() {
        if (label != null){
            return CdmUtils.concat("", "ToggleableTextElement[", label.getText(),"]");
        }else{
            return super.toString();
        }
    }

}