/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.mvc.combo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.cdm.model.term.IEnumTerm;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.ui.element.AbstractRelevanceFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.mvc.interfaces.CdmCompositeController;

/**
 * @author pplitzner
 * @date 29.01.2014
 */
public class EnumTermComboController<T extends IEnumTerm<T>>
		extends AbstractRelevanceFormElement
		implements SelectionListener, IEnableableFormElement, ISelectable, CdmCompositeController {

    private static CdmEnumDataHolder<?>[] cdmEnumDataHolders = {
        new CdmEnumDataHolder<NomenclaturalCode>(){

            @Override
            public String getName() {
                return "Nomenclatural Code";
            }

            @Override
            public Collection<NomenclaturalCode> getElements() {
            	Collection<NomenclaturalCode> result = Arrays.asList(NomenclaturalCode.values());
            	result.remove(NomenclaturalCode.ICVCN);
                return  result;
            }

            @Override
            public Class<NomenclaturalCode> getClazz() {
                return NomenclaturalCode.class;
            }

        },
        new CdmEnumDataHolder<ReferenceType>(){

            @Override
            public String getName() {
                return "Reference Type";
            }

            @Override
            public Collection<ReferenceType> getElements() {
                return Arrays.asList(ReferenceType.values());
            }

            @Override
            public Class<ReferenceType> getClazz() {
                return ReferenceType.class;
            }

        },
        new CdmEnumDataHolder<SpecimenOrObservationType>(){

            @Override
            public String getName() {
                return "Specimen / Observation Type";
            }

            @Override
            public Collection<SpecimenOrObservationType> getElements() {
                return Arrays.asList(SpecimenOrObservationType.values());
            }

            @Override
            public Class<SpecimenOrObservationType> getClazz() {
                return SpecimenOrObservationType.class;
            }

        },
        new CdmEnumDataHolder<OriginalSourceType>(){

            @Override
            public String getName() {
                return "Original Source Type";
            }

            @Override
            public Collection<OriginalSourceType> getElements() {
                return Arrays.asList(OriginalSourceType.values());
            }

            @Override
            public Class<OriginalSourceType> getClazz() {
                return OriginalSourceType.class;
            }

        }

    };

    private static final int DEFAULT_VISIBLE_ITEMS = 10;

    protected T selection;

    protected List<T> elementTypeList = new ArrayList<T>();

    protected Combo combo;

    private final Class<T> enumType;

    /**
     * Constructor for AbstractEnumComboElement.
     */
    public EnumTermComboController(Combo combo, CdmFormFactory formFactory,
            ICdmFormElement parentElement, Class<T> enumType, int style) {
        super(formFactory, (Composite)null);
        this.enumType = enumType;
        // create combo
        this.combo = combo;
        addControl(combo);
        setVisibleItemCount(DEFAULT_VISIBLE_ITEMS);

        populateTypes();

        combo.addSelectionListener(this);
    }

    /**
     * populateTypes
     */
    private void populateTypes(){
        Collection<T> elementsForClass = getElementsForClass(enumType);
        // special hierarchical order for specimens
        if(enumType.equals(SpecimenOrObservationType.class)){
            LinkedHashMap<T, String> orderedTerms = AbstractUtility.orderTerms(elementsForClass);
            for(Entry<T, String> keyLabelPair : orderedTerms.entrySet()){
                elementTypeList.add(keyLabelPair.getKey());
                combo.add(keyLabelPair.getValue());
            }
        }
        // default order
        else{
            for(T enumElement : getElementsForClass(enumType)){
                elementTypeList.add(enumElement);
                combo.add(((Enum) enumElement).name());
            }
        }
    }

    public void setSelection(T selection) {
        this.selection = selection;
        combo.select(elementTypeList.indexOf(selection));
    }

    public void addSelectionListener(SelectionListener listener) {
        combo.addSelectionListener(listener);
    }

    public void removeSelectionListener(SelectionListener listener) {
        combo.removeSelectionListener(listener);
    }

    @Override
    public void setSelected(boolean selected) {
        setBackground(selected ? SELECTED : getPersistentBackground());
    }

    public T getSelection() {
        return selection;
    }

    @Override
    public void setEnabled(boolean enabled) {
        combo.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return combo.isEnabled();
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        selection = elementTypeList.get(combo.getSelectionIndex());
    }

    @Override
    public void updateCacheRelevance() {
        Color color = cacheRelevance().getColor();
        combo.setBackground(color);
    }

    @Override
    public void setBackground(Color color) {
//        label.setBackground(color);
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
    }

    public void setVisibleItemCount(int visibleItems){
        combo.setVisibleItemCount(visibleItems);
    }

    private Collection<T> getElementsForClass(Class<T> clazz){
        CdmEnumDataHolder<T> dataHolder = getCdmEnumDataHolderForClass(clazz);
        if (dataHolder != null) {
            return dataHolder.getElements();
        }
        return null;
    }

    private CdmEnumDataHolder<T> getCdmEnumDataHolderForClass(Class<T> clazz){
        for (CdmEnumDataHolder<?> dataHolder : cdmEnumDataHolders) {
            if (dataHolder.getClazz().equals(clazz)){
                return (CdmEnumDataHolder<T>)dataHolder;
            }
        }
        return null;
    }

    private interface CdmEnumDataHolder<T> {
        Class<T> getClazz();
        String getName();
        Collection<T> getElements();
    }
}