/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section;

import java.util.Collection;
import java.util.EnumSet;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.permission.CRUD;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractRelevanceFormElement;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevance;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevanceHelper;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICacheRelevantFormElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 */
public abstract class AbstractCdmDetailElement<T>
		extends AbstractRelevanceFormElement
		implements ICdmDetailElement<T>, Observer{

	private T entity;

	protected boolean enabled = true;

	private LabelElement warnForReferencedObjects;

	protected EnumSet<CRUD> requiredCrud = null;

	public AbstractCdmDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
		// register as property change listener
		formFactory.addPropertyChangeListener(this);
		CdmStore.getLoginManager().addObserver(this);

		getLayoutComposite().getParent().addDisposeListener(
		        e -> CdmStore.getLoginManager().deleteObserver(AbstractCdmDetailElement.this));
	}

	/**
	 * Sets the entity and updates controls
	 */
	@Override
    public void setEntity(T entity) {
        this.entity = entity;
        // the id is always 0 if the entity was not yet saved, so it is new in this case
        if(getEntity() == null || (getEntity() instanceof CdmBase && StoreUtil.getCdmEntity(getEntity()).getId() == 0)) {
            // new entity, not yet saved
            requiredCrud = EnumSet.of(CRUD.CREATE);
        } else {
            requiredCrud = EnumSet.of(CRUD.UPDATE);
        }
        updateContent();
	}

	public void setEntityWithoutUpdate(T entity){
	    this.entity = entity;
        // the id is always 0 if the entity was not yet saved, so it is new in this case
        if(getEntity() == null || (getEntity() instanceof CdmBase && StoreUtil.getCdmEntity(getEntity()).getId() == 0)) {
            // new entity, not yet saved
            requiredCrud = EnumSet.of(CRUD.CREATE);
        } else {
            requiredCrud = EnumSet.of(CRUD.UPDATE);
        }
	}

	@Override
    public T getEntity() {
		return entity;
	}

	/**
	 * Updates all widgets to display the latest data
	 */
	protected void updateContent() {
		removeElements();
		createControls(this, entity, SWT.NONE);
		updateControlStates();
	}

    @Override
    public void setSelected(boolean selected) {
		Composite section = getLayoutComposite().getParent();
		section.setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/**
	 * Sets all field to the given enablement state except the objects in
	 * the collection except parameter.
	 *
	 * @see eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled, Collection<Object> except) {
		boolean enabled_old = this.enabled;
	    this.enabled = enabled;
	    if (this.enabled != enabled_old){
			for (ICdmFormElement element : getElements()) {
				if (element instanceof IEnableableFormElement) {
					IEnableableFormElement enableableElement = (IEnableableFormElement) element;
					if (except != null && except.contains(enableableElement)) {
						// enableableElement.setEnabled(enabled);
					} else {
						enableableElement.setEnabled(enabled);
					}
				}
			}
	    }
	}

	@Override
	public boolean isEnabled() {
	    return enabled;
	}

    @Override
    public void setEnabled(boolean enabled) {
        setEnabled(enabled, null);
    }

    @Override
    public CacheRelevance cacheRelevance() {
        return cacheRelevanceHelper.cacheRelevance();
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        cacheRelevanceHelper.addDependsOnCache(toggleElement);
    }

    protected void registerCacheRelevance(ICacheRelevantFormElement element, ToggleableTextElement... toggleableCaches) {
        if (element != null && toggleableCaches != null) {
             for (ToggleableTextElement toggleableCache : toggleableCaches){
                 if (toggleableCache != null){
                     element.addDependsOnCache(toggleableCache);
                 }
             }
        }
//        updateCacheRelevance();
    }

	/**
	 * Updates the cache relevance state of all internal elements of this
	 * AbstractCdmDetailElement.
	 */
	@Override
    public void updateCacheRelevance(){
	    CacheRelevanceHelper.updateCacheRelevanceForSubelements(getElements());
	}

	/**
	 * Create all content elements
	 */
	protected abstract void createControls(ICdmFormElement formElement,
			T entity, int style);

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}

		handleTitleCacheRelevantChange(event);

		Object eventSource = event.getSource();

		if (eventSource instanceof ICdmFormElement) {
			ICdmFormElement eventElement = (ICdmFormElement) eventSource;
			ICdmFormElement eventElementContainer = eventElement.getParentElement();
			if (eventElementContainer == this) {
				if (event instanceof CdmPropertyChangeEvent) {
					if (((CdmPropertyChangeEvent) event).hasException()) {
						handleException((CdmPropertyChangeEvent) event);
						return;
					}
				}
				// call specific handler implementation
				handleEvent(eventSource);
				// refire the event
				firePropertyChangeEvent(this, event);
				// update possible parents
				updateParentSection();
			}
		}
	}

	protected void handleTitleCacheRelevantChange(PropertyChangeEvent event) {
		boolean isRelevant = getEntity() instanceof TaxonName
				|| getEntity() instanceof TaxonBase;
		isRelevant &= this instanceof AbstractIdentifiableEntityDetailElement;

		if (isRelevant) {
			((AbstractIdentifiableEntityDetailElement<?>)this).updateToggleableCacheField();
		}
	}

	private void updateParentSection() {
		if (getParentElement() instanceof AbstractCdmDetailSection) {
			((AbstractCdmDetailSection<?>) getParentElement()).updateTitle();
		}
		//NOTE: This slows down the rendering of the details view
//		StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	}

	/**
	 * Gets called when the source of the event is a child widget of this
	 * widget.
	 */
	public abstract void handleEvent(Object eventSource);

	public void handleException(CdmPropertyChangeEvent event) {
		// override this in subclasses if you want to deal with the error
		MessagingUtils.error(event.getSource().getClass(), event.getException());
	}

	@Override
    public ConversationHolder getConversationHolder() {
		if (getParentElement() instanceof IConversationEnabled) {
			return ((IConversationEnabled) getParentElement())
					.getConversationHolder();
		}
		throw new IllegalArgumentException(
				"Parent element should be IConversationEnabled");
	}

	@Override
    public void update(CdmDataChangeMap changeEvents) {
	}

	@Override
    public void update(Observable o, Object arg){
        if(o instanceof LoginManager){
            updateControlStates();
        }
    }

    protected void updateControlStates(){
        if (getEntity() != null && getEntity() instanceof CdmBase){
            enabled = enabled && CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(getEntity()), requiredCrud);
        }else{
            enabled = true;
        }
        setEnabled(enabled);
    }

	public LabelElement getWarnForReferencedObjects() {
		return warnForReferencedObjects;
	}

	public void setWarnForReferencedObjects(LabelElement warnForReferencedObjects) {
		this.warnForReferencedObjects = warnForReferencedObjects;
	}

	public void setWarnForReferencingObjects(ICdmFormElement formElement, int defaultReferencingObjects){
	    if (getEntity() instanceof CdmBase){
	    	CdmBase cdmBase = (CdmBase) getEntity();
	    	if (cdmBase.getId() != 0){
		        long referencingObjectsCount = CdmStore.getCommonService().getReferencingObjectsCount(cdmBase);

	    		if (referencingObjectsCount > defaultReferencingObjects){
	    			setWarnForReferencedObjects(formFactory.createLabel(formElement, CdmUtils.Nz("The "+ cdmBase.getUserFriendlyTypeName()+" is referenced by " + referencingObjectsCount+ " objects, if you change it, it is changed for all these objects")));
	    			getWarnForReferencedObjects().setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
	    			getWarnForReferencedObjects().setLayout(LayoutConstants.FILL(2, 3));
	    			getWarnForReferencedObjects().setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
	    			formFactory.createLabel(formElement, "");
	    		}
	    	}
	    }
	}

	public void setWarnForReferencingObjects(ICdmFormElement formElement){
	    setWarnForReferencingObjects(formElement, 1);
	}

	public void setWarnForReferencingObjectsVisible(boolean isVisible){
		if (getWarnForReferencedObjects() != null){
			getWarnForReferencedObjects().setVisible(isVisible);
		}
	}
}