/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.EnumSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.permission.CRUD;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;

/**
 * This class visualizes an CDM entity of type ENTITY and additionally provides
 * the functionality to add other elements of type ELEMENT to them.
 *
 * @param <ENTITY> A CDM entity which should be visualized by this section.
 * @param <ELEMENT> An element that can be added (multiple times) to this entity.
 *
 * @author n.hoffmann
 */
public abstract class AbstractEntityCollectionSection<ENTITY, ELEMENT>
        extends AbstractFormSection<ENTITY>
        implements IExpansionListener, Observer, IEnableableFormElement {

    private static final EnumSet<CRUD> UPDATE = EnumSet.of(CRUD.UPDATE);

    protected Composite container;

	private Label label_empty;

	private String title;

    private AbstractEntityCollectionElement<ENTITY> entityCollectionElement;
    
    private boolean isEnabled = true;

	public AbstractEntityCollectionSection(CdmFormFactory formFactory, ConversationHolder conversation, ICdmFormElement parentElement, String title, int style) {
		super(formFactory, parentElement, ExpandableComposite.CLIENT_INDENT | style);
		this.title = title;
		this.setText(getTitleString());
		updateToolbar();
		layout();
		addExpansionListener(this);

		CdmStore.getLoginManager().addObserver(this);
		addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                CdmStore.getLoginManager().deleteObserver(AbstractEntityCollectionSection.this);
            }
        });
	}

	protected Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);

		Action addAction = new Action("Add", IAction.AS_PUSH_BUTTON){
			@Override
			public void run() {
			    ELEMENT element = createNewElement();
			    if(element != null){
			        addElement(element);
			        if(! getSection().isExpanded()) {
			            getSection().setExpanded(true);
			        }
			        internalUpdateSection(true);
			    }
			}
		};
		addAction.setImageDescriptor(new ImageDescriptor() {

			@Override
			public ImageData getImageData() {
				return ImageResources.getImage(ImageResources.ADD_ICON).getImageData();
			}
		});
		addAction.setToolTipText(getTooltipString());

		Action browseAction = null;
		if(allowAddExisting()){
		    browseAction = new Action("Browse", IAction.AS_PUSH_BUTTON){
		        @Override
		        public void run() {
		            ELEMENT element = addExisting();
		            if(element != null){
		                addElement(element);
		                if(! getSection().isExpanded()) {
		                    getSection().setExpanded(true);
		                }
		                internalUpdateSection(true);
		            }
		        }
		    };
		    browseAction.setImageDescriptor(new ImageDescriptor() {

		        @Override
		        public ImageData getImageData() {
		            return ImageResources.getImage(ImageResources.BROWSE_ICON).getImageData();
		        }
		    });
		    browseAction.setToolTipText("Browse");
		}

		toolBarManager.add(addAction);
		if(browseAction!=null){
		    toolBarManager.add(browseAction);
		}

		addAction(toolBarManager);

		return toolBarManager.createControl(this);
	}

    protected void addAction(ToolBarManager toolBarManager) {
        // default implementation empty
    }

    /**
	 * using this method is discouraged, use updateToolBar() instead
	 */
	public void showToolbar(){
		setTextClient(createToolbar());
	}

    /**
     * using this method is discouraged, use updateToolBar() instead
     */
	public void removeToolbar(){
		setTextClient(null);
	}

	@Override
	public void setEntity(ENTITY entity) {
		if(entity != null){
			super.setEntity(entity);
			internalUpdateSection(false);
		}
		setSectionTitle();
		updateToolbar();
		layout();
	}

	/**
	 * Sets the title for the section. Adds a "+" sign if the collection is not empty for this section.
	 * Override in subclasses if you want to have a different behaviour.
	 */
	protected void setSectionTitle() {
	    ENTITY entity = getEntity();
		Collection<ELEMENT> collection = getCollection(entity);
		if(collection != null && collection.size() > 0){
			this.setText(getTitleString() + " +");
		}else{
			this.setText(getTitleString());
		}
	}

	/**
	 * Removes all content from the container
	 */
	protected void destroyDynamicContent(){
		if(label_empty != null){
			label_empty.dispose();
			label_empty = null;
		}
		removeElements();
	}

	/**
	 * Call this method after dynamically changing the client area.
	 * If the options changed is set to <code>true</code>, will also fire a state changed
	 * event to inform the user of unsaved changes.
	 *
	 * @param changed a boolean.
	 */
	protected void internalUpdateSection(boolean changed){
	    setSectionTitle();
		destroyDynamicContent();
		if(isExpanded() || expandSectionWhenContentAvailable()) {
            renderContent(isExpanded());
        }
		if(changed) {
            firePropertyChangeEvent(this);
        }
	}

	/**
	 * Create the elements to be shown in this section client area
	 */
	protected void renderContent(boolean forceExpansion){
		Collection<ELEMENT> collection = getCollection(getEntity());

		if(collection == null || collection.isEmpty()){
			createEmptyContent();
		}else{
		    List<ELEMENT> elements = new ArrayList<>(collection);
		    Collections.sort(elements, getComparator());
			createDynamicContents(elements);
			forceExpansion = true;
		}

		this.setExpanded(forceExpansion);

		reflow();
	}

	protected void createEmptyContent(){
		if (label_empty == null){
			label_empty = formFactory.createLabel(getLayoutComposite(), getEmptyString());
		}
	}

	/**
	 * Creates the widgets for the collection
	 */
	protected void createDynamicContents(Collection<ELEMENT> elements)
	{
		int i = 0;
		for(final ELEMENT element : elements){
			SelectionAdapter removeListener = new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					removeElement(element);
					internalUpdateSection(true);
				}
			};
			boolean modulo = i++%2 == 0;
			String colorResource = modulo ? Resources.COLOR_LIST_EVEN : Resources.COLOR_LIST_ODD;
			createElementComposite(element, removeListener, AbstractUtility.getColor(colorResource));
		}
	}

	/**
	 * Create the specific widget for the element
	 *
	 * @param element a ELEMENT object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param backgroundColor a {@link org.eclipse.swt.graphics.Color} object.
	 */
	protected void createElementComposite(ELEMENT element, SelectionListener removeListener, Color backgroundColor){
		entityCollectionElement = formFactory.createEntityCollectionElement(this, element, removeListener, backgroundColor, SWT.NULL);
		entityCollectionElement.setEnabled(isEnabled);
	}

	@Override
	public void setBackground(Color color) {
		if(label_empty != null && !label_empty.isDisposed()){
			label_empty.setBackground(color);
		}
		super.setBackground(color);
	}

	/**
	 * getTitleString
	 */
	public String getTitleString() {
		return CdmUtils.Nz(title);
	}

	/**
	 * setTitleString
	 */
	public void setTitleString(String title){
		this.title = title;
		setSectionTitle();
		layout();
	}

	@Override
    public void expansionStateChanging(ExpansionEvent e) {
//		logger.warn("Expansion State Changing");
	}

	@Override
    public void expansionStateChanged(ExpansionEvent e) {
		if(isExpanded()){
			renderContent(isExpanded());
		}else{
			destroyDynamicContent();
		}
	}

	protected boolean expandSectionWhenContentAvailable(){
		return PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOULD_EXPAND_SECTION_WHEN_DATA_AVAILABLE, true);
	}

	/**
	 * Remove an element from the entities collection and update the section
	 *
	 * @param element a ELEMENT object.
	 */
	public void removeElementAndUpdate(ELEMENT element) {
		removeElement(element);
		internalUpdateSection(true);
	}

	@Override
    public void update(Observable o, Object arg){
	    if(o instanceof LoginManager){
	        updateToolbar();
	    }
	}

    protected void updateToolbar() {
        if( !(getEntity() instanceof CdmBase) || (getEntity() != null && CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(getEntity()), UPDATE)) && isEnabled){
            showToolbar();
        } else {
            removeToolbar();
        }
    }

    public AbstractEntityCollectionElement<ENTITY> getEntityCollectionElement() {
        return entityCollectionElement;
    }

    /**
     * Returns the {@link Comparator} specific for the ELEMENTs
     * which is used to sort the elements
     * @return the comparator for ELEMENT
     */
    public abstract Comparator<ELEMENT> getComparator();

	/**
	 * Get the specific collection of this entity
	 *
	 * @param entity a ENTITY object.
	 * @return a {@link java.util.Collection} object.
	 */
	public abstract Collection<ELEMENT> getCollection(ENTITY entity);


	/**
	 * Create a new Element for this collection
	 *
	 * @return a ELEMENT object.
	 */
	public abstract ELEMENT createNewElement();

	/**
	 * Add an element to the entities collection
	 *
	 * @param element a ELEMENT object.
	 */
	public abstract void addElement(ELEMENT element);

	/**
	 * Add an existing element to the entities collection.
	 * @return the existing element
	 */
	public abstract ELEMENT addExisting();

	/**
	 * If <code>true</code> the section will also display
	 * a browse icon to choose from existing elements.
	 * <br>
	 * <b>Note:</b> when returning true you have to make sure
	 * to implement the {@link #addExisting()} method
	 * @return true if existing entities can be added;
	 * false otherwise
	 */
	public abstract boolean allowAddExisting();

	/**
	 * Remove an element from the entities collection
	 *
	 * @param element a ELEMENT object.
	 */
	public abstract void removeElement(ELEMENT element);

	/**
	 * String to display when the collection is empty
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public abstract String getEmptyString();

	/**
	 * <p>getTooltipString</p>
	 *
	 * @return String to display when hovering the add button
	 */
	protected abstract String getTooltipString();
	
	public boolean isEnabled(){
		return isEnabled;
	}
	
	public void setEnabled(boolean enabled){
		isEnabled=enabled;
		if (!isEnabled){
			removeToolbar();
		}
		
		for (ICdmFormElement element: getElements()){
			if (element instanceof IEnableableFormElement){
				((IEnableableFormElement)element).setEnabled(isEnabled);
			}
		}
		
	}
}
