/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * DescriptionElementSourceSection class.
 *
 * @author n.hoffmann
 * @created Nov 17, 2009
 */
public class DescriptionElementSourceSection extends
        AbstractEntityCollectionSection<DescriptionElementBase, OriginalSourceBase> {

    private Reference defaultSource;

    public DescriptionElementSourceSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(cdmFormFactory, conversation, parentElement, "Sources", style);
    }

    public DescriptionElementSourceSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, Reference defaultSource, int style) {
        super(cdmFormFactory, conversation, parentElement, "Sources", style);
        this.defaultSource = defaultSource;
    }

    @Override
    public DescriptionElementSource createNewElement() {
        DescriptionElementSource source =  DescriptionElementSource.NewInstance(OriginalSourceType.PrimaryTaxonomicSource);
        if (defaultSource != null){
            source.setCitation(defaultSource);
        }
        return source;
    }
    @Override
    protected void internalUpdateSection(boolean changed){
	    setSectionTitle();
		destroyDynamicContent();
		if(isExpanded() || expandSectionWhenContentAvailable()) {
            renderContent(isExpanded());
        }
		if(changed) {
            firePropertyChangeEvent(this);
        }
		if (getEntity() != null && getEntity().getInDescription() != null &&(getEntity().getInDescription().isComputed() || getEntity().getInDescription().isCloneForSource())){
    		this.setEnabled(false);;
    	}
	}
    @Override
    protected void renderContent(boolean forceExpansion){
    	
		Collection<OriginalSourceBase> collectionElement = getCollectionElement(getEntity());
		Collection<OriginalSourceBase> collectionDescription = getCollectionDescription(getEntity());

		if((collectionElement == null || collectionElement.isEmpty()) && (collectionDescription == null || collectionDescription.isEmpty())){
			createEmptyContent();
		}
		if (collectionElement != null && !collectionElement.isEmpty()){
		    List<OriginalSourceBase> elements = new ArrayList<>(collectionElement);
		    Collections.sort(elements, getComparator());
			createDynamicContents(elements);
			forceExpansion = true;
		}

		this.setExpanded(forceExpansion);
		

		
		
		if(collectionDescription != null && !collectionDescription.isEmpty()){
			List<OriginalSourceBase> elements = new ArrayList<>(collectionDescription);
		    Collections.sort(elements, getComparator());
		    createDynamicContentsDisabled(elements);			
			forceExpansion = true;
		}

		this.setExpanded(forceExpansion);

		reflow();
	}



	public Collection<OriginalSourceBase> getCollectionDescription(DescriptionElementBase entity) {
        Collection<OriginalSourceBase> sources = new HashSet<>();
//        sources.addAll(entity.getSources());
        sources.addAll(entity.getInDescription().getSources());

        return sources;
    }
    public Collection<OriginalSourceBase> getCollectionElement(DescriptionElementBase entity) {
        Collection<OriginalSourceBase> sources = new HashSet<>();
        sources.addAll(entity.getSources());
//        sources.addAll(entity.getInDescription().getSources());

        return sources;
    }

    @Override
    public Comparator<OriginalSourceBase> getComparator() {
        return new SourceComparator<>();
    }

    @Override
    public String getEmptyString() {
        return "No references yet.";
    }

    @Override
    protected String getTooltipString() {
        return "Create a new reference";
    }

    @Override
    public void addElement(OriginalSourceBase element) {
        if (element instanceof DescriptionElementSource){
            getEntity().addSource((DescriptionElementSource)element);
        }
    }

    @Override
    public void removeElement(OriginalSourceBase element) {
        if (element instanceof DescriptionElementSource){
            getEntity().removeSource((DescriptionElementSource)element);
        }
    }

    @Override
    public DescriptionElementSource addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }

	@Override
	public Collection<OriginalSourceBase> getCollection(DescriptionElementBase entity) {
		Collection<OriginalSourceBase> sources = new HashSet<>();
	    sources.addAll(entity.getSources());
	    sources.addAll(entity.getInDescription().getSources());
	    return sources;
	    
	}
	
	
	private void createDynamicContentsDisabled(Collection<OriginalSourceBase> elements)
	{
		int i = 0;
		for(final OriginalSourceBase element : elements){
			SelectionAdapter removeListener = new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					removeElement(element);
					internalUpdateSection(true);
				}
			};
			boolean modulo = i++%2 == 0;
			String colorResource = modulo ? Resources.COLOR_LIST_EVEN : Resources.COLOR_LIST_ODD;
			createElementComposite(element, removeListener, AbstractUtility.getColor(colorResource));
			getEntityCollectionElement().setEnabled(false);
		}
	}
}