/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.Collection;
import java.util.Comparator;

import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.graphics.ImageData;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.compare.name.TypeDesignationComparator;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.name.TextualTypeDesignation;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.ITaxonEditor;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.operation.DeleteTypeDesignationOperation;
import eu.etaxonomy.taxeditor.ui.section.name.type.CloneTypeWizard;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;
import eu.etaxonomy.taxeditor.workbench.part.IE4SavablePart;

/**
 * @author n.hoffmann
 * @created May 17, 2010
 */
public class TypeDesignationSection extends AbstractEntityCollectionSection<TaxonName, TypeDesignationBase> implements ITaxonBaseDetailSection {

	private TaxonBase<?> taxonBase;

	public TypeDesignationSection(CdmFormFactory formFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, Messages.TypeDesignationSection_TYPE_DESIGNATIONS, style);
	}

	@Override
	protected void addAction(ToolBarManager toolBarManager) {

	    // add textual type designation button
	    Action addTextualTypeDesignationAction = new Action("Add", IAction.AS_PUSH_BUTTON){
	        @Override
	        public void run() {
	            TextualTypeDesignation textualTypeDesignation = TextualTypeDesignation.NewInstance();
	            addElement(textualTypeDesignation);
	            if(! getSection().isExpanded()) {
	                getSection().setExpanded(true);
	            }
	            internalUpdateSection(true);
	        }
	    };
	    addTextualTypeDesignationAction.setImageDescriptor(new ImageDescriptor() {

	        @Override
	        public ImageData getImageData() {
	            return ImageResources.getImage(ImageResources.ADD_TEXT).getImageData();
	        }
	    });
	    addTextualTypeDesignationAction.setToolTipText("Add textual type designation");
	    toolBarManager.add(addTextualTypeDesignationAction);

        // clone button
	    if(!isSpecimenType() ||
	            getEntity().getTypeDesignations().isEmpty()
	            || getEntity().getTypeDesignations().stream().noneMatch(designation->designation instanceof SpecimenTypeDesignation)){
	        return;
	    }
	    Action cloneAction = new Action(Messages.TypeDesignationSection_CREATE_DUPLICATE, IAction.AS_PUSH_BUTTON){
            @Override
            public void run() {
                DetailsPartE4 detailsView = AbstractUtility.getDetailsView(getFormFactory().getContext().get((EPartService.class)));
                if(detailsView!=null
                        && detailsView.getSelectionProvidingPart().getObject() instanceof IE4SavablePart
                        && StoreUtil.promptCheckIsDirty(((IE4SavablePart)detailsView.getSelectionProvidingPart().getObject()))){
                    return;
                }
                CloneTypeWizard wizard = new CloneTypeWizard(getEntity());
                WizardDialog dialog = new WizardDialog(getShell(), wizard);

                if (dialog.open() == IStatus.OK) {
                    SpecimenTypeDesignation baseTypeDesignation = HibernateProxyHelper.deproxy(wizard.getBaseTypeDesignation());
                    UpdateResult result = CdmStore.getService(INameService.class).cloneTypeDesignation(
                            wizard.getTaxonName().getUuid(),
                            baseTypeDesignation,
                            wizard.getAccessionNumber(),
                            wizard.getBarcode(),
                            wizard.getCatalogNumber(),
                            wizard.getCollection().getUuid(),
                            wizard.getTypeStatus(),
                            wizard.getStableIdentifier());
                    if(!result.isOk()){
                        MessagingUtils.warningDialog(Messages.TypeDesignationSection_DUPLICATE_FAILED, this, result.getExceptions().toString());
                    }
                    StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
                    internalUpdateSection(false);
                }
            }
        };
        cloneAction.setImageDescriptor(new ImageDescriptor() {

            @Override
            public ImageData getImageData() {
                return ImageResources.getImage(ImageResources.ADD_DOUBLE).getImageData();
            }
        });
        cloneAction.setToolTipText(Messages.TypeDesignationSection_CREATE_DUPLICATE);

        toolBarManager.add(cloneAction);
	}

	@Override
	public void addElement(TypeDesignationBase element) {
	    getEntity().addTypeDesignation(element, PreferencesUtil.getBooleanValue(IPreferenceKeys.ADD_TYPES_TO_ALL_NAMES));
		updateToolbar();
	}

	@Override
	public TypeDesignationBase createNewElement() {
		if(isSpecimenType()){
			return SpecimenTypeDesignation.NewInstance();
		}else{
			return NameTypeDesignation.NewInstance();
		}
	}

	@Override
	public Collection<TypeDesignationBase> getCollection(TaxonName entity) {
		if (entity == null){
			return null;
		}
		return entity.getTypeDesignations();
	}

	@Override
	public Comparator<TypeDesignationBase> getComparator() {
	    return new TypeDesignationComparator();
	}

	@Override
	public String getEmptyString() {
		return Messages.TypeDesignationSection_NO_TYPES_YET;
	}

	@Override
	protected String getTooltipString() {
		return Messages.TypeDesignationSection_ADD_TYPE;
	}

	@Override
	public void removeElement(TypeDesignationBase element) {
	    DetailsPartE4 detailsView = AbstractUtility.getDetailsView(getFormFactory().getContext().get((EPartService.class)));
	    if(detailsView!=null
	            && detailsView.getSelectionProvidingPart().getObject() instanceof IE4SavablePart
	            && StoreUtil.promptCheckIsDirty(((IE4SavablePart)detailsView.getSelectionProvidingPart().getObject()))){
	        return;
	    }
		boolean removeTypeDesignationFromAllTypifiedNames = PreferencesUtil.getBooleanValue(IPreferenceKeys.ADD_TYPES_TO_ALL_NAMES);
		TaxonName entity = removeTypeDesignationFromAllTypifiedNames ? null : getEntity();
		if(!element.isPersited() && entity!=null){
            entity.removeTypeDesignation(element);
		}
		else if(detailsView!=null
		        && detailsView.getSelectionProvidingPart().getObject() instanceof ITaxonEditor
		        && detailsView.getSelectionProvidingPart().getObject() instanceof IPostOperationEnabled) {
            DeleteTypeDesignationOperation operation = new DeleteTypeDesignationOperation(
                    "Remove type designation",
                    IOperationHistory.GLOBAL_UNDO_CONTEXT,
                    entity,
                    element,
                    (IPostOperationEnabled) detailsView.getSelectionProvidingPart().getObject(),
                    null);
            ((ITaxonEditor) detailsView.getSelectionProvidingPart().getObject()).addOperation(operation);
            if(entity!=null){
                entity.removeTypeDesignation(element);
            }
		}
		updateToolbar();
	}

	@Override
    public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		TaxonName name = HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}

	private boolean isSpecimenType(){
		Rank rank = getEntity() == null ? null : getEntity().getRank();
		if(rank == null){
			return false;
		}
		return rank.isSpecies() || rank.isInfraSpecific();
	}

	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

    @Override
    public TypeDesignationBase addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }
}
