/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.media;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.model.term.VocabularyEnum;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.media.ITogglableMediaElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 16.12.2013
 */
public class MediaSpecimenGeneralDetailElement
        extends AbstractCdmDetailElement<MediaSpecimen>
        implements ITogglableMediaElement{

    private final TermVocabulary<?> mediaSpecimenVocabulary = CdmStore.getService(IVocabularyService.class).find(VocabularyEnum.MediaSpecimenKindOfUnit.getUuid());

    private TermComboElement<DefinedTerm> comboKindOfUnit;
    private LanguageStringWithLabelElement textTitleLanguageString;
    private EntitySelectionElement<AgentBase> selectionArtist;
    private TimePeriodElement date;
    private LanguageStringWithLabelElement textMethodLanguageString;
    private EntitySelectionElement<Collection> selection_collection;
    private TextWithLabelElement text_accessionNumber;
    private EntitySelectionElement<Reference> selection_publishedIn;
    private TextWithLabelElement text_published_in_detail;
    private CheckboxElement checkIsPublish;

    private MediaDetailElement mediaDetailElement;

    public MediaSpecimenGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, MediaSpecimen entity, int style) {

        Media media = entity == null ? null: entity.getMediaSpecimen();

        if(media==null){
            formFactory.createLabel(formElement, "No media attached to this MediaSpecimen!");
        }
        else{
            comboKindOfUnit = formFactory.createDefinedTermComboElement(mediaSpecimenVocabulary, formElement, "Kind of Media", entity.getKindOfUnit(), style);
            textTitleLanguageString = formFactory.createLanguageStringWithLabelElement(formElement, "Subject", media.getTitle(), style);
            LabelElement label1 = formFactory.createLabel(formElement, "");

            selectionArtist = formFactory.createSelectionElement(AgentBase.class, //getConversationHolder(),
                    formElement, "Prepared by", media.getArtist(), EntitySelectionElement.ALL, style);
            TimePeriod mediaCreated = entity.getMediaSpecimen().getMediaCreated();

            date = formFactory.createTimePeriodElement(formElement, "Preparation Date", mediaCreated, style);
            textMethodLanguageString = formFactory.createLanguageStringWithLabelElement(formElement, "Method", media.getDescription(Language.getDefaultLanguage()), style);
//            EmptyElement spacer = formFactory.createEmptyElement(formElement, ""); //$NON-NLS-1$
//            spacer.setBackground(getPersistentBackground());
            LabelElement label2 = formFactory.createLabel(formElement, "");

            selection_collection = formFactory.createSelectionElement(Collection.class,
                    formElement, "Collection", entity.getCollection(), EntitySelectionElement.ALL, style);
            text_accessionNumber = formFactory.createTextWithLabelElement(formElement, "Accession Number", entity.getAccessionNumber(), style);

            LabelElement label3 = formFactory.createLabel(formElement, "");
            //the first PrimaryMediaSource is used for storing the reference
            Reference publishedIn = null;
            String publishedInDetail = null;
            for(IdentifiableSource source:media.getSources()){
                if(source.getType()==OriginalSourceType.PrimaryMediaSource){
                    publishedIn = source.getCitation();
                    publishedInDetail = source.getCitationMicroReference();
                    break;
                }
            }
            selection_publishedIn = formFactory.createSelectionElement(Reference.class,
                    formElement, "Published in", publishedIn, EntitySelectionElement.ALL, style);
            text_published_in_detail = formFactory.createTextWithLabelElement(formElement, "Published in Detail", publishedInDetail, style);
            LabelElement label4 = formFactory.createLabel(formElement, "");

            checkIsPublish = formFactory.createCheckbox(formElement, "Publish", entity.isPublish(), style);

            mediaDetailElement = formFactory.createMediaDetailElement(formElement);
            mediaDetailElement.setEntity(media);
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        Media media = getEntity().getMediaSpecimen();
        if(media==null){
            media = Media.NewInstance();
            getEntity().setMediaSpecimen(media);
        }
        if(eventSource==comboKindOfUnit){
            getEntity().setKindOfUnit(comboKindOfUnit.getSelection());
        }
        else if(eventSource==textTitleLanguageString){
            getEntity().getMediaSpecimen().putTitle(textTitleLanguageString.getLanguageString());
            getEntity().resetTitleCache();
        }
        else if(eventSource==date){
            TimePeriod dateTime = date.getTimePeriod();
            getEntity().getMediaSpecimen().setMediaCreated(dateTime);
            getEntity().resetTitleCache();
        }
        else if(eventSource==selectionArtist){
            getEntity().getMediaSpecimen().setArtist(selectionArtist.getSelection());
            getEntity().resetTitleCache();
        }
        else if(eventSource==textMethodLanguageString){
            media.putDescription(textMethodLanguageString.getLanguageString());
            getEntity().setMediaSpecimen(media);
        }
        else if (eventSource == selection_collection) {
            getEntity().setCollection(selection_collection.getSelection());
        }
        else if (eventSource == text_accessionNumber) {
            getEntity().setAccessionNumber(text_accessionNumber.getText());
        }
        else if(eventSource==selection_publishedIn){
            IdentifiableSource primaryMediaSource = null;
            for(IdentifiableSource source:media.getSources()){
                if(source.getType()==OriginalSourceType.PrimaryMediaSource){
                    primaryMediaSource = source;
                }
            }
            if(primaryMediaSource==null){
                primaryMediaSource = IdentifiableSource.NewInstance(OriginalSourceType.PrimaryMediaSource);
                media.addSource(primaryMediaSource);
            }
            else{
                primaryMediaSource.setCitation(selection_publishedIn.getSelection());
            }
        }
        else if(eventSource==text_published_in_detail){
            IdentifiableSource primaryMediaSource = null;
            for(IdentifiableSource source:media.getSources()){
                if(source.getType()==OriginalSourceType.PrimaryMediaSource){
                    primaryMediaSource = source;
                }
            }
            if(primaryMediaSource==null){
                primaryMediaSource = IdentifiableSource.NewInstance(OriginalSourceType.PrimaryMediaSource);
                media.addSource(primaryMediaSource);
            }
            else{
                primaryMediaSource.setCitationMicroReference(text_published_in_detail.getText());
            }
        }
        else if(eventSource==checkIsPublish){
            getEntity().setPublish(checkIsPublish.getSelection());
        }

    }

    @Override
    public void toggleAdvancedMediaView() {
        if(mediaDetailElement!=null){
            mediaDetailElement.toggleAdvancedMediaView();
        }
    }

    public boolean isAdvancedMediaView() {
        if(mediaDetailElement==null){
            return false;
        }
        return mediaDetailElement.isAdvancedMediaView();
    }

}
