/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.selection;

import java.util.EnumSet;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.permission.CRUD;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;
import eu.etaxonomy.taxeditor.ui.element.AbstractRelevanceFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.ILabeledElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityLabelTextProvider;

/**
 * @author n.hoffmann
 * @created Nov 17, 2009
 */
public class EntitySelectionElement<T extends CdmBase>
        extends AbstractRelevanceFormElement
        implements  SelectionListener, IEnableableFormElement, ISelectableElement,
            IEntityElement<T>, ILabeledElement, Observer {

	private static final EnumSet<CRUD> UPDATE = EnumSet.of(CRUD.UPDATE);
	private static final EnumSet<CRUD> DELETE = EnumSet.of(CRUD.DELETE);
	private static final EnumSet<CRUD> CREATE = EnumSet.of(CRUD.CREATE);

	/**
	 * Bitmask for configuring functionality of selection element
	 */
	public static final int NOTHING = 0; // 0000
	public static final int EDITABLE = 1 << 0; // 0001
	public static final int DELETABLE = 1 << 1; // 0010
	public static final int SELECTABLE = 1 << 2; // 0100
	public static final int EXTERNAL = 1 << 3; // 1000
	public static final int ALL = EDITABLE | DELETABLE | SELECTABLE ; // 0111
	public static final int ALL_WITH_EXT = EDITABLE | DELETABLE | SELECTABLE | EXTERNAL ; // 1111
	protected T entity;
	protected T filteredEntity;

	protected Label label;
	protected Text text;
	protected Button button_selection;

	protected Button button_selectionExt;

	private SelectionArbitrator selectionArbitrator;

	protected Button button_edit;

	private final String labelString;

	private Composite selectableComposite;

	private Button button_remove;

	protected final boolean isEditable;

	private final boolean isDeletable;
	private final boolean isExternal;

	private Class<T> clazz;

	public EntitySelectionElement(CdmFormFactory formFactory,
            ICdmFormElement parentElement, Class<T> clazz,
            String labelString, T entity, int mode, int style, boolean filterElement){
	    this(formFactory, parentElement, clazz, labelString, entity, mode, style, filterElement, null);
	}

	public EntitySelectionElement(CdmFormFactory formFactory,
	        ICdmFormElement parentElement, Class<T> clazz,
			String labelString, T entity, int mode, int style, boolean filterElement, Integer limit) {
		super(formFactory, parentElement);

		this.clazz = clazz;
		this.isEditable = (mode & EDITABLE) == EDITABLE;
		this.isDeletable = (mode & DELETABLE) == DELETABLE;
		this.isExternal= (mode & EXTERNAL) == EXTERNAL;
		boolean isSelectable = (mode & SELECTABLE) == SELECTABLE;

		this.labelString = (labelString == null || labelString.equals("")) ? "" : labelString;

		if (isSelectable && formFactory.getSelectionProvider() != null) {
			selectionArbitrator = formFactory.createSelectionArbitrator(this);
		}

		createControls(getLayoutComposite(), SWT.NULL, limit);
		if (filterElement){
		    setFilteredEntity(entity);
		}else{
		    setEntity(entity);
		}
	}

    private void setFilteredEntity(T filterEntity) {
        this.filteredEntity =filterEntity;
    }

    public EntitySelectionElement(CdmFormFactory formFactory,
             ICdmFormElement parentElement, Class<T> clazz,
            String labelString, T entity, int mode, int style){
        this(formFactory,
                parentElement, clazz, labelString, entity, mode, style, false);
    }

    public EntitySelectionElement(CdmFormFactory formFactory,
            ICdmFormElement parentElement, Class<T> clazz,
           String labelString, T entity, int mode, int style, Integer limit){
       this(formFactory,
               parentElement, clazz, labelString, entity, mode, style, false, limit);
    }

	private void createControls(Composite parent, int style, Integer limit) {

		label = formFactory.createLabel(getLayoutComposite(), labelString,
				SWT.NULL);

		addControl(label);

		selectableComposite = formFactory.createComposite(getLayoutComposite());

		int columns = 2;
		if (isEditable) {
			columns += 1;
		}
		if (isDeletable) {
			columns += 1;
		}
		if (isExternal) {
            columns += 1;
        }

		selectableComposite.setLayout(LayoutConstants.LAYOUT(columns, false));
		selectableComposite.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		addControl(selectableComposite);

		text = formFactory.createText(selectableComposite, null, SWT.WRAP);
		text.setEditable(false);

		addControl(text);

		text.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
		text.setBackground(AbstractUtility.getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND));
		if (limit != null){
		    text.setTextLimit(limit);
		}
		button_selection = formFactory.createButton(selectableComposite, null,
				SWT.PUSH);
		button_selection.setImage(ImageResources
				.getImage(ImageResources.BROWSE_ICON));
		button_selection.setToolTipText("Browse existing");

		addControl(button_selection);
		button_selection.addSelectionListener(this);
		if (isExternal){
    		button_selectionExt = formFactory.createButton(selectableComposite, null,
                    SWT.PUSH);
    		button_selectionExt.setImage(ImageResources
                    .getImage(ImageResources.BROWSE_ICON));
    		button_selectionExt.setToolTipText("Browse existing from external cdm store");
    		button_selectionExt.setText("Ext");
            addControl(button_selectionExt);
            button_selectionExt.addSelectionListener(this);
		}
		if (isEditable) {
			button_edit = formFactory.createButton(selectableComposite, null,
					SWT.PUSH);
			button_edit.setImage(ImageResources
					.getImage(ImageResources.EDIT_ICON));
			button_edit.setToolTipText("Edit");
			addControl(button_edit);
			button_edit.addSelectionListener(new EditListener(this));
		}

		if (isDeletable) {
			button_remove = formFactory.createButton(selectableComposite, null,
					SWT.PUSH);
			button_remove.setImage(ImageResources
					.getImage(ImageResources.TRASH_ICON));
			button_remove.setToolTipText("Remove");
			addControl(button_remove);
			button_remove.addSelectionListener(new DeleteListener(this));
		}
	}

	@Override
    public void widgetSelected(SelectionEvent e) {
	    if (e.getSource().equals(button_selection) ){
	        T selection = SelectionDialogFactory.getSelectionFromDialog(clazz, getShell(),
	                getEntity(), getParentElement());
            setSelectionInternal(selection);
            if(!getLayoutComposite().isDisposed()){
                StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
            }
	    }else{
	        Reference selection = SelectionDialogFactory.getSelectionFromExtDialog(Reference.class, getShell(),
	                getParentElement());
            setSelectionInternal((T)selection);
	    }
	}

    public T getFilteredEntity() {

        return this.filteredEntity;
    }

    /**
	 * Return the selected object
	 */
	public T getSelection() {
		return entity;
	}

	@Override
	public void setEnabled(boolean enabled) {
		this.label.setEnabled(enabled);
		button_selection.setEnabled(enabled);
		if (isDeletable){
		    button_remove.setEnabled(enabled);
		}
		if (isEditable) {
			updateButtonStates();
		}

	}

	@Override
	public boolean isEnabled() {
	    return button_selection.isEnabled();
	}

	/**
	 * setSelectionInternal
	 */
	protected void setSelectionInternal(T selection) {
		if (selection != null && !selection.equals(this.entity)) {
			setEntity(selection);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
		}
	}

	public void setEntity(T selection) {
		this.entity = selection;
		updateElement();
	}

	/**
	 * Updates this elements view
	 */
	protected void updateElement() {
		String title = CdmUtils.Nz(getTitle());

		text.setText(title); // title can be null
		if (isEditable) {
			updateButtonStates();
		}
	}

    public void updateFromWizard() {
		updateElement();
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	protected String getTitle() {
		if (entity != null){
			if(entity instanceof Group){
				return ((Group) entity).getName();
			} else if(entity instanceof GrantedAuthority){
				return GrantedAuthorityLabelTextProvider.getText(((GrantedAuthority) entity));
			} else if(entity instanceof User){
                return ((User) entity).getUsername();
            } else if (entity instanceof Primer){
                return ((Primer) entity).getLabel();
            } else if (entity instanceof Amplification){
                return ((Amplification) entity).getLabelCache();
            }
            else if(entity instanceof IIdentifiableEntity) {
                return ((IIdentifiableEntity) entity).getTitleCache();
            }

		}
		return "";
	}

	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	@Override
	public T getEntity() {
		return entity;
	}

	@Override
	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}

	/**
	 * Convenient access to current shell
	 *
	 * @return a {@link org.eclipse.swt.widgets.Shell} object.
	 */
    public Shell getShell() {
		return getLayoutComposite().getShell();
	}

	@Override
	public void updateCacheRelevance() {
		Color color = cacheRelevance().getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND);
		text.setBackground(color);
	}

	private class DeleteListener extends SelectionAdapter {

		private final EntitySelectionElement<T> selectionElement;

		public DeleteListener(EntitySelectionElement<T> selectionElement) {
			this.selectionElement = selectionElement;
		}

		@Override
		public void widgetSelected(SelectionEvent e) {
			setEntity(null);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(
					selectionElement, null));
		}
	}

	private class EditListener extends SelectionAdapter {

        private final EntitySelectionElement<T> selectionElement;

		public EditListener(EntitySelectionElement<T> selectionElement) {
			this.selectionElement = selectionElement;
		}

		@Override
		public void widgetSelected(SelectionEvent e) {
			WizardDialog dialog = new WizardDialog(selectionElement.getShell(),
					new EditFromSelectionWizard(selectionElement));
			if (dialog.open() == IStatus.OK) {
				selectionElement.updateFromWizard();
				//if the edited entity has already been persisted
				//but the transient entity is still set in this
				//EntitySelectionElement, re-load it and set it
				if(!entity.isPersited()){
				    IService<T> service = CdmStore.getService(entity);
					T loadedEntity = service.load(entity.getUuid());
					if(loadedEntity != null){
						setEntity(loadedEntity);
					}
				}
			}
		}
	}

	// not used
	@Override
	public void widgetDefaultSelected(SelectionEvent e) {
	}

//	@Override
	public ConversationHolder getConversationHolder() {
	    if(getParentElement() instanceof IConversationEnabled) {
            return ((IConversationEnabled)getParentElement()).getConversationHolder();
        }
	    return null;
	}

	@Override
	public void setBackground(Color color) {

	    if(label != null && !label.isDisposed()){
	        label.setBackground(color);
	    }
	}

	@Override
	public void setLabel(String labelString) {
		if (label != null) {
			label.setText(labelString);
		}
	}

	@Override
	public String getLabel() {
		if (label != null) {
			return label.getText() + " : ";
		}
		return null;
	}

	@Override
    public void removeElements(){
		super.removeElements();
		LoginManager loginManager = CdmStore.getLoginManager();
		loginManager.addObserver(this);
	}

	@Override
	public void update(Observable o, Object arg) {
		if(o instanceof LoginManager){
			updateButtonStates();
		}
	}

	protected void updateButtonStates() {
	    if(button_edit != null && !button_selection.isDisposed()){
	        button_edit.setEnabled(isEditable && button_selection.isEnabled() && getEntity() != null  && CdmStore.currentAuthentiationHasPermission(getEntity(), UPDATE));
	    }
	}

	public void setIndent(int indent){
        TableWrapData tableWrapData = (TableWrapData)label.getLayoutData();

        if (tableWrapData == null){
            tableWrapData = new TableWrapData();

        }
        tableWrapData.indent = indent;
        label.setLayoutData(tableWrapData);

        getLayoutComposite().layout();
    }
}