/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.util;

import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.SubMonitor;
import org.springframework.remoting.RemoteAccessException;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.IProgressMonitorService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.taxeditor.operation.IFeedbackGenerator;
import eu.etaxonomy.taxeditor.operation.IPostMoniteredOperationEnabled;

/**
 * Manages client side progress monitors
 *
 * @author cmathew
 * @date 23 Oct 2015
 */
public class ProgressMonitorClientManager {
    private static final Logger logger = Logger.getLogger(ProgressMonitorClientManager.class);

    /**
     * Polls the progress monitor service for the progress status of a monitor
     * corresponding to the given uuid.
     *
     * @param label for the operation
     * @param uuid of the remoting monitor already started on the server
     * @param pollInterval in milliseconds
     * @param cancelable flag which determines whether the operation can be cancelled
     * @param postOp callback for running post operation logic
     * @param feedbackGenerator feedback generator corresponding to the
     *        'wait on feedback' request made on the remoting monitor
     * @param monitor to be updated
     * @return a final progress monitor after the operation is completed
     * @throws InterruptedException
     */
    public IRemotingProgressMonitor pollMonitor(final String label,
            final UUID uuid,
            final int pollInterval,
            final IPostMoniteredOperationEnabled postOp,
            IFeedbackGenerator feedbackGenerator,
            SubMonitor monitor) throws InterruptedException {
        return pollMonitor(label, uuid, pollInterval, postOp, Arrays.asList(feedbackGenerator), monitor);
    }

    /**
     * Polls the progress monitor service for the progress status of a monitor
     * corresponding to the given uuid.
     *
     *
     * @param label for the operation
     * @param uuid of the remoting monitor already started on the server
     * @param pollInterval in milliseconds
     * @param cancelable flag which determines whether the operation can be cancelled
     * @param postOp callback for running post operation logic
     * @param feedbackGenerators list of feedback generators corresponding to the
     *        size and exact order of the 'wait on feedback' requests made on the
     *        remoting monitor
     * @param monitor to be updated
     * @return a final progress monitor after the operation is completed
     * @throws InterruptedException
     */
    public IRemotingProgressMonitor pollMonitor(final String label,
            final UUID uuid,
            final int pollInterval,
            final IPostMoniteredOperationEnabled postOp,
            List<IFeedbackGenerator> feedbackGenerators,
            SubMonitor monitor) throws InterruptedException {

        IProgressMonitorService progressMonitorService = CdmApplicationState.getCurrentAppConfig().getProgressMonitorService();
        IRemotingProgressMonitor remotingMonitor = progressMonitorService.getRemotingMonitor(uuid);
        try {
            final int START_DELAY=10;
            // wait about 10 seconds for the remoting monitor to be initialised
            // (i.e. for the begin task method to be called ON THE REMOTING MONITOR)
            if (remotingMonitor != null && remotingMonitor.isDone()){
                return remotingMonitor;
            }
            for(int i=0;remotingMonitor==null && i<START_DELAY;i++) {
                Thread.sleep(500);
                logger.info("Waiting for monitored work to start ..");
                remotingMonitor = progressMonitorService.getRemotingMonitor(uuid);
            }
            if (remotingMonitor == null){
                return null;
            }

            // if the total work is still not been set then we assume that the
            // operation has zero work units
            if(remotingMonitor.getTotalWork() == 0 && remotingMonitor.isDone()) {
                return remotingMonitor;
            }else if (remotingMonitor.getTotalWork() == 0 ){
                Object result = remotingMonitor.getResult();
                InterruptedException exception = new InterruptedException("Monitor has zero work units");
                if (result instanceof UpdateResult){
                    ((UpdateResult)result).addException(exception);
                }else if (result == null){
                    result = new UpdateResult();
                }
            }

            // start the client monitor
            logger.info("Work to be done: " + remotingMonitor.getTotalWork());
            int editorTotalWorkDone = 0;
            int serverTotalWorkDone = 0;
            // loop until the operation is done
            int feedbackCount = 0;
            int failCount = 10;
            while(!(remotingMonitor.isCanceled() || remotingMonitor.isFailed() || remotingMonitor.isDone())) {
                // wait for pollInterval, then
                // .... retrieve remoting monitor, then
                //      .... set client monitor info

                IRemotingProgressMonitor previousRemotingMonitor = remotingMonitor;
                Thread.sleep(pollInterval);
                try{
                    remotingMonitor = progressMonitorService.getRemotingMonitor(uuid);
                }catch(RemoteAccessException e){
                    failCount--;
                    if (failCount > 0){
                        remotingMonitor = previousRemotingMonitor;
                        continue;
                    }else{
                        throw e;
                    }
                }

                // check if remoting monitor is waiting for feedback
                if(remotingMonitor.getIsWaitingForFeedback()) {
                    if(feedbackGenerators != null) {
                        // if we have run out of feedback generators while
                        // the remoting monitor is waiting on feedback
                        // then throw exception
                        if(feedbackCount + 1 > feedbackGenerators.size()) {
                            IllegalStateException exception = new IllegalStateException("Remoting monitor waiting on feedback that does not exist");
                            Object result = remotingMonitor.getResult();
                            if (result instanceof UpdateResult){
                                ((UpdateResult)result).addException(exception);
                            }
                            return remotingMonitor;
                        }
                        feedbackGenerators.get(feedbackCount).setFeedbackForMonitor(uuid);
                        feedbackCount++;
                    }
                }

                serverTotalWorkDone = (int) remotingMonitor.getWorkDone();
                logger.info("Work done from start: " + serverTotalWorkDone);

                if (serverTotalWorkDone > editorTotalWorkDone){
                    int worked = serverTotalWorkDone - editorTotalWorkDone;
                    if(worked > 0) {
                        logger.info("Work done since last check: " + worked);
                        monitor.worked(worked);
                    }
                }

                // set dialog text
                String subTaskName = remotingMonitor.getSubTask();
                String percentage = "0";
                if (remotingMonitor.getTotalWork() != 0){
                    percentage = new DecimalFormat("#.##").format(remotingMonitor.getPercentage());
                }
                subTaskName = subTaskName + " (" + percentage +"% done)";
                //Note: do not update taskname as it resets percentage in status line
                monitor.subTask(subTaskName );

                editorTotalWorkDone = serverTotalWorkDone;
            }
            if(remotingMonitor.getResult() instanceof Exception) {
                UpdateResult result = new UpdateResult();
                result.addException((Exception) remotingMonitor.getResult());
            }
            return remotingMonitor;
        } finally {
            if(postOp != null && remotingMonitor != null && remotingMonitor.isDone()) {
                postOp.postOperation(remotingMonitor);
            }
        }
    }
}
