/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.e4;

import javax.inject.Inject;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.services.log.Logger;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.remoting.CdmEagerLoadingException;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.EmptySection;
import eu.etaxonomy.taxeditor.view.detail.CdmSectionPart;

/**
 *
 * @author pplitzner
 * @date 18.07.2017
 *
 */
public abstract class AbstractCdmDataViewer extends Viewer implements IConversationEnabled{
	 @Inject
	 private Logger logger;

	protected ManagedForm managedForm;

	protected CdmFormFactory formFactory;

	protected ScrolledForm scrolledForm;

	private Composite body;

	protected RootElement rootElement;

	@Inject
	protected IEclipseContext context;

	//TODO: create a super class for this?
	private Object input;

	protected IDirtyMarkable part;

	@Inject
	public AbstractCdmDataViewer() {

	}

	public void init(Composite parent, IDirtyMarkable part) {
	    this.part = part;

		managedForm = new ManagedForm(parent){

			@Override
			public void dirtyStateChanged() {
			    markViewPartDirty();
			}
		};

		createFormFactory();

		scrolledForm = managedForm.getForm();

		body = scrolledForm.getBody();

		body.setLayout(LayoutConstants.LAYOUT());

		rootElement = new RootElement(formFactory, body);
	}


    protected void createEmptySection(String message, RootElement parent) {
        destroySections();

        EmptySection emptySection = formFactory.createEmptySection(message, formFactory, parent, SWT.NONE);

        addPart(emptySection);
    }

    public void showEmptyPage(String message){
    	destroySections();
        createEmptySection(message, rootElement);
    }

    protected void markViewPartDirty(){
        part.changed(input);
    }

	protected void createFormFactory() {
		if(formFactory != null){
			formFactory.dispose();
			formFactory = null;
		}
		formFactory =  new CdmFormFactory(Display.getCurrent(), this);
		ContextInjectionFactory.inject(formFactory, context);
	}

	@Override
	public Control getControl() {
	    if(body!=null){
	        if(body.isDisposed()){
	            return null;
	        }
	        for(Control child : body.getChildren()){
	            return child;
	        }
	    }

		return body;
	}

	@Override
	public void setInput(Object input) {
		this.input = input;
		if(input!=null){
		    try{
		        refresh();
		    }catch(CdmEagerLoadingException e){
		        logger.error(e);
		    }
		}
	}

	@Override
	public Object getInput() {
		return input;
	}

	@Override
	public void refresh() {
		showParts();

		if(input!=null){
		    managedForm.setInput(input);
		}

		managedForm.refresh();

//		managedForm.reflow(true);
	}

	public void layout(){
		body.layout();
	}

	protected abstract void showParts();

	public void destroySections() {
		for (IFormPart formPart : managedForm.getParts()){
			removePart((CdmSectionPart<?>) formPart);
		}

		managedForm.setInput(null);
		try{
		    formFactory.destroyElement(rootElement);
		}catch (SWTException e){
		    if (PreferencesUtil.isShowUpWidgetIsDisposedMessages() && e.getMessage().equals("Widget is disposed")){
                MessagingUtils.errorDialog("Widget is disposed",
                        null,
                        MessagingUtils.WIDGET_IS_DISPOSED_MESSAGE,
                        null,
                        e,
                        true);

            }
		}

		createFormFactory();

		rootElement = new RootElement(formFactory, body);

		for(Control control : body.getChildren()){
			try{
				if (control != null && !control.isDisposed()){
					control.dispose();
				}
			}catch(SWTException e){
				if (PreferencesUtil.isShowUpWidgetIsDisposedMessages() && e.getMessage().equals("Widget is disposed")){
                    MessagingUtils.errorDialog("Widget is disposed",
                            null,
                            MessagingUtils.WIDGET_IS_DISPOSED_MESSAGE,
                            null,
                            e,
                            true);
                }
			}
			control = null;
		}
	}

	public void setFocus(){
		// we have to set focus to a control of this viewer
		// otherwise, after opening a dialog from the details, the focus will not be
		// given back to the details view but to the editor
		for(Control child : body.getChildren()){
			child.setFocus();
			break;
		}
	}

	public void reflow(){
		managedForm.reflow(true);
	}

	protected void removePart(CdmSectionPart<?> sectionPart){
		managedForm.removePart(sectionPart);
		formFactory.removePropertyChangeListener(sectionPart);
	}

	protected void addPart(AbstractFormSection<?> section){
		CdmSectionPart<?> sectionPart = new CdmSectionPart<>(section);
		managedForm.addPart(sectionPart);
		formFactory.addPropertyChangeListener(sectionPart);
	}

    @Override
    public ConversationHolder getConversationHolder() {
        return null;
	}

}
