/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.facet.term;

import java.io.InputStream;
import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;
import java.util.stream.Collectors;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaUtils;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.view.search.facet.CheckBoxSearchResultComposite;

/**
 * @author pplitzner
 * @since Jan 23, 2019
 */
public class TermSearchResultComposite extends CheckBoxSearchResultComposite<AbstractTermDto, TermSearchResult> {

    private Collection<Image> imageBuffer;



    public TermSearchResultComposite(TermSearchResult result, Composite parent, int style) {
        super(result, parent, style);

    }

    @Override
    public void dispose() {
        for (Image image : imageBuffer) {
            image.dispose();
            imageBuffer = null;
        }

        super.dispose();
    }

    @Override
    public Composite createContent(Composite parent) {
        Composite content = new Composite(this, SWT.NONE);
        content.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        GridLayout layout = new GridLayout(1, false);
        content.setLayout(layout);
        Label label = new Label(content, SWT.WRAP);
        label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        label.setText(result.getContent().getRepresentation_L10n());
        label.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
        GridDataFactory.fillDefaults().applyTo(label);

        // label
        String facetText = result.getFacets().stream()
                .map(facet -> (facet.getFacet() != null ? facet.getFacet() : "")
                        + (facet.getCategory() != null ? " (" + facet.getCategory() + ")" : ""))
                .collect(Collectors.joining(","));
        if (CdmUtils.isNotBlank(facetText)) {
            Label labelFacets = new Label(content, SWT.WRAP);
            labelFacets.setText(facetText);
            labelFacets.setFont(JFaceResources.getFontRegistry().getItalic(JFaceResources.DEFAULT_FONT));
        }
        // description
        String representation_L10n_text = result.getContent().getRepresentation_L10n_text();
        if(representation_L10n_text!=null){
            Label lblDescription = new Label(content, SWT.WRAP);
            GridData layoutData = new GridData();
            lblDescription.setLayoutData(layoutData);
            parent.addControlListener(new ControlAdapter() {
                @Override
                public void controlResized(ControlEvent e) {
                    layoutData.widthHint = parent.getClientArea().width - (6*layout.marginWidth);
                    parent.layout(true);
                }
            });
            lblDescription.setText(representation_L10n_text);
        }
        // media
        if (result.getContent() instanceof TermDto){
            Collection<UUID> mediaUuids = ((TermDto)result.getContent()).getMedia();
            if(mediaUuids!=null){
                if(imageBuffer==null){
                    imageBuffer = new HashSet<>();
                }
                new Job("Load term media") {
                    @Override
                    protected IStatus run(IProgressMonitor monitor) {
                        Collection<URI> mediaUris = new HashSet<>();
                        for (UUID uuid : mediaUuids) {
                            if (TermSearchResultComposite.this.isDisposed()){
                                this.cancel();
                            }
                            Media media = CdmStore.getService(IMediaService.class).load(uuid);
                            if(media==null){
                                continue;
                            }
                            URI uri = MediaUtils.getFirstMediaRepresentationPart(media).getUri();
                            mediaUris.add(uri);
                        }
                        TermSearchResultComposite.this.getDisplay().asyncExec(()->
                        {
                            for (URI uri : mediaUris) {
                                try {
                                    InputStream imageStream = UriUtils.getInputStream(uri);
                                    imageBuffer.add(new Image(Display.getCurrent(), imageStream));
                                } catch (Exception e1) {
                                    // ignore
                                }
                            }
                            try{
                                if (!content.isDisposed()){
                                    Composite mediaContainer = new Composite(content, SWT.NONE);
                                    mediaContainer.setLayout(new FillLayout());
                                    for (Image image : imageBuffer) {
                                        Composite composite = new Composite(mediaContainer, SWT.NONE);
                                        composite.addPaintListener(new PaintListener() {
                                            @Override
                                            public void paintControl(PaintEvent e) {
                                                if(image!=null){
                                                    e.gc.setAntialias(SWT.ON);
                                                    e.gc.setInterpolation(SWT.HIGH);
                                                    e.gc.drawImage(image, 0, 0,image.getBounds().width, image.getBounds().height,0, 0, 40, 40);
                                                    e.gc.dispose();
                                                }
                                            }
                                        });
                                    }
                                }
                            }catch(IllegalArgumentException e){
                                System.err.println(e.getStackTrace());
                            }
                            if (!TermSearchResultComposite.this.isDisposed() && !TermSearchResultComposite.this.getParent().isDisposed()){
                                TermSearchResultComposite.this.getParent().layout();
                            }
                        });
                        return Status.OK_STATUS;
                    }
                }.schedule();
            }
        }
        return content;
    }




}
