/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.specimen;

import java.util.Calendar;
import java.util.GregorianCalendar;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;

/**
 * Controller class for handling a {@link SpecimenSearchComposite}.
 * @author pplitzner
 * @date 03.09.2013
 *
 */
public class SpecimenSearchController implements Listener{

    private SpecimenSearchComposite specimenSearchComposite;
    private OccurenceQuery lastQuery = null;

    private static SpecimenSearchController instance = null;

    public static SpecimenSearchController getInstance(Composite parent){
        if(instance==null){
            instance = new SpecimenSearchController(parent);
            return instance;
        }
        instance.init(parent);
        return instance;
    }

    /**
     * Constructs a new controller which will itself construct the composite
     * @param parent the parent {@link Composite} for the one handles by this controller
     */
    private SpecimenSearchController(Composite parent) {
        init(parent);
    }

    private void init(Composite parent){
        this.specimenSearchComposite = new SpecimenSearchComposite(parent, SWT.NONE);
        this.specimenSearchComposite.addListener(SWT.Selection, this);
        specimenSearchComposite.getBtnShowDate().addListener(SWT.Selection, this);
        specimenSearchComposite.getBtnShowDate().setSelection(false);
        specimenSearchComposite.getDateFrom().setEnabled(false);
        specimenSearchComposite.getDateTo().setEnabled(false);

        loadLastState();
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
     */
    @Override
    public void handleEvent(Event event) {
        Button btnShowDate = specimenSearchComposite.getBtnShowDate();
        if(event.widget==btnShowDate){
            specimenSearchComposite.getDateFrom().setEnabled(btnShowDate.getSelection());
            specimenSearchComposite.getDateTo().setEnabled(btnShowDate.getSelection());
        }
    }


    private void loadLastState(){
        if(lastQuery!=null){
            specimenSearchComposite.getTextAccessionNumber().setText(lastQuery.accessionNumber);
            specimenSearchComposite.getTextCollector().setText(lastQuery.collector);
            specimenSearchComposite.getTextCollectorNumber().setText(lastQuery.collectorsNumber);
            specimenSearchComposite.getTextCountry().setText(lastQuery.country);
            specimenSearchComposite.getTextHerbarium().setText(lastQuery.herbarium);
            specimenSearchComposite.getTextLocality().setText(lastQuery.locality);
            specimenSearchComposite.getTextTaxonName().setText(lastQuery.taxonName);
            specimenSearchComposite.getTextHigherTaxon().setText(lastQuery.higherTaxon);
            if(lastQuery.dateFrom!=null){
                specimenSearchComposite.getDateFrom().setDate(lastQuery.dateFrom.get(Calendar.YEAR), lastQuery.dateFrom.get(Calendar.MONTH), lastQuery.dateFrom.get(Calendar.DAY_OF_MONTH));
            }
            if(lastQuery.dateTo!=null){
                specimenSearchComposite.getDateTo().setDate(lastQuery.dateTo.get(Calendar.YEAR), lastQuery.dateTo.get(Calendar.MONTH), lastQuery.dateTo.get(Calendar.DAY_OF_MONTH));
            }
        }
    }

    public void saveLastSate() {
        lastQuery = new OccurenceQuery(getTaxonName(), getCollector(), getCollectorNumber(), getAccessionNumber(), getHerbarium(), getCountry(false), getLocality(), getDateFrom(), getDateTo(), false);
        lastQuery.higherTaxon =getHigherTaxon();
    }

    /**
     * Returns the {@link Composite} handled by this controller
     * @return
     */
    public Composite getComposite() {
        return specimenSearchComposite;
    }

    /**
     * Returns the taxon name entered in the search view
     * @return the taxon name as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextTaxonName()
     */
    public String getTaxonName() {
        return specimenSearchComposite.getTextTaxonName().getText();
    }

    /**
     * Returns the higher taxon entered in the search view
     * @return the higher taxon as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextHigherTaxon()
     */
    public String getHigherTaxon() {
        return specimenSearchComposite.getTextHigherTaxon().getText();
    }

    /**
     * Returns the collector entered in the search view
     * @return the collector as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextCollector()
     */
    public String getCollector() {
        return specimenSearchComposite.getTextCollector().getText();
    }

    /**
     * Returns the collecting number entered in the search view
     * @return the collecting as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextCollectorNumber()
     */
    public String getCollectorNumber() {
        return specimenSearchComposite.getTextCollectorNumber().getText();
    }

    /**
     * Returns the accession number entered in the search view
     * @return the accession number as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextAccessionNumber()
     */
    public String getAccessionNumber() {
        return specimenSearchComposite.getTextAccessionNumber().getText();
    }

    /**
     * Returns the herbarium entered in the search view
     * @return the herbarium as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getComboHerbarium()
     */
    public String getHerbarium() {
        return specimenSearchComposite.getTextHerbarium().getText();
    }

    /**
     * Returns the country entered in the search view
     * @return the country as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getComboCountry()
     */
    public String getCountry(boolean biocase) {

        String result = specimenSearchComposite.getTextCountry().getText();
        if (result.contains("-") && biocase){
            result = result.substring(4).trim();
        }else if (result.contains("-")){
            result = result.substring(0,2).trim();
        }
        return result;

//        return specimenSearchComposite.getComboCountry().getItem(specimenSearchComposite.getComboCountry().getSelectionIndex());
    }

    /**
     * Returns the locality entered in the search view
     * @return the locality as a {@link String}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getTextLocality()
     */
    public String getLocality() {
        return specimenSearchComposite.getTextLocality().getText();
    }

    /**
     * Returns the start date entered in the search view.
     * @return the date as an instance of {@link Calendar}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getDateFrom()
     */
    public Calendar getDateFrom() {
        if(specimenSearchComposite.getBtnShowDate().getSelection()){
            Calendar dateFrom = new GregorianCalendar();
            dateFrom.clear();
            dateFrom.set(specimenSearchComposite.getDateFrom().getYear(), specimenSearchComposite.getDateFrom().getMonth(), specimenSearchComposite.getDateFrom().getDay());
            return dateFrom;
        }
        return null;
    }

    /**
     * Returns the end date entered in the search view.
     * @return the date as an instance of {@link Calendar}
     * @see eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchComposite#getDateTo()
     */
    public Calendar getDateTo() {
        if(specimenSearchComposite.getBtnShowDate().getSelection()){
            Calendar dateTo = new GregorianCalendar();
            dateTo.clear();
            dateTo.set(specimenSearchComposite.getDateTo().getYear(), specimenSearchComposite.getDateTo().getMonth(), specimenSearchComposite.getDateTo().getDay());
            return dateTo;
        }
        return null;
    }

}
